/**
  ******************************************************************************
  * @file board_config.c
  * @brief This file contains all the functions utilized to configure hardware.
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Include ------------------------------------------------------------------
#include "board_config.h"

// Global Variable to check whether the communication has to be made using SPi or UART interface
u8 SPI_or_UART_Communication;
// Global Variable to keep the last baud rate used by MCU in UART communication. 
u32 MCU_Baud_Rate;



/**  
* @brief  	This function returns if the CR95HF uses SPI or UART as external interface. Note that this function is forced to SPI by default. See the function unlock the configuration choice
* @param  	void	:	no parameter expected
* @retval		SPI_INTERFACE_SELECTED	: MCU and CR95HF communicate using SPI interface
* @retval		UART_INTERFACE_SELECTED	: MCU and CR95HF communicate using UART interface

*/
u8 Board_Get_SPI_or_UART_Communication(void)
{
	u8 status;
	BitStatus bit_status;
	 // read the SSI0 pin status 
	bit_status= GPIO_ReadInputDataBit(SSIO_TEST_GPIO_PORT, SSIO_TEST_GPIO_PIN);
	if(bit_status==RESET)// UART is selected
		status=UART_INTERFACE_SELECTED;
	
	else // SPI is selected
		status=SPI_INTERFACE_SELECTED;
		
	// Update the global variable to avoid recalling the function
	// Assume that the communication interface should not be switched during the execution
	SPI_or_UART_Communication=SPI_INTERFACE_SELECTED; // replace by status to unlock UART or SPI choice
	
	return SPI_INTERFACE_SELECTED ;	// replace by status to unlock UART or SPI choice
}


/**  
* @brief  	this function configures the pins depending on the board used and the communication type selected. 
* @param  	void	:	no parameter expected
* @retval		void	: no return value
*/
void Board_GPIO_Config(void)
{
	// For communication with CR95HF
	// See board_config.h and stm8l15x_gpio.h for further information on values
	
	
	// Configure first PB3 to be able to check SSIO and determine communication method
	GPIO_Init(SSIO_TEST_GPIO_PORT, SSIO_TEST_GPIO_PIN, GPIO_Mode_In_FL_No_IT);

	if(Board_Get_SPI_or_UART_Communication()==SPI_INTERFACE_SELECTED)
	{
		//Configure pins for SPI communication
		// Configure GPIOB for SPI pins ( MISO MOSI SCK NSS)
		//PB7
		GPIO_Init(SPI_MISO_GPIO_PORT,SPI_MISO_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		//PB6
		GPIO_Init(SPI_MOSI_GPIO_PORT,SPI_MOSI_GPIO_PIN , GPIO_Mode_Out_PP_Low_Fast );
		//PB5
		GPIO_Init(SPI_SCK_GPIO_PORT ,SPI_SCK_GPIO_PIN , GPIO_Mode_Out_PP_Low_Fast );
		//PB4
		GPIO_Init(SPI_NSS_GPIO_PORT,SPI_NSS_GPIO_PIN ,GPIO_Mode_Out_OD_HiZ_Slow );
		// Configure pins for IRQ_in IRQ_out
		//PC3
		GPIO_Init(SPI_IRQ_OUT_GPIO_PORT,SPI_IRQ_OUT_GPIO_PIN ,GPIO_Mode_Out_OD_HiZ_Slow );
		//PC2
		GPIO_Init(SPI_IRQ_IN_GPIO_PORT,SPI_IRQ_IN_GPIO_PIN ,GPIO_Mode_In_PU_No_IT );
	}
	else// Configure for UART communication
	{
		// Configure GPIOB for UARt pins (  NSS)
		//PB4
		GPIO_Init(UART_NSS_GPIO_PORT,UART_NSS_GPIO_PIN ,GPIO_Mode_Out_OD_HiZ_Slow );
		// Configure pins for IRQ_in IRQ_out
		//PC3
		GPIO_Init(UART_TX_GPIO_PORT,UART_TX_GPIO_PIN ,GPIO_Mode_Out_PP_High_Fast );
		//PC2
		GPIO_Init(UART_RX_GPIO_PORT,UART_RX_GPIO_PIN ,GPIO_Mode_In_PU_No_IT );
	}
	
	#ifdef BOARD_SELECTED_EVALBOARD
		
		// For communication with LCD
		GPIO_Init(LCD_MISO_GPIO_PORT,LCD_MISO_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		GPIO_Init(LCD_MOSI_GPIO_PORT,LCD_MOSI_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		GPIO_Init(LCD_SCK_GPIO_PORT,LCD_SCK_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		GPIO_Init(LCD_NSS_GPIO_PORT,LCD_NSS_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		
		// For the LED on the board 	
		GPIO_Init(LED_1_GPIO_PORT,LED_1_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		GPIO_Init(LED_2_GPIO_PORT,LED_2_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		GPIO_Init(LED_3_GPIO_PORT,LED_3_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		GPIO_Init(LED_4_GPIO_PORT,LED_4_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );

		// configure joystick
		GPIO_Init(JOYSTICK_GPIO_PORT,JOYSTICK_UP_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		GPIO_Init(JOYSTICK_GPIO_PORT,JOYSTICK_DOWN_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		GPIO_Init(JOYSTICK_GPIO_PORT,JOYSTICK_LEFT_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		GPIO_Init(JOYSTICK_GPIO_PORT,JOYSTICK_RIGHT_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		GPIO_Init(JOYSTICK_GPIO_PORT,JOYSTICK_SEL_GPIO_PIN ,GPIO_Mode_In_FL_No_IT );
		
		//Configure I2C pins 
		// Configure sEE_I2C pins: SCL 
		GPIO_Init(sEE_I2C_SCL_GPIO_PORT, sEE_I2C_SCL_PIN, GPIO_Mode_In_PU_No_IT);
		// Configure sEE_I2C pins: SDA 
		GPIO_Init(sEE_I2C_SDA_GPIO_PORT, sEE_I2C_SDA_PIN, GPIO_Mode_In_PU_No_IT);
	
	#else // Discovery board
		
		// Set output for blue and green LED on discovery Board
		GPIO_Init(D_LED_3_GPIO_PORT,D_LED_3_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		GPIO_Init(D_LED_4_GPIO_PORT,D_LED_4_GPIO_PIN ,GPIO_Mode_Out_PP_Low_Slow );
		
	#endif
}

/**  
* @brief  	this function configures and enables the SPI communication with the LCD screen
* @param  	void	:	no parameter expected
* @retval		void	: no return value
*/
void Board_SPI_LCD_init(void)
{
	// Enable SPI2 clk
	CLK_PeripheralClockConfig(CLK_Peripheral_SPI2, ENABLE);
	//Configure SPI
	SPI_DeInit(SPI2);
	SPI_Init(SPI2, SPI_FirstBit_MSB,
              SPI_BaudRatePrescaler_64,SPI_Mode_Master, 
							SPI_CPOL_High,SPI_CPHA_2Edge, 
							SPI_Direction_2Lines_FullDuplex,
              SPI_NSS_Hard, SPI_CRC_TX);
							
/* SPI Control Registers-------------------------
//SPI1_CONTROL_REGISTER_1 0x2F
// Bit7 : MSB first
// Bit6 : SPI Disable
// Bit5-3 : Freq= Freq syst/64
// Bit2 : Master
// Bit1 : SCK to 1 when idle(CPOL=1)
// Bit0 : Second clock transition = first data (CPHA=1)

SPI1_CONTROL_REGISTER_2 0x03
// Bit7 : 2-line unidirectional data mode
// Bit6 : 0 input enabled (bit7 is not set,thus this bit has no effect) 
// Bit5 : CRC calculation disabled
// Bit4 : CRC next tansmit value is from TX buffer (no effect CRC disabled)
// Bit3 : reserved
// Bit2 : Full Duplex
// Bit1 : Software slave management enabled
// Bit0 : Master mode	*/

	//Enable SPI
	SPI_Cmd(SPI2, ENABLE);
}


/**  
* @brief  	this function configures and enables the SPI communication with the CR95HF
* @param  	void	:	no parameter expected
* @retval		void	: no return value
*/	
void Board_SPI_CR95HF_init(void)
{
	//Enable SPI1 clk
	CLK_PeripheralClockConfig(CLK_Peripheral_SPI1, ENABLE);
	//Configure SPI
	SPI_DeInit(SPI1);
	SPI_Init(SPI1, SPI_FirstBit_MSB,
              SPI_BaudRatePrescaler_64,SPI_Mode_Master, 
							SPI_CPOL_High,SPI_CPHA_2Edge, 
							SPI_Direction_2Lines_FullDuplex,
              SPI_NSS_Soft, SPI_CRC_TX);
/* SPI Control Registers-------------------------
SPI1_CONTROL_REGISTER_1 0x2F
// Bit7 : MSB first
// Bit6 : SPI Disable
// Bit5-3 : Freq= Freq syst/64
// Bit2 : Master
// Bit1 : SCK to 1 when idle(CPOL=1)
// Bit0 : Second clock transition = first data (CPHA=1)

SPI1_CONTROL_REGISTER_2 0x03
// Bit7 : 2-line unidirectional data mode
// Bit6 : 0 input enabled (bit7 is not set,thus this bit has no effect) 
// Bit5 : CRC calculation disabled
// Bit4 : CRC next tansmit value is from TX buffer (no effect CRC disabled)
// Bit3 : reserved
// Bit2 : Full Duplex
// Bit1 : Software slave management enabled
// Bit0 : Master mode	*/
	//Enable SPI
	SPI_Cmd(SPI1, ENABLE);
}


/**  
* @brief  	this function configures and enables the UART communication with the CR95HF
* @param  	void	:	no parameter expected
* @retval		void	: no return value
*/	
void Board_UART_CR95HF_init(void)
{
	CLK_PeripheralClockConfig(CLK_Peripheral_USART1, ENABLE);
	// Initialize UART according to constants provided by the stm8l15x_usart.h file
	// Use USART1 at MCU_Baud_Rate with 8 bits by word and 1 bit stop without parity check( CRC will find out any error and enable both transmitter and receiver
	USART_DeInit(USART1); // De-initialize first to disable all interrupt and modes
	MCU_Baud_Rate=MCU_DEFAULT_BAUD_RATE; // Baud rate is CR95HF default baud rate value to initiate communication
	USART_Init( USART1, MCU_Baud_Rate, USART_WordLength_8b, USART_StopBits_2,
                USART_Parity_No,  (USART_CR2_TEN | USART_CR2_REN));
	USART_Cmd(USART1, ENABLE);
}


/**  
* @brief  	this function unlocks the eeprom part within the flash memory of the microcontroller in order write some data ( for command using "heavy" amount of data)
* @param  	void	:	no parameter expected
* @retval		void : no return value
*/	
void Board_MCU_EEPROM_Unlock_for_data_storage(void)
{
	FLASH_Unlock(FLASH_MemType_Data);
	delay_ms(1);
}

#ifdef USE_EXTERN_EEPROM_FUNCTION

/**
  * @brief  DeInitializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void Board_sEE_LowLevel_DeInit(void)
{
	
  // sEE_I2C Peripheral Disable 
  I2C_Cmd(sEE_I2C, DISABLE);

  // sEE_I2C DeInit 
  I2C_DeInit(sEE_I2C);

  // !< sEE_I2C Periph clock disable 
  CLK_PeripheralClockConfig(sEE_I2C_CLK, DISABLE);

  // Disable and Deinitialize the DMA channels 
  DMA_Cmd(sEE_I2C_DMA_CHANNEL_TX, DISABLE);
  DMA_Cmd(sEE_I2C_DMA_CHANNEL_RX, DISABLE);
  DMA_DeInit(sEE_I2C_DMA_CHANNEL_TX);
  DMA_DeInit(sEE_I2C_DMA_CHANNEL_RX);
}


/**
  * @brief  Initializes peripherals used by the I2C EEPROM driver.
  * @param  None
  * @retval None
  */
void Board_sEE_LowLevel_Init(void)
{
  // !< sEE_I2C Periph clock enable 
  CLK_PeripheralClockConfig(sEE_I2C_CLK, ENABLE);
  // !< Enable the DMA clock 
  CLK_PeripheralClockConfig(CLK_Peripheral_DMA1, ENABLE);

  // I2C TX DMA Channel configuration 
  DMA_DeInit(sEE_I2C_DMA_CHANNEL_TX);
  DMA_Init(sEE_I2C_DMA_CHANNEL_TX,
           0, /* This parameter will be configured during communication */
           sEE_I2C_DR_Address,
           0xFF, /* This parameter will be configured during communication */
           DMA_DIR_PeripheralToMemory,/* This parameter will be configured during communication */
           DMA_Mode_Normal,
           DMA_MemoryIncMode_Inc,
           DMA_Priority_VeryHigh,
           DMA_MemoryDataSize_Byte);

  // I2C RX DMA Channel configuration 
  DMA_DeInit(sEE_I2C_DMA_CHANNEL_RX);
  DMA_Init(sEE_I2C_DMA_CHANNEL_RX, 0, /* This parameter will be configured during communication */
           sEE_I2C_DR_Address,
           0xFF, /* This parameter will be configured during communication */
           DMA_DIR_PeripheralToMemory,/* This parameter will be configured during communication */
           DMA_Mode_Normal,
           DMA_MemoryIncMode_Inc,
           DMA_Priority_VeryHigh,
           DMA_MemoryDataSize_Byte);

}
#endif

/**
  * @brief  this function configures RTC peripheral, in order to wake up the STM8L (user_application)
  * @param  void : no parameter expected
  * @retval void : no return value
  */
void RTC_Config(void)
{
	
  // Configures the RTC 
  CLK_RTCClockConfig(CLK_RTCCLKSource_LSE, CLK_RTCCLKDiv_1);
  CLK_PeripheralClockConfig(CLK_Peripheral_RTC, ENABLE);
  
  RTC_WakeUpClockConfig(RTC_WakeUpClock_CK_SPRE_16bits);
  RTC_ITConfig(RTC_IT_WUT, ENABLE);
	
}


/**  
* @brief  	this function initializes the board by calling all initialization functions
* @param  	void	:	no parameter expected
* @retval		void : no return value
*/	
 void Board_Init_Board(void)
{
	// Set clock source  and enable it 
	
#ifdef BOARD_SELECTED_EVALBOARD // use High Speed external oscillator
	CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
	CLK_SYSCLKSourceConfig(CLK_SYSCLKSource_HSE);
	CLK_HSEConfig(CLK_HSE_ON);
	SYSCFG_REMAPPinConfig(REMAP_Pin_SPI2Full, ENABLE);// remap SPI2 to PIx
	#ifdef USE_RTC_CLOCK_FUNCTION
		RTC_Config(); // configure the RTC peripheral for low power mode
	#endif
#else // discovery board use High Speed Internal coscillator
	CLK_SYSCLKDivConfig(CLK_SYSCLKDiv_1);
	CLK_HSICmd(ENABLE);
	CLK_SYSCLKSourceConfig(CLK_SYSCLKSource_HSI);
	
#endif

	

	// Init Peripherals
	Board_GPIO_Config();
	
#ifdef BOARD_SELECTED_EVALBOARD
		Board_SPI_LCD_init();
		LCD_Initialize();// Initialize LCD, define PF2 as LCD chip select 
#endif
	Init_Timers_Values();// Initialize the timer values
	Timing_TimeOut_Init(); // Initialyze the timer 2 
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		// Init SPI 
		Board_SPI_CR95HF_init();
		// generate an IRQ request for waking up the CR95HF
		SPI_IRQ_OUT_GPIO_PORT->ODR &= ~SPI_IRQ_OUT_GPIO_PIN; 
		delay_10us_base(WAIT_10us);
		SPI_IRQ_OUT_GPIO_PORT->ODR |= SPI_IRQ_OUT_GPIO_PIN;
	}
	else // Init UART
	{
		Board_UART_CR95HF_init();
	}
	// Unlock eeprom for some data storage
	Board_MCU_EEPROM_Unlock_for_data_storage();
	
	// Note that I2C and DMA are initialized only in calling fucntion in order to avoid useless consumption
	
}


/**  
* @brief  	this function initializes the display on the screen by drawing ST logotype and writing name of the product
* @param  	void	:	no parameter expected
* @retval		void	: no return value
*/	
void Board_Init_Display(void)
{
	LCD_Clear();
	LCD_DisplayLogo(LCD_LOGOTYPE_LINE1_LEFT_CORNER);
	LCD_Print("  CR95HF  ");
}



