/**
  **********************************************************
  * @file cr95hf_driver.c
  * @brief This file contains the functions involving hardware settings like pins number to send, receive data or orders to the cr95HF
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 20 09 2011
  ******************************************************************************
  *
  */
// Includes ----------------------------------------------
#include "cr95hf_driver.h"


// The global variable below is used to point out any problem in communication between the MCU and the CR95HF
// This varaible is initialized and checked in main.c and is incremented in the CR95HF_Wait_Answer function within this file
u8 communication_watchdog; 


// This variable is retrieved from the board_config files to determine the communication interface used.
extern u8 SPI_or_UART_Communication;



// Functions------------------------------------------------

/**
* @brief Enable or Disable the communication with the CR95HF using the NSS pin
* @param NewState :  the state of the CR95HF : ENABLE  DISABLE
* @retval void : no return value
  */
void CR95HF_SPIChipSelect(FunctionalState NewState)
{
  if (NewState == ENABLE)
  {
		// Chip Select pin low: CR95HF communication enabled 
		GPIO_ResetBits(SPI_NSS_GPIO_PORT, SPI_NSS_GPIO_PIN);
  }
  else
  {
		// Chip Select pin high: CR95HF communication disabled  
		GPIO_SetBits(SPI_NSS_GPIO_PORT, SPI_NSS_GPIO_PIN);
		delay_10us_base(WAIT_10us);// set a delay for hardware 
  }
}


/**
* @brief Send /receive a byte to/from CR95HF through the SPI peripheral
* @param DataToSend :  Data to be sent
* @retval Data read : Data read
  */
u8 CR95HF_SPISendReceive_Byte(u8 DataToSend)
{
	u8 DummyRx;
  // Send byte through the SPI peripheral 
	SPI_SendData(SPI1, DataToSend);
	
	while (SPI_GetFlagStatus(SPI1, SPI_FLAG_RXNE)!=SET);
	DummyRx = SPI_ReceiveData(SPI1);
	while (SPI_GetFlagStatus(SPI1, SPI_FLAG_TXE)!=SET);
	// Wait until last byte transmit complete.
	while (SPI_GetFlagStatus(SPI1, SPI_FLAG_BSY)==SET);

	return(DummyRx);
}

/**
* @brief This function sends a byte to CR95HF through the communication interface selected
* @param DataToSend :  Data to be sent
* @retval void : no return value
  */
void CR95HF_Send_Byte(u8 DataToSend)
{
	// Check the communication interface with CR95HF
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
		CR95HF_SPISendReceive_Byte(DataToSend);// Send data
	else // UART is selected
	{
		// Wait for USART Tx buffer empty 
		USART_SendData8(USART1,DataToSend);
		// Wait Transmission complete flag
		while(USART_GetFlagStatus(USART1, USART_FLAG_TC)==RESET);
		//TC and TXE flags will be reset when the next byte will be written in the Data Register.
	}
}


/**
* @brief This function reads a byte received from CR95HF through the communication interface selected
* @param void:  no requested parameter
* @retval Data read : Data read
  */
u8 CR95HF_Read_Byte(void)
{
	u8 data_read ;
	u8 remaining_time;// Delay to prevent  the communication from getting stuck in infinite loop
	// Check the communication interface with CR95HF
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
		data_read=CR95HF_SPISendReceive_Byte(CTRLBYTE_READ_DATA);
	
	else // UART is selected
	{
		Set_TimeOut(DEFAULT_DELAY_BEFORE_TIMEOUT);// Set delay before timeout
		remaining_time= Get_Remaining_Time_before_TimeOut();// get the remaining time before the end of the delay allowed for receiving data
	
			// Wait the end of the reception
			// Flag error are set after the RXNE flag
			// CRC check in upper layer will ensure that data are not corrupted
			while(USART_GetFlagStatus(USART1,USART_FLAG_RXNE)==RESET && (remaining_time!=INIT_VALUE))
			{
				remaining_time= Get_Remaining_Time_before_TimeOut();// update  remaining time
			}
			// check if loop ended because of timeout delay
			if(remaining_time==INIT_VALUE)
			{
				communication_watchdog++; // Increment number of failed communication for higher layer
				/*GPIO_ResetBits(GPIOB, 0x01);
				delay_ms(500);
				GPIO_SetBits(GPIOB, 0x01);*/
				data_read=ERROR_CHECK_VALUE; // Set a false value that do not correspond
				// Note that this value is greater than the max value of data set thus, if the byte corresponds to datalength no further data will be read
			}
			else
				data_read=USART_ReceiveData8(USART1);// Receive data 
		
		Reset_TimeOut();// disable the timer so that no interrupt can occure if the limit timout value has not been reached
	}
	/* The flag USART_FLAG_RXNE and all other flags are cleared by the sequence above :
		reading Status register and reading  data, 
		note : parity flag is not checked because parity is disabled*/
	
	return data_read;
}


/**
* @brief This function waits until : 
SPI Communication : either the polling mode receive the right answer or the IRQ mode receive an interrupt on the IRQ_in pin 
USART communication : CR95HF transmits a byte, usefull only for Idle command
* @param Timeout :  maximum time for waiting an answer before quiting.
* @retval EXIT_COMMUNICATION_TIMEOUT : Communication with CR95HF aborted because timeout lilmit was reached
* @retval EXIT_COMMUNICATION_OK : Communication with CR95HF succeed
  */
u8 CR95HF_Wait_Answer(u8 Timeout)
{
	u8 cr95hf_response=INIT_VALUE;
	u8 remaining_time;// Delay to prevent  the communication from getting stuck in infinite loop
	u8 status=EXIT_COMMUNICATION_TIMEOUT;
	
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		if(WAITING_CR95HF_RESPONSE_METHOD==WAITING_USING_IRQ)
		{
			// wait for IRQ signal
			Set_TimeOut(Timeout);// Set delay before timeout
			remaining_time= Get_Remaining_Time_before_TimeOut();// get the remaining time before the end of the delay allowed for receiving data
			
			while((GPIO_ReadInputDataBit(SPI_IRQ_IN_GPIO_PORT, SPI_IRQ_IN_GPIO_PIN)!=RESET) 
			&& (remaining_time!=INIT_VALUE))
			{
				remaining_time= Get_Remaining_Time_before_TimeOut();// update  remaining time
			}
				
			if(remaining_time!=INIT_VALUE)// loop ended owing to an interrupt on IRQ_in pin 
				status=EXIT_COMMUNICATION_OK;
			else// loop ended because the timer reached its limit value
			{
				communication_watchdog++; // Increment number of failed communication for higher layer
			}
			Reset_TimeOut();// disable the timer so that no interrupt can occure if the limit timeout value has not been reached
		}
		else
		{
			// Polling method, enable communication
			CR95HF_SPIChipSelect(ENABLE);
			Set_TimeOut(Timeout);// Set delay before timeout
			remaining_time= Get_Remaining_Time_before_TimeOut();// get the remaining time before the end of the delay allowed for receiving data
			//	wait for ready signal
			cr95hf_response=
			(CR95HF_SPISendReceive_Byte(CTRLBYTE_POLLING))>>POLLING_CR95HF_SHIFT;
			while((cr95hf_response!=POLLING_CR95HF_READY) && (remaining_time!=INIT_VALUE))
			{
				cr95hf_response=
				(CR95HF_SPISendReceive_Byte(CTRLBYTE_POLLING))>>POLLING_CR95HF_SHIFT;
				remaining_time= Get_Remaining_Time_before_TimeOut();// update remaining time
			}
			
			if(remaining_time!=INIT_VALUE)// loop ended owing to the reception of the ready message while polling
				status=EXIT_COMMUNICATION_OK;
			else// loop ended because the timer reached its limit value
				communication_watchdog++; // Increment number of failed communication for higher layer	
			
			
				Reset_TimeOut();// disable the timer so that no interrupt can occure if the limit timeout value has not been reached
		
			CR95HF_SPIChipSelect(DISABLE);;
		}
	}
	else//UART communication
	// For an idle command CR95HF answers( command success) only when it exits idle mode. Thus, application should wait the answer before resuming communication  
	{
		Set_TimeOut(Timeout);// Set delay before timeout
		remaining_time= Get_Remaining_Time_before_TimeOut();// get the remaining time before the end of the delay allowed for receiving data
		
		while(USART_GetFlagStatus(USART1,USART_FLAG_RXNE)==RESET && (remaining_time!=INIT_VALUE))
		{
			remaining_time= Get_Remaining_Time_before_TimeOut();// update remaining time
		}
		
		if(remaining_time!=INIT_VALUE)// loop ended owing to an interrupt on IRQ_in pin 
			status=EXIT_COMMUNICATION_OK;
		else// loop ended because the timer reached its limit value
		communication_watchdog++; // Increment number of failed communication for higher layer
		
		Reset_TimeOut();// disable the timer so that no interrupt can occure if the limit timeout value has not been reached
	}
	
	return status;
}


/**
* @brief This function operates a software reset of the CR95HF via SPI connection.
* @param void :  no parameter expected
* @retval void : no return value 
  */
 void CR95HF_SPIReset (void)
{
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		CR95HF_SPIChipSelect(ENABLE);
		CR95HF_SPISendReceive_Byte(RESET_COMMAND);
		CR95HF_SPIChipSelect(DISABLE);
		delay_10us_base(WAIT_6ms);// Wait for a while to let the CR95HF returns to init state
		CR95HF_IRQ_in_Interrupt(); // Send an IRQ to the CR95HF
		delay_ms(WAIT_6ms);
	}
}


/**
* @brief This function sends an negative pulse on the IRQ_in pin of the CR95HF. 
* @param void :  no parameter expected
* @retval void : no return value 
  */
void CR95HF_IRQ_in_Interrupt(void)
{
	GPIO_ResetBits(SPI_IRQ_OUT_GPIO_PORT, SPI_IRQ_OUT_GPIO_PIN);
	delay_10us_base(WAIT_100us);
	GPIO_SetBits(SPI_IRQ_OUT_GPIO_PORT, SPI_IRQ_OUT_GPIO_PIN);
	delay_ms(WAIT_10ms);
}


/**
* @brief This function sends an negative pulse on the NSS pin of the CR95HF. This can be used in UART comunication to wake up the CR95HF from an Idle mode.
* @param void :  no parameter expected
* @retval void : no return value 
  */
void CR95HF_NSS_Interrupt(void)
{
	GPIO_ResetBits(UART_NSS_GPIO_PORT, UART_NSS_GPIO_PIN);
	delay_10us_base(WAIT_100us);
	GPIO_SetBits(UART_NSS_GPIO_PORT, UART_NSS_GPIO_PIN);
	delay_ms(WAIT_10ms);
}


/**
* @brief This functions checks if SPI communication is used before enabling the communication with the CR95HF and turning it into read byte mode to read data transmitted by the CR95HF to the MCU.
* @param void :  no parameter expected
* @retval void : no return value 
  */
void CR95HF_SPI_Chip_Enable_Read_Byte_Mode(void)
{
	// Enable communication
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		CR95HF_SPIChipSelect(ENABLE);// Chip select for SPI commuication
		CR95HF_Read_Byte();// Dummy : in SPI communication the first byte sent is to set the CR95HF in reading mode so the receive byte does not contain any useful information
	}
}

/**
* @brief This functions checks if SPI communication is used before enabling the communication with the CR95HF and turning it into send byte mode to send data to the CR95HF .
* @param void :  no parameter expected
* @retval void : no return value 
  */
void CR95HF_SPI_Chip_Enable_Send_Byte_Mode(void)
{
	// Enable communication
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		CR95HF_SPIChipSelect(ENABLE);// Chip select for SPI commuication
		CR95HF_Send_Byte(CTRLBYTE_SEND);// Send the control byte to the CR95HF
	}	
}


/**
* @brief This functions checks if SPI communication is used before disabling the communication with the CR95HF.
* @param void :  no parameter expected
* @retval void : no return value 
  */
void CR95HF_SPI_Chip_Disable(void)
{
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
		CR95HF_SPIChipSelect(DISABLE);// Disable SPI communication
}




