/**
  **********************************************************
  * @file m24lrxx_command.c
  * @brief This file contains all the functions in order to send commands to a contactless tag of the M24LRXX family performing iso15693 standards. Note that command dealing with multiple blocks (write read get status) utilize internal eeprom of the MCU whereas other commands use flash program memory.
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Includes ----------------------------------------------
#include "m24lrxx_command.h"

// this structure is retrieved so that the functions could access the protocol settings used by the CR95HF (used to test configuration :request flags and respect312s delay or wait SOF parameter,  for Lock functions for instance and to check data rate for fast commands.
extern CR95HF_Protocol_Config Protocol_Settings;/*!< Protocol and parameters related used*/

// Those two structure are retrieved from the iso15693_command.c file to share the structures in order to save memory space for code
extern CR95HF_CommandSending Cmd;/*!< Common Stucture used  to transmit data within commands*/

extern CR95HF_ResponseReceiving Response;/*!< Common Stucture used  to receive data within commands*/


// Main Architecture of the doxyfile for this file 
 // @addtogroup  Contactless_Tags
	
	// * @defgroup  M24LRXX_Tags
	
			// * @defgroup  M24LRXX_Commands


	
			// * @defgroup  M24LRXX_Support_Functions
	
			// * @defgroup M24LRXX_Energy_Harvesting
	

 // @addtogroup M24LRXX_Commands
/**  
* @brief  	This function sends, through the CR95HF, to a contactless tag posssibly of the M24LRXX family, an inventory command and fills the Tag structure if some contactless tag answers.
* @param  	MyTag		:	pointer on the structure
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here inventory flag should be set, if AFI is present, AFI flag should be set. 
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		M24LRXX_ERROR_CODE		: the command failed. Either there is no contactless  tag or the tag did not manage to perform the request.
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : The inventory flag is not set, the mask size overweight the limited size or there is an other eoor into the request flags parameters
* @retval  LM24LRXX_SUCCESS_CODE : A contactless tag answered the command.
*/
u8 M24LRXX_Inventory(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, 
const u8 MaskLength,const u8* Mask,  u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flag to 0
	request_flags &= ~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_FLAG;
	// Force the inventory flag to 1
	request_flags |= ISO15693_BIT3_INVENTORY_FLAG_SET;
	// Call the ISO15693 command
	return   ISO15693_Inventory(MyTag, request_flags,  Tag_AFI, 
 MaskLength, Mask, Inventory_16_slots_Nb_UID_Retrieved);
}



/**  
* @brief  	This function sends, through the CR95HF, a stay quiet command to a unique contactless tag possibly belonging to the M24LRXX family designated by its UID.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to put in quiet mode.
* @retval		M24LRXX_ERROR_CODE		: the command failed. Either there is no contactless tag or the tag did not manage to perform the request.
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : The inventory flag is set or request is not well addressesed
* @retval  M24LRXX_SUCCESS_CODE : contactless tag is supposed to be in quiet mode.
*/
u8 M24LRXX_Stay_Quiet(const u8 Request_flags,const  u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_SELECT_FLAG;
	// Force the address flag to 1
	request_flags |= ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED;
	// Call the ISO15693 command
	return   ISO15693_Stay_Quiet(request_flags, Tag_UID);
}



/**  
* @brief  	This function sends, through the CR95HF, a read single block command to the contactless tag possibly of the M24LRX family in the field or to an unique contactless tag designated by its UID. Note that for more than 8kbits device, the protocol extension should be set within the request flags as the block number is on two bytes. 
* @param  	MyTag		:	pointer of the structure to write data read
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Block_number	:	address of the block to read.
* @retval		M24LRXX_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered the command.
*/
u8 M24LRXX_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 Block_number)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag to 0
	request_flags &= ~ISO15693_BIT3_INVENTORY_FLAG_SET;
	
	//Check the request flags
	if(M24LRXX_Is_Request_Flags_ok(request_flags,ISO15693_CMD_READ_SINGLE_BLOCK)==FALSE_CODE)
	{
		// See the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,ISO15693_CMD_READ_SINGLE_BLOCK);
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number 
		// Check if the protocol extension flag is set to determine if the block number is on two bytes
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// split it into two bytes with LSB first
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Block_number)+ 1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Block_number) );
		}
		else // block number is on one byte (less than 8kbits device)
			Cmd.data[cptrdatalength++]=(u8)Block_number;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Single_Block_cmd( MyTag,&Response, status_cr95hf, Request_flags );
	
	}
	return status;
	
}


/**  
* @brief  	This function sends, through the CR95HF, a write single block command to the contactless tag opssibly from the M24LRXX family in the field or to an unique contactless tag designated by its UID.Note that for more than 8kbits device, the protocol extension should be set within the request flags as the block number is on two bytes. 
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Block_number	:	address of the block to write.
* @param  	Data	:	data to write.
* @retval		M14LRXX_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct)
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Write_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 Block_number, const u8* Data)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 cptrdatalength=INIT_VALUE; // counter fo command length
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag to 0
	request_flags &= ~ISO15693_BIT3_INVENTORY_FLAG_SET;
	
	//Check the request flags
	if(M24LRXX_Is_Request_Flags_ok(request_flags, ISO15693_CMD_WRITE_SINGLE_BLOCK)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,ISO15693_CMD_WRITE_SINGLE_BLOCK);
	
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add the block number
		// Check if the protocol extension flag is set to determine if the block number is on two bytes
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// split it into two bytes with LSB first
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Block_number)+1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Block_number));
		}
		else // block number is on one byte (less than 8kbits device)
			Cmd.data[cptrdatalength++]=(u8)Block_number;
		// Add block data
		CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,M24LRXX_DEFAULT_BLOCK_SIZE, Data);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a read multiple block command to the contactless tag possibly belonging to the M24LRXX family  in the field or to an unique contactless tag designated by its UID. Note that data received are stored into the internal EEPROM of the MCU.
* @param  	MyTag		:	pointer of the structure to write the command status and eventual error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one. Limited to 32 blocks for device with more than 8kibts memory assuming that the blocks are located in the same sector, an error is returned if it is not the case 
* @retval		M24LRXX_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters   
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered the command.
*/
u8 M24LRXX_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 First_Block_number, const u8 Number_of_Blocks)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag to 0
	request_flags &= ~ISO15693_BIT3_INVENTORY_FLAG_SET;
	//Check the request flags
	if(M24LRXX_Is_Request_Flags_ok(request_flags,ISO15693_CMD_READ_MULTIPLE_BLOCKS)==FALSE_CODE || Number_of_Blocks>ISO15693_MAX_SIZE_BLOCK_READ_CMD)
	{
		// if the inventory flag is  set the command is not sent. This flag involving different meaning for other bits, it could not be forced to 1 by the function.
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters and add the send receive code
		Cmd.cmd_code=CR95HF_CMD_SEND_REC;
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,ISO15693_CMD_READ_MULTIPLE_BLOCKS);
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add first block number
		// Check if the protocol extension flag is set to determine if the block number is on two bytes
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// split it into two bytes
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&First_Block_number)+ 1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&First_Block_number) );
		}
		else // block number is on one byte (less than 8kbits device)
			Cmd.data[cptrdatalength++]=(u8)First_Block_number;
		
		// Add number of blocks to read after the first one
		Cmd.data[cptrdatalength++]=Number_of_Blocks;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command 
		CR95HF_SendCommand(&Cmd);
		// Receive the answer
		status_cr95hf=ISO15693_Receive_Multiple_Blocks_cmd(&Response);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd( MyTag,&Response, status_cr95hf,Number_of_Blocks );
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a select command to the unique contactless tag designated by its UID possibly belonging to the M24LRXX family.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		M24LRXX_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Select(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_SELECT_FLAG;
	// Force the adressed flag
	request_flags |= ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED;
	
	// Call the ISO15693 command
	return ISO15693_Select( MyTag,request_flags,Tag_UID);
}

/**  
* @brief  	This function sends, through the CR95HF, a reset to ready state command to a contactless tag possibly belong to the M24LRXX family. This command could be used in two different case: 
- thecontactless tag is in quiet mode, it returns into ready mode (request could be addressed or not),
- the contactless tag is in select mode it returns into ready mode (select flag shall be set). Note that for this case another select command with another UID turn automatically the tag into ready mode.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		M24LRXX_ERROR_CODE		:the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Reset_to_Ready(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_FLAG;
	// Call the ISO15693 command
	return ISO15693_Reset_to_Ready(MyTag, request_flags, Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, a write AFI command to the contacless tag in the field possibly belonging to the  M24LRXX family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Tag_AFI	:	AFI of the contacless tag, refer to datasheet for further information.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Write_AFI(ISO15693_Tag* MyTag,const u8 Request_flags,const  u8* Tag_UID,const  u8 Tag_AFI)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the protocol extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	// Call the ISO15693 command
	return ISO15693_Write_AFI( MyTag, request_flags,  Tag_UID,  Tag_AFI);
}


/**  
* @brief  	This function sends, through the CR95HF, a Lock AFI command to the contacless tag in the field possibly from the M24LRXX family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Lock_AFI(ISO15693_Tag* MyTag, const u8 Request_flags, const  u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the protocol extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	// Call the ISO15693 command
	return ISO15693_Lock_AFI( MyTag, request_flags,  Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, a write DSFID command to the contacless tag in the field possibly belonging to the M24LRXX family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Tag_DSFID	:	DSFID of the contacless tag, refer to datasheet for further information.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Write_DSFID(ISO15693_Tag* MyTag,const u8 Request_flags,const  u8* Tag_UID,const  u8 Tag_DSFID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the protocol extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	// Call the ISO15693 command
	return  ISO15693_Write_DSFID(MyTag, request_flags,  Tag_UID,  Tag_DSFID);
}



/**  
* @brief  	This function sends, through the CR95HF, a Lock DSFID command to the contacless tag in the field possibly from the M24LRXX family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Lock_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the protocol extension flag to 0
	request_flags&=~ISO15693_BIT4_PROTOCOL_EXTENSION;
	// Call the ISO15693 command
	return ISO15693_Lock_DSFID(MyTag, request_flags, Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, an get System Information command to the contacless tag ,possibly from the M24LRXX family, in the field or to an unique contactless tag designated by its UID. Note that here the protocol extension flag is set.
* @param  	MyTag		:	pointer on the structure
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Get_System_Info(ISO15693_Tag* MyTag, const u8 Request_flags,const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option and inventory flags to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_INVENTORY_FLAG;
	// Force the  extension flag to 1
	request_flags |= ISO15693_PROTOCOL_EXTENSION;
	// Call the ISO15693 command
	return ISO15693_Get_System_Info( MyTag, request_flags, Tag_UID);
}


//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
// Custom commands  -------------------------------------------------------------------------

/**  
* @brief  	This function sends, through the CR95HF, a get multiple blocks security status command to the contactless tag in the field,possibly from the M24LRXX family, or to an unique contactless tag designated by its UID. note that data are stored into the eeprom.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one. Note that this number is limited to 32 blocks (status for 1kbits of data),furthermore the CR95HF datalength byte is on one byte, this means that frame of 255 bytes can be received.  
* @retval		M24LRXX_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered the command.
*/
u8 M24LRXX_Get_Multiple_Blocks_Security_Status(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 First_Block_number, const u8 Number_of_Blocks)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option and extension flags to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_INVENTORY_FLAG;
	// Force the  extension flag to 1
	request_flags |= ISO15693_PROTOCOL_EXTENSION;
	
	//Check the request flags
	if(M24LRXX_Is_Request_Flags_ok(request_flags,ISO15693_CMD_GET_MULTIPLE_BLOCKS_STATUS)==FALSE_CODE || Number_of_Blocks>=SIZE_1KBITS_BYTES)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters and add the send receive code
		Cmd.cmd_code=CR95HF_CMD_SEND_REC;
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,ISO15693_CMD_GET_MULTIPLE_BLOCKS_STATUS);
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// for devices with more than 8kibts memory
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// Add first block number
			// split it into two bytes with LSB first
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&First_Block_number)+ 1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&First_Block_number));
			
			// Add number of blocks to read after the first one
			// As the CR95HF length of data received is on one byte then the number of blocks concerned is on one byte too, therefore the MSB byte is filled with 0
			Cmd.data[cptrdatalength++]=Number_of_Blocks;
			Cmd.data[cptrdatalength++]=INIT_VALUE;
		}
		else // for devices with less than 8kbits memory
		{
			// Add first block number
			Cmd.data[cptrdatalength++]=(u8)First_Block_number;
			// Add number of blocks to read after the first one
			Cmd.data[cptrdatalength++]=Number_of_Blocks;
		}
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command 
		CR95HF_SendCommand(&Cmd);
		// Receive the answer
		status_cr95hf=ISO15693_Receive_Multiple_Blocks_cmd(&Response);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Get_Multiple_Block_Status_cmd( MyTag,&Response, status_cr95hf, Number_of_Blocks);
		
	
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a write sector password command to a M24LRXX contactless tag in the field or designated by its UID . The password written will be usefull to protect data contained within the contactless tag. Note that on delivery state the passwords are 00.00.00.00 h and a present password must be issued whith this value  before issuing the command and then when each time password is changed.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to write the kill code if  addressed flag is set within the request flags.
* @param  	Password_Number	:	number identifying the password (between 1 and 3)
* @param  	Password	:	password to write 
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Write_Sector_Password(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID,const u8 Password_Number ,const u8* Password)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variable to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the entesnsion and the inventory flags to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	
	
	// Check the request_flags	
	if(M24LRXX_Is_Request_Flags_ok(request_flags, M24LRXX_CMD_WRITE_SECTOR_PASSWORD)==FALSE_CODE )
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_WRITE_SECTOR_PASSWORD);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		// Add the password number
		Cmd.data[cptrdatalength++]=Password_Number;
		// Add the password provided
		CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,M24LRXX_DEFAULT_BLOCK_SIZE, Password);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}

/**  
* @brief  	This function sends, through the CR95HF, a lock sector password command to a M24LRXX contactless tag in the field or designated by its UID.  Note that for devices with more than 8kbits memory this command locks a entire sector (i.e : 32 blocks). This command can also be used to lock only one block (less than 8kbits devices.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag.
* @param  	Sector_Number	:	one of the block addresses contained into the sector. For instance, as one sector gathers 32 blocks, blocks 0 to 31 refer to sector 0, blocks 32 to 63 refer to sector 1...
* @param  	Sector_Security_Status	:	defines the read write protection and the number of the password protecting the sector. Please refer to datasheet and dedicated functions ( ) for further information and calculation of the byte
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/ 
u8 M24LRXX_Lock_Sector_Password(ISO15693_Tag* MyTag, u8 Request_flags, const u8* Tag_UID, u16 Sector_Number, u8 Sector_Security_Status)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag to 0
	request_flags&=~ISO15693_BIT3_INVENTORY_FLAG_SET;

	// Check the request_flags	 
	if(M24LRXX_Is_Request_Flags_ok(request_flags,M24LRXX_CMD_LOCK_SECTOR_PASSWORD )==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_LOCK_SECTOR_PASSWORD);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		// Add sector number
		// Check if the protocol extension flag is set to determine if the block number is on two bytes
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// split it into two bytes with LSB first
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Sector_Number)+ 1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Sector_Number) );
		}
		else // block number is on one byte (less than 8kbits device)
			Cmd.data[cptrdatalength++]=(u8)Sector_Number;
		// Add the sector security status
		Cmd.data[cptrdatalength++]=Sector_Security_Status;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		

	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a present sector password command to a M24LRXX contactless tag in the field or designated by its UID . The password provided is compared with the one within the conctactless tag. If they macth the access to all memory linked to the password is changed as set with the lock sector password command previously.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to write the kill code if  addressed flag is set within the request flags.
* @param  	Password_Number	:	number identifying the password (between 1 and 3)
* @param  	Password	:	password to compare
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Present_Sector_Password(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID,const u8 Password_Number ,const u8* Password)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory and protocol extension flags to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	
	
	// Check the request_flags	
	if(M24LRXX_Is_Request_Flags_ok(Request_flags, M24LRXX_CMD_PRESENT_SECTOR_PASSWORD)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,M24LRXX_CMD_PRESENT_SECTOR_PASSWORD);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		// Add the password number
		Cmd.data[cptrdatalength++]=Password_Number;
		// Add the password provided
		CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,M24LRXX_DEFAULT_BLOCK_SIZE, Password);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a read single block command to the M24LRXX contactless tag in the field or to an unique contactless tag designated by its UID. The particularity of this command is the datatrate used by the contactless tag to answer which is 53kbits/s. 
* @param  	MyTag		:	pointer of the structure to write data read
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Block_number	:	address of the block to read.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Fast_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u16 Block_number)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag to 0
	request_flags&=~ISO15693_BIT3_INVENTORY_FLAG_SET;

	//Check the request flags
	// the command has to issued with 52kbits/s parameters for the CR95HF protocol select parameter and high datarate for the request flags
	if(M24LRXX_Is_Request_Flags_ok(request_flags,M24LRXX_CMD_FAST_READ_SINGLE_BLOCK)==FALSE_CODE||M24LRXX_Is_Fast_Datarate_ok(request_flags,&Protocol_Settings)!= TRUE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_FAST_READ_SINGLE_BLOCK);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE;
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number 
		// Check if the protocol extension flag is set to determine if the block number is on two bytes
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// split it into two bytes with LSB first
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Block_number)+ 1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&Block_number) );
		}
		else // block number is on one byte (less than 8kbits device)
			Cmd.data[cptrdatalength++]=(u8)Block_number;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Single_Block_cmd( MyTag,&Response, status_cr95hf, Request_flags );
	
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a fast inventory command to the M24LRXX contactless tags in the field. The datarate of the response is multiplied by 2 (here 52kbits/s). Note that a fast initiate command should be issued before calling this function.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here inventory flag should be set, if AFI is present, AFI flag should be set. 
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Fast_Inventory_Initiated(ISO15693_Tag* MyTag,const  u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flagto 0
	request_flags &= ~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_FLAG;
	// Force the inventory flag to 1
	request_flags |= ISO15693_BIT3_INVENTORY_FLAG_SET;
	
	//Init Inventory_16_slots_Nb_UID_Retrieved
	*Inventory_16_slots_Nb_UID_Retrieved==INIT_VALUE;
	
	// Check the request_flags	
	if(M24LRXX_Is_Request_Flags_ok(request_flags, M24LRXX_CMD_FAST_INVENTORY_INITIATED)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_FAST_INVENTORY_INITIATED);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		if(ISO15693_Get_AFI_Flag(Request_flags)==ISO15693_AFI_PRESENT)
			Cmd.data[cptrdatalength++]=Tag_AFI;
		
		// Add mask 
		ISO15693_Add_Mask_to_command(&Cmd, &cptrdatalength,MaskLength, Mask);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		status=ISO15693_Send_Receive_Inventory_cmd(MyTag, &Response,&Cmd, Request_flags,&status_cr95hf,   Inventory_16_slots_Nb_UID_Retrieved);
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a fast initiate command to a M24LRXX contactless tag in the field or designated by its UID . On receiving the fast initiate command the M24LRXX contactless tag set the internal initiate flag ( to send a inventory initited command) and returns its UID and its DSFID, with a datarate multiplied by two (here 52kbits/s). Note that the command should not be in addressed or selected mode, the flag is reset afther a power off.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here neither the addressed or select flag should be set. 
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Fast_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;// create a buffer in order to force the option flag

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	// Check the request_flags	
	// This command has to be issued in neither adressed or select mode
	// force the request flags
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_CMD_INITIATE;
	// the datarate of the response is mulitplied by 2, check data rate 
	if((M24LRXX_Is_Request_Flags_ok(request_flags, M24LRXX_CMD_FAST_INITIATE)==FALSE_CODE)
	||(M24LRXX_Is_Fast_Datarate_ok(request_flags,&Protocol_Settings)!= TRUE_CODE))
	{
		// See the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_FAST_INITIATE);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
	status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);// Send command and receive response
		// Fill the contactless tag's structure, the response has the same format as the inventory command
			status = ISO156693_Fill_in_TagStructure_Inventory_cmd(MyTag, &Response, status_cr95hf);
			
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a read multiple block command a fast read multiple blocks command to a M24LRXX contactless tag in the field or designated by its UID . Note that data received are stored into the internal EEPROM of the MCU.
* @param  	MyTag		:	pointer of the structure to write the command status and eventual error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Fast_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag to 0
	request_flags &= ~ISO15693_BIT3_INVENTORY_FLAG_SET;

	//Check the request flags
	// high datarate must be selected
	if((M24LRXX_Is_Request_Flags_ok(request_flags,M24LRXX_CMD_FAST_READ_MULTIPLE_BLOCKS))==FALSE_CODE || (Number_of_Blocks>ISO15693_MAX_SIZE_BLOCK_READ_CMD)
	||(M24LRXX_Is_Fast_Datarate_ok(request_flags,&Protocol_Settings)!= TRUE_CODE))
	{
		// if the inventory flag is  set the command is not sent. This flag involving different meaning for other bits, it could not be forced to 1 by the function.
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters and add the send receive code
		Cmd.cmd_code=CR95HF_CMD_SEND_REC;
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_FAST_READ_MULTIPLE_BLOCKS);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE;
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add first block number 
		// Check if the protocol extension flag is set to determine if the block number is on two bytes
		if(ISO15693_Get_ProtocolExtension_Flag(Request_flags)==ISO15693_PROTOCOL_EXTENSION)
		{
			// split it into two bytes with LSB first
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&First_Block_number)+ 1);
			Cmd.data[cptrdatalength++]=*((uint8_t*)(&First_Block_number) );
		}
		else // block number is on one byte (less than 8kbits device)
			Cmd.data[cptrdatalength++]=(u8)First_Block_number;
		
		// Add number of blocks to read after the first one
		Cmd.data[cptrdatalength++]=Number_of_Blocks;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command 
		CR95HF_SendCommand(&Cmd);
		// Receive the answer
		status_cr95hf=ISO15693_Receive_Multiple_Blocks_cmd(&Response);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd( MyTag,&Response, status_cr95hf,Number_of_Blocks );
			
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, an inventory initiated command to the M24LRXX contactless tags in the field. On receiving the initiate command the M24LRXX contactless tag set the internal initiate flag ( to send a inventory initited command) and returns its UID and its DSFID. Note that a initiate command should be issued before calling this function.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here inventory flag should be set, if AFI is present, AFI flag should be set. 
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Inventory_Initiated(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	
	// Force the option flag and the protocol extension flag to 0
	request_flags &= ~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_FLAG;
	// Force the inventory flag to 1
	request_flags |= ISO15693_BIT3_INVENTORY_FLAG_SET;
	
	//Init Inventory_16_slots_Nb_UID_Retrieved
	*Inventory_16_slots_Nb_UID_Retrieved==INIT_VALUE;
	
	// Check the request_flags	
	if(M24LRXX_Is_Request_Flags_ok(Request_flags, M24LRXX_CMD_INVENTORY_INITIATED)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,M24LRXX_CMD_INVENTORY_INITIATED);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in addressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		if(ISO15693_Get_AFI_Flag(Request_flags)==ISO15693_AFI_PRESENT)
			Cmd.data[cptrdatalength++]=Tag_AFI;
		
		// Add mask 
		ISO15693_Add_Mask_to_command(&Cmd, &cptrdatalength,MaskLength, Mask);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		status=ISO15693_Send_Receive_Inventory_cmd(MyTag, &Response,&Cmd, Request_flags,&status_cr95hf,   Inventory_16_slots_Nb_UID_Retrieved);
		
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, an initiate command to a M24LRXX contactless tag in the field or designated by its UID . On receiving the initiate command the M24LRXX contactless tag set the internal initiate flag ( to send a inventory initited command) and returns its UID and its DSFID. Note that the command should not be in addressed or selected mode, the flag is reset afther a power off.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here neither the addressed or select flag should be set. 
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags)
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	// Check the request_flags	
	// This command has to be issued in neither adressed or select mode
	// the option flag and the protocol extension should be reset
	// force the request flags
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_CMD_INITIATE;
	if(M24LRXX_Is_Request_Flags_ok(request_flags, M24LRXX_CMD_INITIATE)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_INITIATE);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
	status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);// Send command and receive response
		// Fill the contactless tag's structure the response has the same format as the inventory command
			status = ISO156693_Fill_in_TagStructure_Inventory_cmd(MyTag, &Response, status_cr95hf);
			
		
	}
	return status;
}
// End of M24LRXX_commands group

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Energy Harvesting commands -----------------------------------------------
 // @addtogroup M24LRXX_Energy_Harvesting
	
	// * @defgroup M24LRXX_EH_Commands


//#ifdef USE_ENERGY_HARVESTING_COMMANDS

/**  
* @brief  	This function sends, through the CR95HF, an read Config command to a M24LRXX-E contactless tag in the field or designated by its UID . On receiving the read Config command the M24LRXX-E contactless tag reads the configuration byte and sends back its value.
* @param  	MyTag		:	pointer on the structure to write command status and the configuration byte.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_ReadCfg(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID) 
{
	u8 request_flags= Request_flags;
	u8 parameter_length=INIT_VALUE; // length of the parameters
	// the protocol extension, inventory  and the option flags  should be reset
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_FLAG;
	/* Use the custom command defined into the iso156693 layer
		The only parameter could be the UID if the command is addressed therefore if the command is addressed parameter length is UID size, else there is no parameter.
	*/ 
	if(ISO15693_Get_Address_Flag(request_flags)==ISO15693_ADDRESS_FLAG_SET)
		parameter_length=ISO15693_UID_SIZE;
	
	return  ISO15693_Custom_Commands(MyTag,request_flags, M24LRXX_CMD_READ_CFG, ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE,parameter_length , Tag_UID);
	// The configuration byte is the second byte into the data fiel,d available at byte [1] 

}


/**  
* @brief  	This function sends, through the CR95HF, an WriteEHCfg command to a M24LRXX-E contactless tag in the field or designated by its UID . On receiving the write Energy Harvesting Configuration command the M24LRXX-E contactless tag writes the data provided to the configuration byte and reports the status of the command.Note that only bits 0 to 2 are written bit 3 is ignored.
* @param  	MyTag		:	pointer on the structure to write command status and the configuration byte.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Configuration_Byte	:	the byte to write to the configuration byte.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Write_EH_Cfg(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID, u8 Configuration_Byte) 
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Force the extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	// Check the request_flags
	if(M24LRXX_Is_Request_Flags_ok(Request_flags, M24LRXX_CMD_WRITE_EH_CFG)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_WRITE_EH_CFG);
	
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, request_flags);
		
		// Add the data
		Cmd.data[cptrdatalength++]=Configuration_Byte	;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, an WriteDOCfg command to a M24LRXX-E contactless tag in the field or designated by its UID . On receiving the write Data Out Configuration command the M24LRXX-E contactless tag writes the data provided to the configuration byte and reports the status of the command. Note that only the fourth bit (bit 3) is written, bits 0 to 2 are ignored.
* @param  	MyTag		:	pointer on the structure to write command status and the configuration byte.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Configuration_Byte	:	the byte to write to the configuration byte.
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Write_DO_Cfg(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID, u8 Configuration_Byte) 
{
	u8 status=M24LRXX_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Force the extension flag to 0
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_EXTENSION_INVENTORY_FLAG;
	// Check the request_flags
	if(M24LRXX_Is_Request_Flags_ok(request_flags, M24LRXX_CMD_WRITE_DO_CFG)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,M24LRXX_CMD_WRITE_DO_CFG);
	
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, request_flags);
		
		// Add the data
		Cmd.data[cptrdatalength++]=Configuration_Byte	;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, an setRstEHen command to a M24LRXX-E contactless tag in the field or designated by its UID . On receiving the set reset Energy Harvesting Enable command the M24LRXX-E contactless tag set or reset the EHenable bit, within the volatile control register.
* @param  	MyTag		:	pointer on the structure to write command status.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Set_Reset 		:	Enable the energy harvesting (M24LRXX_CONTROL_BYTE_EH_ENABLE) or disable (M24LRXX_CONTROL_BYTE_EH_DISABLE)
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_SetRst_EH_en(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID, u8 Set_Reset) 
{
	u8 request_flags= Request_flags;
	u8 parameter_length=1; // length of the parameters at least the data
	u8 parameters [9];
	// the protocol extension, inventory and the option flags  should be reset
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_FLAG;
	/* Use the custom command defined into the iso156693 layer
		The only further parameter could be the UID if the command is adressed therefore if the command is adressed parameter length is UID size, else there is only the data parameter.
	*/
	if(ISO15693_Get_Address_Flag(request_flags)==ISO15693_ADDRESS_FLAG_SET)
	{
		memcpy(parameters,Tag_UID,ISO15693_UID_SIZE);// copy the UID into the parameters
		parameter_length+=ISO15693_UID_SIZE;
		parameters[8]=Set_Reset; // add the data to the parameters
	}
	else // command is not adressed
	{
		parameters[INIT_VALUE]=Set_Reset; // add data
	}
	
	return ISO15693_Custom_Commands(MyTag,request_flags, M24LRXX_CMD_SET_RST_EH_EN, ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE,parameter_length , Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, an Check EH En  command to a M24LRXX-E contactless tag in the field or designated by its UID . On receiving the Check Energy Harvesting enabled command the M24LRXX-E contactless tag reads the control register and sends back its value. Note that as this command is a RF command the bit 1 (field on ) is expected to be set, the main purpose of this command is detemrine if the Energy Harvesting mode is enabled.
* @param  	MyTag		:	pointer on the structure to write command status and the configuration byte.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @retval		M24LRXX_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  M24LRXX_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  M24LRXX_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 M24LRXX_Check_EH_En(ISO15693_Tag* MyTag,  const u8 Request_flags,const u8* Tag_UID) 
{
	u8 request_flags= Request_flags;
	u8 parameter_length=INIT_VALUE; // length of the parameters
	// the protocol extension and the option flags  should be reset
	request_flags&=~M24LRXX_MASK_REQUEST_FLAGS_NO_OPTION_EXTENSION_INVENTORY_FLAG;
	/* Use the custom command defined into the iso156693 layer
		The only parameter could be the UID if the command is adressed therefore if the command is adressed parameter length is UID size, else there is no parameter.
	*/
	if(ISO15693_Get_Address_Flag(request_flags)==ISO15693_ADDRESS_FLAG_SET)
		parameter_length=ISO15693_UID_SIZE;
	
	return ISO15693_Custom_Commands(MyTag,request_flags, M24LRXX_CMD_CHECK_EH_EN, ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE,parameter_length , Tag_UID);
	// The configuration byte is the second byte into the data fiel,d available at byte [1]
}

//#endif

// End of M24LRXX_Energy Harvesting commands
// End of M24LRXX_Energy Harvesting 

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
// Support function
// IS functions -----------------------------------------------------------------
	// * @addtogroup  M24LRXX_Support_Functions
	
		// * @defgroup  M24LRXX_Is_Functions
	
/**  
* @brief  	This function performs several checks to verify that the request flag is not wrong for the M24LRXX family contactless tags.  
* @param  	Request_flags	:	First byte of RF command sent to contacless tag
* @param  	M24LRXX_cmd_code	:	the code of the command to send to the contactless tag
* @retval		FALSE_CODE		: An error has been found out within the request flags
* @retval		TRUE_CODE		: There is no error seen within the request flags
*/
u8 M24LRXX_Is_Request_Flags_ok(const u8 Request_flags, const u8 M24LRXX_cmd_code)
{
	u8 status= FALSE_CODE;
	u8 test;// to store some values used for comparing value
	
	test=ISO15693_Get_DataRate_parameter_CR95HF(&Protocol_Settings);
	// Check datarate and sub carrier if the command is a fast command datarate and requestflags matching is test into each fast command
	if((test==CR95HF_DATARATE_26KBITS && ISO15693_Get_DataRate_Flag(Request_flags)==ISO15693_HIGH_DATARATE)||
		(test==CR95HF_DATARATE_6KBITS && ISO15693_Get_DataRate_Flag(Request_flags)==ISO15693_LOW_DATARATE)||
		M24LRXX_Is_Fast_Command(M24LRXX_cmd_code)==TRUE_CODE)
	{
		// datarates are matching, test subcarrier
		// note that the constants have the same value
		if(ISO15693_Get_SubCarrier_parameter_CR95HF(&Protocol_Settings)==	
			  ISO15693_Get_Subcarrier_Flag(Request_flags))
		{
		
			// to test address and select flags
			test=ISO15693_Get_Select_Flag(Request_flags)&ISO15693_Get_Address_Flag(Request_flags);
			if(test!=BIT_SET)
			{
				if(M24LRXX_Is_Memory_programming_function(M24LRXX_cmd_code)==TRUE_CODE)
				{
					// carry out further tests to forbidd one configuration
					/*If CR95HF protocol select is set with wait312s, the CR95HF sends its 
					command and if no contactless tag has answered within the 312s, the CR95HF 
					returns no Tag or timeout error, if the option flag is not set that means 
					that the contactless is supposed to answer within those delay. 
					But for memory programming command, operations could last several ms to 
					be processed so that only error (contactless tag answers immediately 
					if there is some error) can be retrieved.
					If option flag is set (write and lock command) 
					the contactless tag waits for an EOF to provide its response,
					consequently the response could be retrieved.*/
					test=ISO15693_Get_Delay_parameter_CR95HF(&Protocol_Settings);
					if( test!=CR95HF_RESPECT312_DELAY ||  
					ISO15693_Get_Option_Flag(Request_flags)!=ISO15693_OPTION_FLAG_RESET)
					{
						status=TRUE_CODE;
					}
					// else status remains at its initial value: FALSE_CODE
				}	
				else // for others command the contactless is supposed to answer as soon as it performed the command
				{
					// others commands, no further test
					status=TRUE_CODE;
				}
			} // else addressed and select flag are both set
		} // Number of sub carriers is different between the response of the contactless and the number expected by the CR95HF
	}// else data rate is different between the response of the contactless and the datarate expected by the CR95HF
	return status;
}



/**  
* @brief  	This function returns if the M24LRXX contactless tag performs a memory programming by processing the command (write, lock). Note that this functions includes the M24LRXX custom commands
* @param  	M24LRXX_cmd_code	:	command code 
* @retval		TRUE_CODE : The command performs a memory programming
* @retval		FALSE_CODE :  The command does not perform a memory programming
*/
u8 M24LRXX_Is_Memory_programming_function(const u8 M24LRXX_cmd_code)
{
	if(ISO15693_Is_Memory_programming_function( M24LRXX_cmd_code)==TRUE_CODE)
	{
			return TRUE_CODE;
	}
	else
	{
		if((M24LRXX_cmd_code==M24LRXX_CMD_WRITE_SECTOR_PASSWORD  )||
		(M24LRXX_cmd_code== M24LRXX_CMD_LOCK_SECTOR_PASSWORD )||
		(M24LRXX_cmd_code== M24LRXX_CMD_PRESENT_SECTOR_PASSWORD )||
		(M24LRXX_cmd_code== M24LRXX_CMD_WRITE_EH_CFG )||
		(M24LRXX_cmd_code== M24LRXX_CMD_WRITE_DO_CFG ))
		{
			return TRUE_CODE;
		}
		else
			return FALSE_CODE;
	}
}

/**  
* @brief  	This function returns for fast commands if the datarate selected with the protocol select function is really 52kbits/s and if the requestflags is really in high datarate ( fast command multiplies the data rate per 2  (High datarate : 26kbits/s) 
* @param  	Request_flags	:	the request flags to transmit to the contactless tag for checking the datarate
* @param  	Protocol_Settings	:	the stucture containing the information about the parameters related to the protocol in use
* @retval		TRUE_CODE : Parameters matches
* @retval		FALSE_CODE : Parameters do not macth data rates are different
*/
u8 M24LRXX_Is_Fast_Datarate_ok(const u8 Request_flags,const CR95HF_Protocol_Config* Protocol_Settings)
{
	if((ISO15693_Get_DataRate_Flag(Request_flags)!=ISO15693_HIGH_DATARATE)||
	(ISO15693_Get_DataRate_parameter_CR95HF(Protocol_Settings) !=CR95HF_DATARATE_52KBITS))
		return FALSE_CODE;
	else
		return TRUE_CODE;
}

/**  
* @brief  	This function returns if the command is a fast command.
* @param  	M24LRXX_cmd_code	:	the command code to check
* @retval		TRUE_CODE : Parameters matches
* @retval		FALSE_CODE : Parameters do not macth data rates are different
*/
u8 M24LRXX_Is_Fast_Command(const u8 M24LRXX_cmd_code)
{
	if((M24LRXX_cmd_code==M24LRXX_CMD_FAST_READ_SINGLE_BLOCK)||
	(M24LRXX_cmd_code==M24LRXX_CMD_FAST_INVENTORY_INITIATED)||
	(M24LRXX_cmd_code==M24LRXX_CMD_FAST_INITIATE)||
	(M24LRXX_cmd_code==M24LRXX_CMD_FAST_READ_MULTIPLE_BLOCKS))
	
		return TRUE_CODE ;
	else
		return FALSE_CODE;
}
// End of M24LRXX_Is functions


// GET functions ---------------------------------------------------

	// * @defgroup  M24LRXX_Get_Functions
	

/**  
* @brief  	This function returns if the block is protected by password and the number of the pasword protecting the block. 
* @param  	Byte_Status:	the byte describing the security block status (retrieved either with a read command with option flag set or with a get multiple block security status). 
* @retval		M24LRXX_SECURITY_STATUS_MASK_NO_PASSWORD : no password for this block
* @retval		M24LRXX_SECURITY_STATUS_MASK_PASSWORD_1 : password 1 protects this block
* @retval		M24LRXX_SECURITY_STATUS_MASK_PASSWORD_2 : password 2 protects this block
* @retval		M24LRXX_SECURITY_STATUS_MASK_PASSWORD_3 : password 3 protects this block
*/ 
u8 M24LRXX_Get_Block_Security_Password_number(u8 Byte_Status)
{
	return (Byte_Status & M24LRXX_SECURITY_STATUS_MASK_PASSWORD_CTRL); 
}



/**  
* @brief  	This function returns the read write protection of the block when password is presented. 
** @param  	Byte_Status:	the byte describing the security block status (retrieved either with a read command with option flag set or with a get multiple block security status). 
* @retval M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY		 : the sector is locked 
* @retval	M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_WRITE	 : the sectore can written and read
*/ 
u8 M24LRXX_Get_Block_Security_Read_Write_Protection_with_Password(u8 Byte_Status)
{
	if((Byte_Status & M24LRXX_SECURITY_STATUS_MASK_READ_WRITE_PROTECTION)!=M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY )
		return M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_WRITE; 
	
	else // there are several read write configuration but only one read only
		return M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY;
}


/**  
* @brief  	This function returns the read write protection of the block whithout  password. 
** @param 	Byte_Status :	the byte describing the security block status (retrieved either with a read command with option flag set or with a get multiple block security status). 
* @retval 	M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_ONLY	 : only read is allowed on this block
* @retval	  M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_WRITE	 : read and write are allowed
* @retval	  M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_NO_READ_NO_WRITE	 : no operation on thsi block
*/ 
u8 M24LRXX_Get_Block_Security_Read_Write_Protection_without_Password(u8 Byte_Status)
{
	// if sector is not locked then read and write are allowed
	if((Byte_Status &M24LRXX_SECURITY_STATUS_BLOCK_LOCKED)==M24LRXX_SECURITY_STATUS_BLOCK_LOCKED)
		return M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_WRITE; 
	
	// Without password if b2 is set then the configuration is no read no write
	// So the two bits are retrieved with a mask and b2 is isolated by a shift
	if(((Byte_Status & M24LRXX_SECURITY_STATUS_MASK_READ_WRITE_PROTECTION)>>SHIFT_BIT2)==BIT_SET )
		return M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_NO_READ_NO_WRITE; 
	
	else // return the value of the two bits
		return (Byte_Status & M24LRXX_SECURITY_STATUS_MASK_READ_WRITE_PROTECTION);
}
// End of M24LRXX_Get functions
// End of M24LRXX_Support_Functions

// Energy Harvesting

 // @addtogroup M24LRXX_Energy_Harvesting
	
	// * @defgroup M24LRXX_EH_Support_Functions 
	
		// * @defgroup M24LRXX_EH_Get_Functions 


//#ifdef USE_ENERGY_HARVESTING_COMMANDS
/**  
* @brief  	This function returns the range of energy harvesting using the Configuration byte retrieved with a Read Cfg command. 
** @param 	Configuration_Byte :	the byte containing the data available in the tag structure at dataField[M24LRXX_CONFIGURATION_BYTE_CURSOR]
* @retval M24LRXX_CONFIGURATION_BYTE_RANGE_6_MA : energy harvesting is actived when at least 6mA can be retrieved
* @retval M24LRXX_CONFIGURATION_BYTE_RANGE_3_MA : energy harvesting is actived when at least 3mA can be retrieved
* @retval M24LRXX_CONFIGURATION_BYTE_RANGE_1_MA : energy harvesting is actived when at least 1mA can be retrieved
* @retval M24LRXX_CONFIGURATION_BYTE_RANGE_300_MA : energy harvesting is actived when at least 300A can be retrieved
*/ 
u8 M24LRXX_Get_Energy_Harvesting_Range( u8 Configuration_Byte)
{
	return (Configuration_Byte&M24LRXX_CONFIGURATION_BYTE_MASK_RANGE);
}


/**  
* @brief  	This function returns the RF busy or Write in progress bit using the Configuration byte retrieved with a Read Cfg command. 
** @param 	Configuration_Byte :	the byte containing the data available in the tag structure at dataField[M24LRXX_CONFIGURATION_BYTE_CURSOR]
* @retval M24LRXX_CONFIGURATION_BYTE_RF_BUSY : indicate to the I2C bus when communication in RF is happening
* @retval M24LRXX_CONFIGURATION_BYTE_RF_WIP : indicate to the I2C bus when data have been changed by the RF
*/ 
u8 M24LRXX_Get_RF_BUSY_WIP(u8 Configuration_Byte)
{
	return (Configuration_Byte&M24LRXX_CONFIGURATION_BYTE_MASK_RF_WIP_BUSY);
}

/**  
* @brief  	This function returns the Energy Harvesting Mode bit using the Configuration byte retrieved with a Read Cfg command. 
** @param 	Configuration_Byte :	the byte containing the data available in the tag structure at dataField[M24LRXX_CONFIGURATION_BYTE_CURSOR]
* @retval M24LRXX_CONFIGURATION_BYTE_EH_ENABLED : the energy harvesting is enabled 
* @retval M24LRXX_CONFIGURATION_BYTE_EH_DISABLED : the energy harvesting is disabled
*/ 
u8 M24LRXX_Get_EH_mode_Configuration_Byte(u8 Configuration_Byte)
{
	return (Configuration_Byte&M24LRXX_CONFIGURATION_BYTE_MASK_EH_MODE);
}

/**  
* @brief  	This function returns the Energy Harvesting Mode bit using the Control Register retrieved with a Check EH En command. 
* @param 	Control_Register :	the byte containing the data, available in the tag structure at Tagstructurename.dataField[M24LRXX_CONTROL_REGISTER_CURSOR]
* @retval M24LRXX_CONTROL_REGISTER_EH_ENABLED : the energy harvesting is enabled 
* @retval M24LRXX_CONTROL_REGISTER_EH_DISABLED : the energy harvesting is disabled
*/ 
u8 M24LRXX_Get_EH_mode_Control_Register(u8 Control_Register)
{
	return (Control_Register&M24LRXX_CONTROL_REGISTER_CURSOR);
}
//#endif
// End of M24LRXX_Get_functions Energy Harvesting
	
// End of M24LRXX_Support_functions Energy Harvesting
	
// End of M24LRXX_Energy Harvesting


 // @addtogroup M24LRXX_Support_Functions
//------------------------------------------------------------------------------------------
/**  
* @brief  	The purpose of this function is to compute the security byte given the password number and the read write protection with and without password. 
* @param  	Password_Number: number of the password that will protect the sector (use the constants defined into the M24LRXX_command.h : M24LRXX_SECURITY_STATUS_PASSWORD_x where x designates the number.	
* @param Read_Write_Protection_Password: read write protection when password is presented use the constants defined into the M24lRXX_command.h  : either M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_WRITE	or 	  M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY	
* @param  Read_Write_Protection_No_Password : read write protection without password. Use the constants defined into the   	
 M24lRXX_command.h,  among :  M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_ONLY	 			
 M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_WRITE	  
 M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_NO_READ_NO_WRITE	
* @retval	security_byte : the security byte computed to send within the lock sector password command
*/ 
u8 M24LRXX_Compute_Security_Status_byte(u8 Password_Number, u8 Read_Write_Protection_Password, u8 Read_Write_Protection_No_Password)
{
	u8 security_byte=BIT_SET; // set b0 for block locked
	security_byte |= Password_Number; // add the number of the password that will protect the sector
	// Add the read write protection
	/* There are four cases (two bits), to indicate the read write protection with and without password.
	In each case with only one parameter ( Read_Write_Protection_Password and Read_Write_Protection_No_Password) the security byte can be determined.
	Refer to the datasheet of the contactless tag for further information*/
	// if read write protection with password is read only then whatever the parameter without password the bits are 11
	if(Read_Write_Protection_Password==M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY)
	{
		security_byte |= M24LRXX_SECURITY_STATUS_WITH_PASSWORD_READ_ONLY;
		// return the security byte as all parameters are filled in
		return security_byte; 
	}
	
	// if read write protection without password is read write then whatever the parameter with password the bits are 01
	if(Read_Write_Protection_No_Password==M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_WRITE)
	{
		security_byte |= M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_WRITE;
		// return the security byte as all parameters are filled in
		return security_byte; 
	}
	
	// if read write protection without password is read only then whatever the parameter with password the bits are 00
	if(Read_Write_Protection_No_Password==M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_ONLY)
	{
		security_byte |= M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_READ_ONLY;
		// return the security byte as all parameters are filled in
		return security_byte; 
	}
	/* At this point there is only one remaining case :
		- no read no write without password
		- read write write with password 
		As the No read no write without password and the read no write with password has already been treated.*/
	
	if(Read_Write_Protection_No_Password==M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_NO_READ_NO_WRITE)
	{
		security_byte |= M24LRXX_SECURITY_STATUS_WITHOUT_PASSWORD_NO_READ_NO_WRITE;
		// return the security byte as all parameters are filled in
		return security_byte; 
	}
	/* if an non-existing parameter is provided in parameter, 
	the byte is reset so that the command returns an error
	*/
	return INIT_VALUE;
}
// End of M24LRXX_Support_functions 


 // @addtogroup M24LRXX_Energy_Harvesting
	
	// * @addtogroup M24LRXX_EH_Support_Functions
	// * @defgroup M24LRXX_EH_Command_Parameter_Function
	
//#ifdef USE_ENERGY_HARVESTING_COMMANDS

/**  
* @brief  	The purpose of this function is to compute the configuration byte for energy harvesting mode given the parameters Energy Harvesting activation and range. 
* @param  	RF_WIP_BUSY: RF Write in Progress or RF busy either M24LRXX_CONFIGURATION_BYTE_RF_BUSY or M24LRXX_CONFIGURATION_BYTE_RF_WIP.	
* @param  	EH_Enable :  Energy Harvesting enabled or disable, either M24LRXX_CONFIGURATION_BYTE_EH_ENABLED or  M24LRXX_CONFIGURATION_BYTE_EH_DISABLED.	
* @param EH_Range : Range of power available on the Vout output : among the value  
- M24LRXX_CONFIGURATION_BYTE_RANGE_6_MA 		
- M24LRXX_CONFIGURATION_BYTE_RANGE_3_MA 			
- M24LRXX_CONFIGURATION_BYTE_RANGE_1_MA			
- M24LRXX_CONFIGURATION_BYTE_RANGE_300_UA. 	
* @retval	configuration_byte : the configuration byte  computed to send within the write_DO_Cfg and the Write_EH_Cfg commands.
*/ 
u8 M24LRXX_Compute_Configuration_Byte(u8 RF_WIP_BUSY,u8 EH_Enable,u8 EH_Range)
{
	// The constants set the bits to their location in the byte, 
	//therefore a logical OR allows to compute the byte
	return (RF_WIP_BUSY|EH_Enable|EH_Range);
}
//#endif
// End of M24LRXX_Command_Parameter_Function Energy Harvesting
	
// End of Support_Functions Energy Harvesting
	// End of  M24LRXX_Energy Harvesting

	// End of M24LRXX tags
	
	// End of Contactless tags groups


