/**
  ******************************************************************************
  * @file    delay.c
  * @author  MMY Application team
  * @version V1.0
  * @date    20 09 2011
  * @brief   delay functions for timer 2 and 4
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2010 STMicroelectronics</center></h2>
  */

// Includes ------------------------------------------------------------------


#include "delay.h"


// for TIMER 4
static u8 AutoReload_Value_1ms;//to store the final value of the counter for ms based timer
static u8 AutoReload_Value_10us;//to store the final value of the counter for 10us based timer

// for timer 2
static __IO uint32_t TimingDelay;


//-----------------------------------------------------------------------------------
// timer without interrupt
/**
* @brief  delay for some time in ms unit for frequencies between 1-16MHz. Note that for 1ms delay the function is not accurate enough so it is recommended to call delay_10us_base(100).
* @param  n_ms is how many ms of time to delay
* @retval None
  */
void delay_ms(u16 n_ms)
{	
	//--------------------------------------------!!!!!!
// Init TIMER 4 
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, ENABLE);

// Init TIMER 4 prescaler: / (2^6) = /64 
  TIM4->PSCR = PRESCALER_1ms;

// HSI div by 1 --> Auto-Reload value: 16M / 64 = 1/4M, 1/4M / 1k = 250
  TIM4->ARR = AutoReload_Value_1ms;
  
// Counter value: 2, to compensate the initialization of TIMER
  TIM4->CNTR = 2;

// clear update flag 
  TIM4->SR1 &= ~TIM4_SR1_UIF;

// Enable Counter 
  TIM4->CR1 |= TIM4_CR1_CEN;

  while(n_ms--)
  {
    while((TIM4->SR1 & TIM4_SR1_UIF) == 0) ;
    TIM4->SR1 &= ~TIM4_SR1_UIF;
  }

// Disable Counter 
  TIM4->CR1 &= ~TIM4_CR1_CEN;
	CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, DISABLE);
}

/**
* @brief  delay for some time in 10us unit(partial accurate) for frequencies between 1-16MHz. Note that the function is optimized for 16Mhz frequency. For another one counter value may be adapted.
* @param n_10us is how many 10us of time to delay
* @retval None
  */
void delay_10us_base(u16 n_10us)
{

//--------------------------------------------!!!!!!
// Init TIMER 4 
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, ENABLE);

// prescaler: / (2^0) = /1 
  TIM4->PSCR = 0;

// SYS_CLK_HSI_DIV1 Auto-Reload value: 16M / 1 = 16M, 16M / 100k = 160 
  TIM4->ARR = AutoReload_Value_10us;

// Counter value: 10, to compensate the initialization of TIMER 
	TIM4->CNTR = 40;
	

// clear update flag 
  TIM4->SR1 &= ~TIM4_SR1_UIF;

// Enable Counter 
  TIM4->CR1 |= TIM4_CR1_CEN;

  while(n_10us--)
  {
    while((TIM4->SR1 & TIM4_SR1_UIF) == 0) ;
    TIM4->SR1 &= ~TIM4_SR1_UIF;
		
  }

// Disable Counter 
  TIM4->CR1 &= ~TIM4_CR1_CEN;
 CLK_PeripheralClockConfig(CLK_Peripheral_TIM4, DISABLE);

}


/**
* @brief  This function computes all values required by the timer to avoid to compute them when calling the delay functions which will slow down and corrupt the timer basis. Note that this function must be called each time frequency is changed during the application execution.
* @param void : no parameter requested
* @retval None
  */
void Init_Timers_Values(void)
{
	u32 clock_frequency; //to get the frequency value
	u32 prescal_freq; //frequency divided by the presacler
	
	clock_frequency=CLK_GetClockFreq();// Get frequency
	
	// For 10us based timer 
	AutoReload_Value_10us=clock_frequency/BASE_10us;
	
	// For ms based timer
	prescal_freq=clock_frequency/PRESCALER_1ms_VALUE;// Divide by presacler for counter frequency
	AutoReload_Value_1ms=prescal_freq/BASE_1ms;// Divide by the value desired in Hz 

}
//-------------------------------------------------------------------------------
// Timer with Interrupt 

/**
* @brief  Timeout init:to generate 1 ms time base using TIM2 update interrupt
* @note   The external low speed clock (LSE) is used to ensure timing accuracy. This function should be updated in case of use of other clock source.      
* @param  None
* @retval None
  */
void Timing_TimeOut_Init(void)
{
	
	TIM2_DeInit();
	
  // Enable TIM2 clock 
  CLK_PeripheralClockConfig(CLK_Peripheral_TIM2, ENABLE);

  // Remap TIM2 ETR to LSE: TIM2 external trigger becomes controlled by LSE clock 
  SYSCFG_REMAPPinConfig(REMAP_Pin_TIM2TRIGLSE, ENABLE);

  // Enable LSE clock 
  CLK_LSEConfig(CLK_LSE_ON);
  // Wait for LSERDY flag to be reset 
  while ((CLK_FLAG_LSERDY) == RESET);

  /* TIM2 configuration:
     - TIM2 ETR is mapped to LSE
     - TIM2 counter is clocked by LSE div 4
      so the TIM2 counter clock used is LSE / 4 = 32.768 / 4 = 8.192 KHz
     TIM2 Channel1 output frequency = TIM2CLK / (TIM2 Prescaler * (TIM2_PERIOD + 1))
                                    = 8192 / (1 * 8) = 1024 Hz                */
                                    
  // Time Base configuration 
  TIM2_TimeBaseInit(TIM2_Prescaler_1, TIM2_CounterMode_Up, TIM2_PERIOD);
  TIM2_ETRClockMode2Config(TIM2_ExtTRGPSC_DIV4, TIM2_ExtTRGPolarity_NonInverted, 0);

  TIM2_UpdateRequestConfig(TIM2_UpdateSource_Global);

  // Clear TIM2 update flag 
  TIM2_ClearFlag(TIM2_FLAG_Update);

  // Enable update interrupt 
  TIM2_ITConfig(TIM2_IT_Update, ENABLE);

}

/**
* @brief  Set the delay allowed before timeout and start the timer 2. This function is used after having sent a command to the CR95HF in order to wait its response. if the response does not come the application should not stay in a waiting state.
* @param  nTime: specifies the delay time length, in milliseconds.
* @retval None
  */
void Set_TimeOut(__IO uint32_t nTime)
{
	TIM2_Cmd(ENABLE);// Enable timer
  TimingDelay = nTime;
}

/**
* @brief  This function disables the Timer2 so that no interrupt could be created if the timer is stopped before the end of the delay. For instance if the CR95HF answers before the end of the timeout, 6ms before for example, there would be six remaining interruption before the timer stops. 
* @param  void : no parameter expected
* @retval None
  */
void Reset_TimeOut(void)
{
  TIM2_Cmd(DISABLE);
}


/**
* @brief  This function returns the remaining time (ms) before timeout. This function is linked to the Timer 2 with interruption it should be called after a Set_Timeout 
* @param  nTime: specifies the delay time length, in milliseconds.
* @retval None
  */
u8  Get_Remaining_Time_before_TimeOut( void)
{
	return TimingDelay;
}

/**
  * @brief  Decrements the TimingDelay variable.
  * @note   This function should be called in the
  *         TIM2_UPD_OVF_TRG_BRK_USART2_TX_IRQHandler in the stm8l15x_it.c file.
  *
  *       // INTERRUPT_HANDLER(TIM2_UPD_OVF_TRG_BRK_USART2_TX_IRQHandler, 19)
  *       // {
  *           // TimingDelay_Decrement(); 
  *           // TIM2_ClearITPendingBit(TIM2_IT_Update);
  *             
  *       // }  
  * @param  None
  * @retval None
  */
void Timing_TimeOut_Decrement(void)
{
  if (TimingDelay != 0x00)
  {
    TimingDelay--;
  }
}




