/**
  ******************************************************************************
  * @file miscellaneous.c
  * @brief This file contains various functions useful for different other files
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 20 09 2011
  ******************************************************************************
  *
  */
// Includes ------------------------------------------------------------------
#include "miscellaneous.h"

extern __IO uint8_t* sEEDataWritePointer;

/**
* @brief The purpose of this function is to gather different parameters in one byte. This function is used in paramater byte computing functions for different protocols. Each paramter might be included in the right position in a byte. Note: Since the function operates a logic OR, the order of the parameter does not affect the final result.
  * @param Bit7 : the parameter to place at the MSB bit
	* @param Bit6 : the paramater of the bit number 6
	* @param Bit5 : the paramater of the bit number 5
	* @param Bit4 : the paramater of the bit number 4
	* @param Bit3 : the paramater of the bit number 3
	* @param Bit2 : the paramater of the bit number 2
	* @param Bit1 : the paramater of the bit number 1
	* @param Bit0 : the LSB bit 
  * @retval byte : the byte containing all the parameters
  */
u8 ComputeByte(const u8 Bit7,const  u8 Bit6,const  u8 Bit5,const  u8 Bit4,const  u8 Bit3, const u8 Bit2, const u8 Bit1,const  u8 Bit0)
{
	u8 byte;
	// Computing  parameters at once
	byte = (Bit7 | Bit6 | Bit5 |	Bit4 | Bit3 | Bit2 | Bit1 | Bit0 );
	
	return byte; 
}


/**
* @brief The purpose of this function is to compare the ICref with all the available values defined in include file and returns the number og byte to MemSize field (2 or 3 bytes).
* @param nb_byte : the number of Byte
* @retval byte : the number of byte 
* @retval NO_ICREF_MATCHING : No Icref matches, the parameter should not be written 
  */
u8 Compare_IcRef_Return_Memsize(const u8 ICRef)
{
	
	u8 ICRef_6MSB=ICRef&MASK_ICREF_6MSB;// retrieve only the 6 MSB bits
	
	if(ICRef_6MSB== M24LR64_R )
		return M24LR64_R_MEMSIZE_WORD_SIZE;
	
	// compare Icref	
	if(ICRef_6MSB == M24LRXX_E)
	{
		// Note that if the ICref is retrieved from the UID the information is not available
		// the eight bits are signficant the 2 LSB bits indicates the memory size
		if(ICRef==M24LR16_E)
			return M24LR16_E_MEMSIZE_WORD_SIZE;
		// add here other devices of the M24LRXX-E family	
		/*if(ICRef==M24LR_SIZE_E)
			return M24LR_SIZE_E_MEMSIZE_WORD_SIZE;*/
	}
		
	if(ICRef_6MSB == LRIS64K )
		return LRIS64K_MEMSIZE_WORD_SIZE;
		
	if(ICRef_6MSB == LRI2K )
		return LRI2K_MEMSIZE_WORD_SIZE;

	if(ICRef_6MSB == LRIS2K)
		return LRIS2K_MEMSIZE_WORD_SIZE;
	
	if(ICRef_6MSB == LRI1K)
		return LRI1K_MEMSIZE_WORD_SIZE;	
		
	
	return NO_ICREF_MATCHING;// ICref unknown
}


/**
* @brief The purpose of this function is to compare the ICref with all the available values defined in include file and returns the nulber of byte of a contactless tag memory block
* @param nb_byte : the number of Byte
* @retval byte : the number of byte of a EEPROM block
* @retval NO_ICREF_MATCHING : No Icref matches, the parameter should not be written 
  */
u8 Compare_ICRef_Return_Size_of_Block( const u8 ICRef)
{
	u8 ICRef_6MSB=(ICRef&MASK_ICREF_6MSB);// retrieve only the 6 MSB bits
	
	// compare Icref
	if(ICRef_6MSB==M24LR64_R )
		return M24LR64_R_BLOCK_SIZE;
	
	if(ICRef_6MSB==M24LRXX_E )
		return M24LRXX_E_BLOCK_SIZE;
		
	if(ICRef_6MSB==LRIS64K )
		return LRIS64K_BLOCK_SIZE;
		
	if(ICRef_6MSB==LRI2K )
		return LRI2K_BLOCK_SIZE;

	if(ICRef_6MSB==LRIS2K )
		return LRIS2K_BLOCK_SIZE;
		
	if(ICRef_6MSB== LRI1K)
		return LRI1K_BLOCK_SIZE;
		
	return NO_ICREF_MATCHING; // ICRef unknown
}

/**
  * @brief The purpose of this function is to retrieve data stored into the internal EEPROM of the MCu and to write them into the array provided in parameter
  * @param data_retrieved : pointer to store the data retrieved from eeprom
  * @param Address : adress of the first data to retrieve
  * @param Length : number of data to read 
  * @retval void: no return value
  */
void Get_Data_From_EEPROM(u8* data_retrieved ,u32 Address, u8 Length) 
{
	u8 counter=INIT_VALUE;
	while(Length!=INIT_VALUE)
	{
		data_retrieved[counter++]=FLASH_ReadByte(Address);
		Address++;
		Length--;
	}
}


/**
* @brief The purpose of this function is to write the provided data in parameter to internal MCU EEPROM.
* @param Storing_Address : adress itn the EEPROM where to write the first data
* @param Data_to_Store : pointer on the data to write into the EEPROM
* @param Length : number of data to write (in bytes) 
* @retval void: no return value
*/
void Store_Data_Into_EEPROM(u32 Storing_Address, u8* Data_to_Store, u8 Length) 
{
	u8 counter=INIT_VALUE;
	while(Length!=INIT_VALUE)
	{
		FLASH_ProgramByte(Storing_Address, Data_to_Store[counter++]);
		Storing_Address++;
		Length--;
	}
}


#ifdef BOARD_SELECTED_EVALBOARD
#ifdef USE_EXTERN_EEPROM_FUNCTION
/**
* @brief This function using the DMA transfers data from the internal MCU EEPROM to the external EEPROM through the I2C communication bus. 
* @param Dest_Address_Ext_EEPROM : destination address for data into the external EEPROM 
* @param  Src_Address_Int_EEPROM : actual address of data to be tranferred
* @param  Size_of_data : number of data to tranfer (in bytes)
* @retval void: no return value
  */
void  Transfer_data_internal_to_external_EEPROM(u16 Dest_Address_Ext_EEPROM, u16 Src_Address_Int_EEPROM, u8 Size_of_data )
{
	// Init DMA and I2C communication 
	// Initialize the I2C EEPROM driver ---------------------------------------
	sEE_Init();
		
	// Write on I2C EEPROM from EEPROM_WRITE_ADDRESS1 
	sEE_WriteBuffer(Src_Address_Int_EEPROM, Dest_Address_Ext_EEPROM, Size_of_data);
			
	// Wait for DMA interrupt or end of transmission (size_of_data is reset)
	while (*sEEDataWritePointer > 0)
	{}
	// Wait for EEPROM standby state 
	sEE_WaitEepromStandbyState();		
		
	//Free all used resources 
	sEE_DeInit();
}


/**
  * @brief This function using the DMA receive data from the external EEPROM through the I2C communication bus and stores them into the internal MCU EEPROM. 
  * @param Dest_Address_Int_EEPROM : destination address for data into the external EEPROM 
  * @param  Src_Address_Ext_EEPROM : actual address of data into the external EEPROM to be tranferred
  * @param  Size_of_data : number of data to tranfer (in bytes)
  * @retval void: no return value
  */
void  Transfer_data_external_EEPROM_to_internal_EEPROM(u16 Dest_Address_Int_EEPROM, u16 Src_Address_Ext_EEPROM, u8 Size_of_data)
{
	u16 numDataRead=Size_of_data;
	u8 buffer_read_I2C [129];
	
	// Init DMA and I2C communication 
	// Initialize the I2C EEPROM driver ---------------------------------------
	sEE_Init();
		
	// Read from I2C EEPROM from Src_Address_Ext_EEPROM 
	sEE_ReadBuffer(buffer_read_I2C, Src_Address_Ext_EEPROM, (uint16_t*)&numDataRead);
		
	/* Starting from this point, if the requested number of data is higher than 1,
	then only the DMA is managing the data transfer. Meanwhile, CPU is free to 	perform other tasks:
	For simplicity reasons, this example is just waiting till the end of the transfer. */
			
	// Wait till DMA transfer is complete (Transfer complete interrupt handler
	//resets the variable holding the number of data to be read) 
	while (numDataRead > 0)
	{}
		
	// Wait for EEPROM standby state 
	sEE_WaitEepromStandbyState();
		
	//Free all used resources 
	sEE_DeInit();
		
	/* Store data from RAM into the EEPROM. 
	As EEPROM is longer to program than RAM the DMA could fail to receive data 
	and write those into the EEPROM, so RAM is used as a buffer*/
	Store_Data_Into_EEPROM(Dest_Address_Int_EEPROM, buffer_read_I2C, Size_of_data); 
	
}
#endif
#endif

