/**
  ******************************************************************************
  * @file main.c
  * @brief This file is the entry point of the projects. It calls the hardware configuration functions and launch the customer application
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Includes ----------------------------------------------

// Board and cr95hf driver
#include "mono_lcd.h"
#include "board_led.h"
#include "cr95hf_driver.h"
#include "board_config.h"

// Libraries
#include "iso15693_command.h"
#include "lrixk_command.h"
#include "m24lrxx_command.h"

// Others
#include "user_application.h"

// Functionnalities
// In order to use the external EEPROM function, uncomment the line //#define USE_EXTERN_EEPROM_FUNCTION into the board_config.h file. The function present in the iso15693 command sends several read multiple block command and then store the data into the external EEPROM  using I2C and DMA feature.

// To access all the specific commands linked to energy harvesting products (M14LRXX-E)commands uncomment the line  :   #define USE_ENERGY_HARVESTING_COMMANDS   into the m24lrxx_command.h

// To access the User Application uncomment the line below, the user application allows unsing joystick to  
#define USE_USER_APPLICATION// Comment the line in order to cut the user application (discovery board)

//---------------------------------------------------------
// Exported variables define ------------------------------------------------------------

#define LIMIT_COMMUNICATION_FAILURE  	0x01
#define NO_TAG_ISO15693_DETECTED			0x01
#define DEFAULT_SPEED_DISPLAY_MOTION	0x00
#define MEDIUM_SPEED_DISPLAY_MOTION		0x04
#define LOW_SPEED_DISPLAY_MOTION			0x08

#define ISO15693_DISPLAY_UNHANDLED 0x01
#define	ISO15693_DISPLAY_HANDLED 0x00

// Private variables ---------------------------------------------------------
// Global variables
extern u8 communication_watchdog;// kind of watchdog to keep program from getting stuck
extern u8 SPI_or_UART_Communication;// Communication interface used by MCU to communicate with 


//-----------------------------
//Tests variables
//Global variables for ISO15693
ISO15693_Tag MyTag15693;// Contactless tag structure containing data retrieved with commands
u8 ISO15693_Request_Flags;// Request flags computed for sending commands
u8 ISO15693_Request_Flags_Inventory; // Request flags for inventory commands
u8 ISO15693_ProtocolSelect_Parameters;// Parameters used for RF protocol ISO15693
u8 ISO15693_Inventory_16slots_Nb_UID_Retrieved;// Number of tag seen during the inventory 16 slots commands

// Global variables for CR95HF
// Linked to the RF protocol used
CR95HF_Protocol_Config Protocol_Settings; // Note that this sturture is accessibe for iso15693 functions
// Linked to the Idle commands parameters and response
CR95HF_Idle_Config Idle_Config;
// To receive answer of the CR95HF
CR95HF_ResponseReceiving CR95HF_response;
// Global status of command
u8 Resp_status=ISO15693_ERROR_CODE;


// Private function prototypes -----------------------------------------------

static void TagHunting (void);

static u8 Test_Config(void);

static void Display_Motion(u8 speed);

static void Communication_test_and_recovering_procedure(void);

// Display

static void ISO15693_Display_Tag_Answer_Inventory_GetSys_Info(void) ;

static void ISO15693_Display_Tag_UID(const u8* UID, const u8 Line);

static u8 ISO15693_Display_Tag_ICREF(const u8 Tag_ICRef, const u8 Line);

static u8 ISO15693_Display_Tag_MemSize(const u8* MemSize, const u8 Number_of_Bytes_MemSize, const u8 Line);




// firmware main entry point -------------------------------------------------
void main(void) 
{
	u8 status_test_config;// status of configuration test
	//Initialization of globlal variable
	// Communication 
	communication_watchdog=INIT_VALUE;// Initialization of communication watchdog
	// Initialization of hardware
	Board_Init_Board(); // Init board configure pins
	Board_Init_Display(); // Init Display
	enableInterrupts();// Enable global interrupts
	// Tests
	status_test_config=Test_Config(); // Test communication with CR95HF and one command
	Select_Protocol_ISO15693();// Turn RF on and select the ISO15693 protocol
	
	if(status_test_config==CR95HF_SUCCESS_CODE)
	{
		// Looping
		while (1)
		{
			while (communication_watchdog<= LIMIT_COMMUNICATION_FAILURE)
			{
				delay_ms(WAIT_10ms);
			
				TagHunting ();// Look for contactless tag in the operating field
				#ifdef USE_USER_APPLICATION
				// if this function is not called the user application file will not be compiled
					RunUserApplication();
				#endif
			}
			// communication_watchdog exceeded the limit try to find out if communication is lost and recover it.
			Communication_test_and_recovering_procedure();
		
		}
	}
}


/**  
* @brief  	This function carries out some test by sending an echo command to CR95HF, an IDN command and a protocol select command.  
* @param  	void	:	no parameter expected
* @retval		CR95HF_ERROR_CODE	: Configuration tests  failed
* @retval		CR95HF_SUCCESS_CODE	: Configuration tests  passed 
*/	
static u8 Test_Config(void)
{
	u8 status=CR95HF_ERROR_CODE;// Status of the procedure
	FlagStatus flag_status; // to verify that EEPROM is writable
	u8 Dummy_parameters=0x01;// Dummy parameters only used to send a command
	
	LED_Switch_on(ALL_LED);// activate  all LED
	LED_Switch_off(ALL_LED);// deactivate all LED
	LCD_Clear();
	// display interface communication
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"##    SPI    ##");
		// display polling method
		if(WAITING_CR95HF_RESPONSE_METHOD == WAITING_USING_POLLING)
			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"##  POLLING  ##");
		else
			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"     IRQ_in");
	}
	else
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"##   UART    ##");

	Wait_Display(WAIT_500ms);
	
	flag_status=FLASH_GetFlagStatus(FLASH_FLAG_DUL);// Test if EEPROM is writable for data
	CR95HF_IRQ_in_Interrupt();// Send an interrupt in order to wake up the CR95HF
	while(CR95HF_Echo()!=CR95HF_SUCCESS_CODE)
	{
		// For the SPI functions below note that the interface communication is checked within the functions before sending any pulse. ALso the functions contain sufficient delay for being efficient  
		CR95HF_IRQ_in_Interrupt();// Send an interrupt in order to wake up the CR95HF
		if(CR95HF_Echo()!=CR95HF_SUCCESS_CODE)
		{
			CR95HF_SPIReset();// Try to reset the CR95HF 
		}
		LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"##  Echo KO  ##");
		LED_Switch_on(RED_LED); // activate red LED flag
	}
	LED_Switch_off(RED_LED);
	// if echo failed once and success after, communication_watchdog may have been incremented several times, clear it before entering the main loop 
	communication_watchdog=INIT_VALUE;
	// ask for identification
	CR95HF_IDN(&CR95HF_response);

	while(CR95HF_ProtocolSelect
	(CR95HF_PROT_SEL_ISO15693, CR95HF_PARAMETERSLENGTH_PROT_ISO15693, & Dummy_parameters, &Protocol_Settings)!=CR95HF_SUCCESS_CODE)
	{
		LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"!# CFG ERR  #!");
		LED_Switch_on(RED_LED); // activate red LED flag
	}

	LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"   CONFIG OK");
	LED_Switch_off(RED_LED);
	Wait_Display(WAIT_500ms);
	
	
	// Configure the Tag detector with a calibration
	// Note that the field must be kept clear during this procedure
	status=CR95HF_Idle_Detector_Calibration(&Protocol_Settings, &Idle_Config);
	
	
	
	if(flag_status==RESET)
	{
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"  EEPROM LOCKED");
		LED_Switch_on(RED_LED); // activate red LED flag
		Wait_Display(WAIT_1s);
	}
	else// others tests are ok at this level and EEPROM is writable
	{
		status=CR95HF_SUCCESS_CODE;
	}
	
	// If SPI communication is selected send an interrupt on IRQ out pin to CR95HF
	if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
	{
		CR95HF_IRQ_in_Interrupt();
	}
		
		return status;
}



/**  
* @brief  	This function is called when communication_watchdog overtakes the limit value. The function tries to send several echo and if SPI communication is used try to send a reset command.  
* @param  	void	:	no parameter expected
* @retval		void	: no return value
*/	
static void Communication_test_and_recovering_procedure(void)
{
	// Communications failed
	//Check if the CR95HF still can answer
	if(CR95HF_Echo()==CR95HF_SUCCESS_CODE)
	{
		communication_watchdog=INIT_VALUE;// CR95HF still can
	}
	else
	{
		// Wait and try again
		delay_10us_base(WAIT_100us);
		if(CR95HF_Echo()==CR95HF_SUCCESS_CODE)
		{
			communication_watchdog=INIT_VALUE;// CR95HF answered
		}
		else// command failed once more
		{
			// Display communication error
			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"MCU<->CR95HF ERR");
			delay_ms(WAIT_500ms);
			// Try to send a software reset
			if(SPI_or_UART_Communication==SPI_INTERFACE_SELECTED)
			{
				CR95HF_SPIReset();// Reset the CR95HF
				CR95HF_IRQ_in_Interrupt();
				delay_ms(WAIT_3ms);
			}
		}
	}
}


/**
* @brief This function displays alternatively dots on the screen to give a sign that the MCU is still on. Note that in order to save a timer the display is made with counter, therefore the timing is not regulated and may vary. Also if the inventory made in main loop has 16 slots and if the CR95HF waits for SOF (around 20ms if there is no tag), the speed of blinking slows down.
* @param speed : speed of blinking for the dots, high value : low speed, range : 0-10
* @retval	void : no return value
*/
static void Display_Motion(u8 speed)
{
	static u8 counter=INIT_VALUE; // counter for the dot to display
	static u8 buffer=INIT_VALUE; // counter for incrementing counter
	u8 character; // the character to display ) no symbolize field on, . field off
	if(Protocol_Settings.protocol_in_use==CR95HF_PROT_SEL_FIELDOFF)
		character='.';
	else
		character=')';
	// buffer acts like a divider but only using soustraction
	// check if buffer is greater than speed
	// Indeed, if a change of speed occurs, as buffer is not reset it could become greater than the speed
	if(speed<buffer)
		buffer=INIT_VALUE; // reinitialize buffer 
	// check if buffer is equal to speed 
	if((speed-buffer)==INIT_VALUE)
	{
		counter++; // increment counter to display next dot
		buffer=INIT_VALUE; // reinitialize buffer 
		// if speed = 0 at each call another dot is displayed
		// if speed =4 another dot is displayed every 4 calls ...
	}
	else
		buffer++; /// increment buffer for the next call
	
	LCD_SetCursorPos(LCD_LINE1, LCD_FIRST_DOT);
	switch( counter )
	{
		case 0x01 : 
			
			LCD_PrintChar(character);
		break;
	
		case 0x02 :
			LCD_PrintChar(character);
			LCD_PrintChar(character);
		break;
	
		case 0x03 : 
		
			LCD_SetCursorPos(LCD_LINE1, LCD_THIRD_DOT);
			LCD_PrintChar(character);
		break;
	
		default  : 
			counter=INIT_VALUE;
			LCD_PrintChar(' ');
			LCD_PrintChar(' ');
			LCD_PrintChar(' ');
		break;
	}
		
	
}


/**
  * @brief This function selects the ISO15693 protocol for CR95HF and tries to send an inventory command
  * @param void : no requested parameter
  * @retval NO_TAG_ISO15693_DETECTED: no tag in the field or 
  * @retval ISO15693_COMMAND_SUCCESS_CODE: tag detected in the field
  */
static u8 ISO15693_Detect_Tag(void)
{
	// In order to use the mask into the inventory command:
	// Modify the value below then add the length in bits of the significant data added into the command parameters
	u8 Mask [2];
	u8 Mask_length_bits=INIT_VALUE;
	// To send inventory to only one particular family of contactless tag, change the value below and in the requestflags set AFI present flag 
	u8 AFI_Tag= INIT_VALUE; 
	
	// Add the mask here
	Mask[0]=0x46;// LSB of the UID
	Mask[1]=0x0B;
	// Compute the inventory request flags
	
		ISO15693_Request_Flags_Inventory=ComputeByte(
		ISO15693_BIT3SET_BIT8_RFU,
		ISO15693_BIT3SET_BIT7_OPTIONFLAG_RESET,
		ISO15693_BIT3SET_BIT6_16_SLOTS,//ISO15693_BIT3SET_BIT6_1_SLOT
		ISO15693_BIT3SET_BIT5_AFI_NOT_PRESENT,//ISO15693_BIT3SET_BIT5_AFI_PRESENT
		ISO15693_BIT4_NO_PROTOCOL_EXTENSION,
		ISO15693_BIT3_INVENTORY_FLAG_SET,
		ISO15693_BIT2_LOW_DATARATE,//ISO15693_BIT2_HIGH_DATARATE
		ISO15693_BIT1_SINGLE_SUBCARRIER//ISO15693_BIT1_DUAL_SUBCARRIERS
		);
		
		// Send the inventory command
		return LRIXK_Inventory(&MyTag15693,ISO15693_Request_Flags_Inventory,AFI_Tag,Mask_length_bits,Mask, &ISO15693_Inventory_16slots_Nb_UID_Retrieved);
}


/**
  * @brief this function scan the tag in the field
  * @param void : no requested parameter
  * @retval void: no return value
  */
static void TagHunting (void)
{	
//---------------------

u8 Data_1 [ISO15693_DEFAULT_BLOCK_SIZE];// 1 block of data
u8 Tag_1 [ISO15693_UID_SIZE]; // to store UID of contactless tag
u8 Tag_2 [ISO15693_UID_SIZE];// to store UID of contactless tag
u8 Tag_counter;	
	//-------------------------------------------------------------
	
	if(ISO15693_Detect_Tag()==ISO15693_SUCCESS_CODE)
	{
		LED_Switch_on(GREEN_LED);// activate GREEN
		LED_Switch_off(BLUE_LED);// deactivate BLUE LED
			
		// Compute Get system info flag
		// Note that for device with less more than 8kbits memory the protocol extension flag should be set
		ISO15693_Request_Flags=ComputeByte(
		ISO15693_BIT3RESET_BIT8_RFU,
		ISO15693_BIT3RESET_BIT7_OPTIONFLAG_SET,
		ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED,
		ISO15693_BIT3RESET_BIT5_REQUEST_TAG_NOT_SELECTED,
		ISO15693_BIT4_PROTOCOL_EXTENSION,//ISO15693_BIT4_NO_PROTOCOL_EXTENSION 
		ISO15693_BIT3_INVENTORY_FLAG_RESET,
		ISO15693_BIT2_LOW_DATARATE,
		ISO15693_BIT1_SINGLE_SUBCARRIER
		);
		// For an inventory 16 slots the UID received are stored into the EEPROM
		// The purpose of this simple loop is to retieved each UID and to send a get system info to every contactless which answered the inventory command.
		Tag_counter=ISO15693_Inventory_16slots_Nb_UID_Retrieved; // Store the number of tags seen
		while (Tag_counter!=INIT_VALUE)
		{
			Resp_status= ISO15693_Get_UID_from_EEPROM(Tag_1,Tag_counter);
			Resp_status=M24LRXX_Get_System_Info(&MyTag15693, ISO15693_Request_Flags,Tag_1);
			if(Resp_status!= M24LRXX_SUCCESS_CODE)
			{
				ISO15693_Request_Flags=ComputeByte(
				ISO15693_BIT3RESET_BIT8_RFU,
				ISO15693_BIT3RESET_BIT7_OPTIONFLAG_SET,
				ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED,
				ISO15693_BIT3RESET_BIT5_REQUEST_TAG_NOT_SELECTED,
				ISO15693_BIT4_NO_PROTOCOL_EXTENSION,
				ISO15693_BIT3_INVENTORY_FLAG_RESET,
				ISO15693_BIT2_LOW_DATARATE,
				ISO15693_BIT1_SINGLE_SUBCARRIER);
				Resp_status=LRIXK_Get_System_Info(&MyTag15693, ISO15693_Request_Flags,Tag_1);
			}
		
			
			// Make some display
			ISO15693_Display_Tag_Answer_Inventory_GetSys_Info();
			Tag_counter--;
		}
		
	}
	else 
	{
		if(Protocol_Settings.protocol_in_use==CR95HF_PROT_SEL_FIELDOFF)
		{
			LCD_DisplayLogo(LCD_LOGOTYPE_LINE1_LEFT_CORNER );
			LCD_Print(" CR95HF ");
			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"## Field off ##");
			Display_Motion(LOW_SPEED_DISPLAY_MOTION);
			LED_Switch_on(YELLOW_LED);// activate  Yellow LED
			LED_Switch_off(BLUE_LED);// deactivate  blue LED
		}
		else
		{
			LCD_DisplayLogo(LCD_LOGOTYPE_LINE1_LEFT_CORNER );
			LCD_Print(" CR95HF ");
			LCD_PrintString(LCD_LINE2,ENABLE,DISABLE,"##  No tag   ##");
			Display_Motion(DEFAULT_SPEED_DISPLAY_MOTION);
			LED_Switch_on(BLUE_LED);// activate  BLUE LED
			LED_Switch_off(YELLOW_LED);// deactivate  Yellow LED
		}
			
		LED_Switch_off(GREEN_LED);// deactivate GREEN LED
			
	}
}		


// Some display functions

/**  
* @brief  This function displays the UID retrieved by the Inventory command.If the several contactless tag answered then only the number of contactless tags seen is displayed   
* @param void : no paramter expected, all data are retrieved from the MyTag15693 structure and ISO15693_Request_Flags
* @retval	void : no return value		
*/
static void ISO15693_Display_Tag_Answer_Inventory_GetSys_Info(void) 
{
	if(MyTag15693.command_ok==ISO15693_SUCCESS_CODE)
	{
		LCD_Clear();
		if(ISO15693_Get_Slots_Flag(ISO15693_Request_Flags_Inventory)==ISO15693_1_SLOT)
		{
			ISO15693_Display_Tag_ICREF(MyTag15693.ICRef, LCD_LINE1);
			ISO15693_Display_Tag_UID(MyTag15693.UID_Tag,LCD_LINE2);		
		}		
		else
		{
			// If there is only one tag display its UID
			if(ISO15693_Inventory_16slots_Nb_UID_Retrieved==1)
			{
				// If the ICRef is known then the name of the contacless is overwritten on line 1 of the LCD screen 
				ISO15693_Display_Tag_ICREF(MyTag15693.ICRef, LCD_LINE1);
				ISO15693_Display_Tag_UID(MyTag15693.UID_Tag,LCD_LINE2);
			}
			else // Several contactless tag answereed the inventory command
			{
				LCD_SetCursorPos(LCD_LINE1, LCD_STARTLEFTCORNER);
				// Display the number of contactless tag seen
				LCD_PrintString(LCD_LINE1,DISABLE,ENABLE," TAGS SEEN: ");
				LCD_PrintDec(ISO15693_Inventory_16slots_Nb_UID_Retrieved);
			}
		}
	}
	// in this case the command Get Syst Info failed but the inventory passed. Note that the Get system Info is not aninventory command therefor only the UID is available display it
	else
	{
		LCD_Clear();
		// Display the UID without the MSB 'EO'
		LCD_PrintString(LCD_LINE1,ENABLE,DISABLE,"   TAG UID");
		ISO15693_Display_Tag_UID(MyTag15693.UID_Tag,LCD_LINE2);
	}
}



/**  
* @brief  This function displays the Unique IDendtifier of the contactless tag. EO and constructor on first line others bytes on the  second line.Note that EO is not displayed as it is common to all contactless tags.   
* @param UID : pointer on the UID ( ISO15693Tag.UID for example)
* @param Line : line on the LCD screen used for the display
* @retval	void : no return value		
*/
static void ISO15693_Display_Tag_UID(const u8* UID, const u8 Line)
{
	u8 UID_cursor=ISO15693_UID_SIZE;// Starting from MSB
	UID_cursor--;// strating from UID[7]
	UID_cursor--;// Masking EO
	LCD_SetCursorPos(Line, LCD_STARTLEFTCORNER);
	// Note that as EO is common to all contactless tags the byte is not displayed
	// Print other bytes 6-1 as counter is unsigned 0-1=F => infinite loop
	while(UID_cursor!=INIT_VALUE)
	{
		LCD_PrintHex2(UID[UID_cursor--]);
	}
	LCD_PrintHex2(UID[UID_cursor]);//UID cursor =0 at this level
}


/**  
* @brief  This function displays the name of the product using the ICRef. Only products identified in ISO15693_display_answer.h have their name displayed, for others the field is displayed as is.
* @param Tag_ICRef : the Icref of the product
* @param Line: Line to display Icref on the screen
* @retval	ISO15693_DISPLAY_UNHANDLED : no display done	
* @retval	ISO15693_DISPLAY_HANDLED : a display was done			
*/
static u8 ISO15693_Display_Tag_ICREF(const u8 Tag_ICRef, const u8 Line)
{
	u8 status=ISO15693_DISPLAY_UNHANDLED;
	u8 ICRef_6MSB=Tag_ICRef&MASK_ICREF_6MSB;// retrieve only the 6 MSB bits
	LCD_SetCursorPos(Line, LCD_STARTLEFTCORNER);
	// Check if Icref is different from the init value i.e the manufacturer is ST and ICref has been filled in.
	if( Tag_ICRef!= ISO15693_STRUCT_INITVALUE)
	{
		if(ICRef_6MSB==LRI1K)
		{
			LCD_PrintString(Line,ENABLE,DISABLE, LRI1K_TEXT );
		}
		else
		{
			if(ICRef_6MSB==LRI2K)
			{
				LCD_PrintString(Line,ENABLE,DISABLE, LRI2K_TEXT );
			}
			else
			{
				if(ICRef_6MSB==LRIS2K)
				{
					LCD_PrintString(Line,ENABLE,DISABLE, LRIS2K_TEXT );
				}
				else
				{
					if(ICRef_6MSB==LRIS64K)
					{
						LCD_PrintString(Line,ENABLE,DISABLE, LRIS64K_TEXT );
					}
					else
					{
						if(ICRef_6MSB==M24LR64_R)
						{
							LCD_PrintString(Line,ENABLE,DISABLE, M24LR64_R_TEXT );
						}
						else
						{
							if(ICRef_6MSB==M24LRXX_E)
							{
								LCD_PrintString(Line,ENABLE,DISABLE, M24LRXX_E_TEXT );
								// the two LSB bits indicates the size of the devices
								if(Tag_ICRef==M24LR16_E)
									LCD_PrintString(Line,ENABLE,DISABLE, M24LR16_E_TEXT );
									
							}
							else
							{
								LCD_PrintString(Line,DISABLE,ENABLE,"ICREF: " );
								LCD_PrintHex2(Tag_ICRef);
							}
						}
					}		
				}
			}
		}
		status=ISO15693_DISPLAY_HANDLED;
	}
	return status;
}



/**  
* @brief  This function displays the memsize of the tag. 
* @param MemSize : pointer on data containing the size
* @param Line: Line to display Icref on the screen
* @retval	ISO15693_DISPLAY_UNHANDLED : no display done	
* @retval	ISO15693_DISPLAY_HANDLED : a display was done			
*/
static u8 ISO15693_Display_Tag_MemSize(const u8* MemSize, const u8 Number_of_Bytes_MemSize, const u8 Line)
{
	u32 size;//to retrieve number up to 1023 for memory size
	// Begin display
	LCD_SetCursorPos(Line, LCD_STARTLEFTCORNER);
	LCD_PrintString(Line,DISABLE,ENABLE,"MEMORY: " );
	// Compute size
	size= ISO15693_Get_MemSize_kbits(MemSize,Number_of_Bytes_MemSize);
	if(size==INIT_VALUE)// if the size is less than 1kbits
	{
		size=ISO15693_Get_MemSize_bits(MemSize,Number_of_Bytes_MemSize);
		LCD_PrintDec7(size);// Display =size in bits
		LCD_PrintString(Line,DISABLE,ENABLE," b" );
	}
	else
	{
		LCD_PrintDec(size);// Display =size in kbits
		LCD_PrintString(Line,DISABLE,ENABLE," kb" );
	}
	return ISO15693_DISPLAY_HANDLED;
}

