/**
  **********************************************************
  * @file cr95hf_command.c 
  * @brief This file contains all the functions in order to send commands to the CR95HF
  * @author STMicroelectronics - MMY Application Team
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */

#include "cr95hf_command.h"
	
// This variable is retrieved from the board_config files to determine the communication interface used.
extern u8 SPI_or_UART_Communication;
// This variable is retrieved from the board_config files to determine baud rate used in UART.
extern u32 MCU_Baud_Rate;
// This variable is retrieved from the main file to point out a communication lost, it is used here for the idle command
extern u8 communication_watchdog; 

// those two variables are created in order to save memroy space by creating two common structure for command which do not return the structyure received
CR95HF_CommandSending CR95HF_Cmd_to_Send; /*!< Common Stucture used  to transmit data within commands*/

CR95HF_ResponseReceiving CR95HF_Response;/*!< Common Stucture used  to receive data within commands*/


// Main Architecture of the doxyfile for this file 
 // @defgroup  CR95HF
	
	// * @defgroup  CR95HF_Commands
	
		// * @defgroup  CR95HF_Advanced_Commands
	
	// * @defgroup  CR95HF_Support_Functions
	
			// * @defgroup  CR95HF_Procedures

	

 // @addtogroup CR95HF_Commands
 

/**  
* @brief  	This function sends an IDN command 
* @param  	IDN_Response: pointer on the CR95HF response structure to fill in
* @retval		CR95HF_SUCCESS_CODE : the command is successful
* @retval		CR95HF_ERROR_CODE : the command failed
*/

u8 CR95HF_IDN (CR95HF_ResponseReceiving* IDN_Response)
{
	u8 status=CR95HF_ERROR_CODE;
	
	// init the reponse
	CR95HF_Init_ResponseReceiving(IDN_Response);
	
	// Fill in the command code and the the length
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_IDN;
	CR95HF_Cmd_to_Send.data_length = DATALENGTH_NULL;
	
	// send the command and put the response into the response structure provided 
	*IDN_Response = CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);
	
	// testing success of command
	status=CR95HF_IsReaderResultOK(IDN_Response,CR95HF_IDN_SUCCESS_CODE);
	

	return status;
}


/**  
* @brief  	This function sends a protocol select command
* @param  	Protocol : protocol selected 
* @param 		ParametersLength : Length of eventual parameters attached
* @param		Parameters : Eventual parameters to be attached for different protocols
* @param		Protocol_Settings : structure to update at the end of the command containing last protocol selected and parameters 
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval		CR95HF_ERROR_INVALID_PROTOCOL : The protocol selected is wrong
* @retval		CR95HF_ERROR_INVALID_CMD_LENGTH : The command length is invalid  
* @retval 	CR95HF_ERROR_CODE: unsuccessful command
*/
u8 CR95HF_ProtocolSelect (const u8 Protocol, const u8 ParametersLength,const  u8* Parameters, CR95HF_Protocol_Config* Protocol_Settings)
{
	// private variables
	u8 status=CR95HF_ERROR_CODE;
	u8 DataLengthToSend=INIT_VALUE;
	
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);// Initialyze the response
	
	// Filling the command parameters
	CR95HF_Cmd_to_Send.cmd_code=CR95HF_CMD_PROT_SEL;
	// Protocol selection and update of length at once
	CR95HF_Cmd_to_Send.data[DataLengthToSend++]=Protocol;
	
	// Send parameters after having overflow checked 
	if((ParametersLength+DataLengthToSend)<MAX_DATASENT_LENGTH)
	{
		// Add the parameters to the command
		CR95HF_Add_Data_ToCommand(&CR95HF_Cmd_to_Send, &DataLengthToSend, ParametersLength, Parameters);
	}
	// Set length of data to transmit
	CR95HF_Cmd_to_Send.data_length=DataLengthToSend;
	
	// Send command and receive answer
	CR95HF_Response= CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);	
	
	// Analyzing response
	if(CR95HF_Response.data_length==DATALENGTH_NULL)
	{
		// check if the command is ok
		status=CR95HF_IsReaderResultOK(&CR95HF_Response,CR95HF_PROTOCOL_SUCCESS_CODE);
		// If the command is not ok, try to find the reason
		if((CR95HF_Response.resp_code==CR95HF_ERROR_INVALID_CMD_LENGTH) || 
			(CR95HF_Response.resp_code==CR95HF_ERROR_INVALID_PROTOCOL ))
			
				status=CR95HF_Response.resp_code;
	}
	// if dataLength is not ok, the returned value will be CR95HF_ERROR_CODE
	// Update the protocol selected structure with the protocol and parameters
	CR95HF_Update_Protocol_Selected( Protocol_Settings, status, 
	Protocol,ParametersLength,  Parameters);
	

	return status;	
}

/**  
* @brief  	this function sends an Send Receive command allowing user to perform communication with tag protocol must be selected before calling this function
* @param  	SendRecv_Command :	the command to send
* @param  	SendRecv_Response : pointer on the response 
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval 	CR95HF_ERROR_CODE: unsuccessful command,Response contains the right error code 
*/
u8 CR95HF_SendRecv
(CR95HF_ResponseReceiving* SendRecv_Response, CR95HF_CommandSending* SendRecv_Command)
{
	CR95HF_Init_ResponseReceiving (SendRecv_Response); // Initialize the structure
	// Add the command code, send the command and receive the response
	SendRecv_Command->cmd_code=CR95HF_CMD_SEND_REC;
		
	*SendRecv_Response=CR95HF_SendReceive_Cmd(SendRecv_Command);
	// Check if the command is ok
	return CR95HF_IsReaderResultOK(SendRecv_Response,CR95HF_SENDRECV_SUCCESS_CODE);
}


/**  
* @brief  	This function sends an idle command using the parameters provided
* @param  WakeUpFlags : Specifies wake up condition	
* @param  EnterCtrlL : first byte of setting to enter to Idle mode
* @param  EnterCtrlH : second byte of setting to enter to Idle mode
* @param  WUCtrlL : first byte of setting to wake-up from Idle mode
* @param  WUCtrlH : second byte of setting to wake-up from Idle mode
* @param  LeaveCtrlL : first byte of setting to leave Idle mode
* @param  LeaveCtrlH : second byte of setting to leave Idle mode
* @param  WUPeriod : Period of time between two tags detection
* @param  OscStart :  Waiting time to stabilize HFO
* @param  DacStart : Waiting time to stabilize DAC
* @param  DacDataL : lower compare value for tag detection
* @param  DacDataH : higher compare value for tag detection
* @param  SwingsCnt : Number of HF swings during tag detection
* @param  MaxSleep : maximum number of tag detection trials before timeout
* @param  Idle_Config : pointer on the structure containg the data about some idle parameters to update. 
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval 	CR95HF_ERROR_CODE : unsuccessful command,Response contains the right error code 
*/
u8 CR95HF_Idle ( const u8 WakeUpFlags, const u8 EnterCtrlL,const  u8 EnterCtrlH,  const u8 WUCtrlL, const u8 WUCtrlH, const u8 LeaveCtrlL,const  u8 LeaveCtrlH ,
	const u8 WUPeriod, const u8 OscStart,const  u8 DacStart,const  u8 DacDataL,const  u8 DacDataH,const  u8 SwingsCnt,const  u8 MaxSleep, 
	CR95HF_Idle_Config* Idle_Config)
{
	u8 status=CR95HF_ERROR_CODE;
	u8 command_answer;
	u8 data_counter= INIT_VALUE;
	
	// init the response
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);
	
	// Fill in the command code and the length
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_IDLE;
	CR95HF_Cmd_to_Send.data_length = CR95HF_CMD_IDLE_DATALENGTH;
	// Add data
	CR95HF_Cmd_to_Send.data[data_counter++]=WakeUpFlags;
	CR95HF_Cmd_to_Send.data[data_counter++]=EnterCtrlL;
	CR95HF_Cmd_to_Send.data[data_counter++]=EnterCtrlH;
	CR95HF_Cmd_to_Send.data[data_counter++]=WUCtrlL;
	CR95HF_Cmd_to_Send.data[data_counter++]=WUCtrlH;
	CR95HF_Cmd_to_Send.data[data_counter++]=LeaveCtrlL;
	CR95HF_Cmd_to_Send.data[data_counter++]=LeaveCtrlH;
	CR95HF_Cmd_to_Send.data[data_counter++]=WUPeriod;
	CR95HF_Cmd_to_Send.data[data_counter++]=OscStart;
	CR95HF_Cmd_to_Send.data[data_counter++]=DacStart;
	CR95HF_Cmd_to_Send.data[data_counter++]=DacDataL;
	CR95HF_Cmd_to_Send.data[data_counter++]=DacDataH;
	CR95HF_Cmd_to_Send.data[data_counter++]=SwingsCnt;
	CR95HF_Cmd_to_Send.data[data_counter++]=MaxSleep;
	
	// send the command 
	CR95HF_SendCommand(&CR95HF_Cmd_to_Send);
	// If CR95HF turns into another mode than ready, it will answer only when it wakes up. Thus only error will be retrieved at this time
	// Wait for any answer
	 command_answer=CR95HF_Wait_Answer(DEFAULT_DELAY_BEFORE_TIMEOUT);
	 // Check if wait_answer because of timeout
	 if(command_answer=EXIT_COMMUNICATION_TIMEOUT)
	 {
		 communication_watchdog--;// decrement this variable because a no response is "normal" in this case
		 Idle_Config->wuFlags=WakeUpFlags;// Update the Wake up flags
		 status=CR95HF_SUCCESS_CODE;// Command passed
	 }
	 else // CR95HF has answered immediately, an error occured
	 {
		 CR95HF_Response=CR95HF_ReceiveAnswer();// receive answer with the error
		 // Error is written into response status already equals CR95HF_ERROR_CODE
	 }

	return status;
}


/**  
* @brief  	This function reads the wake up register  of the CR95HF to determine what caused the awakening of the CR95HF after an Idle command.
* @param  	Response : pointer on the response 
* @param  	Idle_Config : pointer on the structure to update 
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval 	CR95HF_ERROR_CODE: The command failed
*/
u8 CR95HF_Rd_Wake_up_Reg(CR95HF_ResponseReceiving* Response, CR95HF_Idle_Config* Idle_Config)
{
	u8 status=CR95HF_ERROR_CODE;
	u8 data_counter= INIT_VALUE;
	
	// init the response
	CR95HF_Init_ResponseReceiving(Response);
	
	// Fill in the command code and the length
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_RDREG;
	CR95HF_Cmd_to_Send.data_length = CR95HF_CMD_RD_REG_DATALENGTH;
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_WAKE_UP_REGISTER_CURSOR;
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_WAKE_UP_REGISTER_SIZE;
	CR95HF_Cmd_to_Send.data[data_counter]=CR95HF_RD_REG_ST_RESERVED;
	
	// send the command and put the response into the response structure provided 
	*Response = CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);
	
	// testing success of command
	status=CR95HF_IsReaderResultOK(Response,CR95HF_RD_REG_SUCCESS_CODE);
	if(status==CR95HF_SUCCESS_CODE )
		Idle_Config->last_reason_of_wakeup=Response->data[0];// Update Idle Config struture with the result 
	

	return status;
}


/**  
* @brief  	This function reads the RCB register of the CR95HF to check the analog configuration Register containing the modulation depth and reader chain gain. Note that a CR95HF_Set_Analog_Register_Config_B_Index command must be issued before reading the register.
* @param  	Response : pointer on the response 
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval 	CR95HF_ERROR_CODE: The command failed
*/
u8 CR95HF_Rd_Analog_Register_Config_B(CR95HF_ResponseReceiving* Response)
{
	u8 data_counter= INIT_VALUE;
	
	// init the response
	CR95HF_Init_ResponseReceiving(Response);
	
	// Fill in the command code and the length
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_RDREG;
	CR95HF_Cmd_to_Send.data_length = CR95HF_CMD_RD_REG_DATALENGTH;
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_ANALOG_CONFIG_REGISTER_CURSOR;
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_ANALOG_CONFIG_REGISTER_SIZE;
	CR95HF_Cmd_to_Send.data[data_counter]=CR95HF_RD_REG_ST_RESERVED;
	
	// send the command and put the response into the response structure provided 
	*Response = CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);
	
	// testing success of command
	return CR95HF_IsReaderResultOK(Response,CR95HF_RD_REG_SUCCESS_CODE);
}


/**  
* @brief  	This command is used to set the address index before reading the analog configuration register.  
* @retval		CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed
*/
u8 CR95HF_Set_Analog_Register_Config_B_Index ( void)
{
	u8 data_counter= INIT_VALUE; // counter for data into the command and command datalength
	
	// Init the response
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);
	
	// Fill in the command code and parameters
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_WRT_REG;
	// Add datalength to command
	CR95HF_Cmd_to_Send.data_length = CR95HF_SET_INDEX_LENGTH_OF_CMD;
	// Add the Analog COnfig register index
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_ANALOG_CONFIG_REGISTER_INDEX;
	// Add Flag increment after write
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_SET_INDEX_FLAG_INCREMENT;
	// Add index pointing  
	CR95HF_Cmd_to_Send.data[data_counter]=CR95HF_ANALOG_CONFIG_INDEX;
	// Send the command and put the response into the response structure provided 
	CR95HF_Response = CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);
	
	// testing success of command
	return CR95HF_IsReaderResultOK(&CR95HF_Response,CR95HF_RD_REG_SUCCESS_CODE);
}


/**  
* @brief  	This command is used to overwrite the Analog Configuration Register.  
* @param Analog_Configuration_Register : value of the Analog Configuration Register to overwrite
* @retval		CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_INVALID_CMD_LENGTH : command failed, invalid length
*/
u8 CR95HF_Wr_Analog_Register_Config_B_Reg(u8 Analog_Configuration_Register)
{
	u8 data_counter= INIT_VALUE; // counter for data into the command and command datalength
	
	// Init the response
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);
	
	// Fill in the command code and parameters
	// Fill in the command code and parameters
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_WRT_REG;
	// Add datalength to command
	CR95HF_Cmd_to_Send.data_length =CR95HF_WR_REG_LENGTH_OF_CMD ;
	// Add the Analog COnfig register index
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_ANALOG_CONFIG_REGISTER_INDEX;
	// Add Flag increment after write
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_WR_REG_FLAG_INCREMENT;
	// Add index pointing  
	CR95HF_Cmd_to_Send.data[data_counter++]=CR95HF_ANALOG_CONFIG_INDEX;
	// Add the new configuration value for modulation and gain
	CR95HF_Cmd_to_Send.data[data_counter]=Analog_Configuration_Register;
	// Send the command and put the response into the response structure provided 
	CR95HF_Response = CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);
	
	// testing success of command
	return CR95HF_IsReaderResultOK(&CR95HF_Response,CR95HF_RD_REG_SUCCESS_CODE);
}


/**  
* @brief  	This function changes the baud rate communication between MCU and CR95HF. If the command fails last
* @param  	New_Baud_Rate : baud rate value computed as described in the datasheet : 117-> default value
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval 	CR95HF_ERROR_CODE: unsuccessful command, UART communication may be lost.
*/
u8 CR95HF_Baud_Rate(const u8 New_Baud_Rate)
{
	u32 MCU_New_Baud_Rate;// to store MCU baud rate
	
	// Init the response
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);
	// Compute Baud rate for mcu refer to CR95HF datasheet for further information about computing process
	MCU_New_Baud_Rate=(13.56*1000000)/(2.0*New_Baud_Rate+2.0); // Computing order is changed to avoid 0 value after 13.56/(2.0*New_Baud_Rate+2)
	
	// Fill in the command code and the length
	CR95HF_Cmd_to_Send.cmd_code = CR95HF_CMD_BAUDRATE;
	CR95HF_Cmd_to_Send.data_length = CR95HF_CMD_BAUD_RATE_DATALENGTH;
	CR95HF_Cmd_to_Send.data[INIT_VALUE]=New_Baud_Rate;
	
	// send the command 
	CR95HF_SendCommand(&CR95HF_Cmd_to_Send);
	// Change MCU baud rate
	USART_Init( USART2, MCU_New_Baud_Rate, USART_WordLength_8b, USART_StopBits_1,
                USART_Parity_No,  (USART_CR2_TEN | USART_CR2_REN));	
	// Try to receive echo answer from CR95HF with new baud rate
	CR95HF_Response= CR95HF_ReceiveAnswer();
	return CR95HF_IsReaderResultOK(&CR95HF_Response,CR95HF_ECHO_SUCCESS_CODE);
}





/**  
* @brief  	this function sends an ECHO command
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval 	CR95HF_ERROR_CODE : the command failed 
*/
u8 CR95HF_Echo (void)
{
	u8 status=CR95HF_ERROR_CODE;
	
	// Init response
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);
	
	// Fill the command parameter
	CR95HF_Cmd_to_Send.cmd_code=CR95HF_CMD_ECHO;
	CR95HF_Cmd_to_Send.data_length=DATALENGTH_NULL;
	
	CR95HF_Response=CR95HF_SendReceive_Cmd(&CR95HF_Cmd_to_Send);// Send command and receive response
	
	// Check result
	status= CR95HF_IsReaderResultOK(&CR95HF_Response,CR95HF_ECHO_SUCCESS_CODE);
	if(status!=CR95HF_SUCCESS_CODE)
		CR95HF_Response.resp_code=CR95HF_COMMUNICATION_WITH_MCU_FAILED;
		
	
	return status;
}

// End of commands

 // @addtogroup CR95HF_Advanced_Commands
 

/**  
* @brief  	This function swithes the RF Field Off using the protocol select comand.
* @param		Protocol_Settings : structure to update at the end of the command containing last protocol selected and parameters 
* @retval		CR95HF_SUCCESS_CODE : The fied was successsfully turned off 
* @retval		CR95HF_ERROR_INVALID_PROTOCOL : The protocol selected is wrong
* @retval		CR95HF_ERROR_INVALID_CMD_LENGTH : The command length is invalid  
* @retval 	CR95HF_ERROR_CODE: unsuccessful command
*/
u8 CR95HF_FieldOff(CR95HF_Protocol_Config* Protocol_Settings)
{
	u8 parameters=CR95HF_FIELDOFF_PARAMETER_BYTE;
	return CR95HF_ProtocolSelect (CR95HF_PROT_SEL_FIELDOFF, CR95HF_PARAMETERSLENGTH_FIELDOFF, &parameters, Protocol_Settings);
}


/**  
* @brief  	This function sends an End Of Frame. This can be used for instance in  ISO15693 inventory command or write command. Right after the EOF the tag could answer  a command (Write or Inventory), so the response is received and written into the structure provided.
* @param		Response: pointer on the structure which will contain the Tag answer 
* @retval		CR95HF_SUCCESS_CODE : The command is successful  
* @retval 	CR95HF_ERROR_CODE unsuccessful command
*/
u8 CR95HF_SendEOF(CR95HF_ResponseReceiving* Response)
{
	// Init command with data_length=0
	CR95HF_Cmd_to_Send.data_length=DATALENGTH_NULL;
	return CR95HF_SendRecv(Response, &CR95HF_Cmd_to_Send);
}


// End of Advanced_commands

 // @addtogroup CR95HF_Procedures
 

/**  
* @brief  	This function is a procedure to change Baud_Rate in UART communication. The function calls the Baud_Rate function and changes the MCU internal baud rate. In case of failure the function tries to recover the communication by returning into the previous baud rate or into the default baud rate. 
* @retval		CR95HF_SUCCESS_CODE : The command is successful 
* @retval		CR95HF_BAUd_RATE_FAILED_PREVIOUS_VALUE_ON : The command failed but keep transmission with the previous baud rate
* @retval		CR95HF_BAUD_RATE_FAILED_DEFAULT_VALUE_ON : The command failed and did not manage to keep previous baud rate. Default baud rate is on.
* @retval 	CR95HF_ERROR_CODE: unsuccessful command, UART communication may be lost.
*/
u8 CR95HF_Modify_Baud_Rate(const u8 New_Baud_Rate)
{
	u8 status=CR95HF_ERROR_CODE; // procedure status
	u8 resp_code_display=CR95HF_BAUD_RATE_ERROR;//As the Baud rate does not send response codes (apart from 0x55) create resp code for display
	u8 baudrate_status;// command baud rate status
	u8 echo_status; // echo command status
	u32 MCU_New_Baud_Rate;// to store MCU baud rate
	
	
	// Compute New baud rate for MCU (also computed into the Baudrate command
	MCU_New_Baud_Rate=(13.56*1000000)/(2.0*New_Baud_Rate+2.0); // Computing order is changed to avoid 0 value after 13.56/(2.0*New_Baud_Rate+2)
	
	// Send Baud Rate command 
	baudrate_status=CR95HF_Baud_Rate(New_Baud_Rate);
	// Check if the command is ok: CR95HF has answered with new baud rate
	if(baudrate_status==CR95HF_ERROR_CODE)
	{
		// Command failed try to send an echo in case MCU baud rate reconfiguration was too long
		delay_ms(WAIT_3ms);
		if( CR95HF_Echo ()!=CR95HF_SUCCESS_CODE)
		{
			// Command definitely failed try to return into previous baud rate
			// Global variable Baud_rate contains the previous valid value
			USART_Init( USART2, MCU_Baud_Rate, USART_WordLength_8b, USART_StopBits_1,
                USART_Parity_No,  (USART_CR2_TEN | USART_CR2_REN));	
		
			// Send an echo command 
			if( CR95HF_Echo ()!=CR95HF_SUCCESS_CODE)
			{
				// Previous baud rate recover failed.
				// Send again command to return into default baud rate
				baudrate_status=CR95HF_Baud_Rate(CR95HF_DEFAULT_BAUD_RATE);
				
				if(baudrate_status==CR95HF_ERROR_CODE)
				{
					// Command failed try to send an echo in case MCU baud rate reconfiguration was too long
					delay_ms(WAIT_3ms);
					if( CR95HF_Echo ()==CR95HF_SUCCESS_CODE)
					{
						// Echo succeed in default baud rate is on
						status=CR95HF_BAUD_RATE_FAILED_DEFAULT_VALUE_ON;
						MCU_Baud_Rate=MCU_DEFAULT_BAUD_RATE;
					}
					// else Command failed status=CR95HF_ERROR_CODE (initial value)
				}
				else
				{
					// First echo succeed in, default baud rate is on
						status=CR95HF_BAUD_RATE_FAILED_DEFAULT_VALUE_ON;
						MCU_Baud_Rate=MCU_DEFAULT_BAUD_RATE;
				}
			}
			else
			{
				// Previous baud rate is on (echo command)
				status=CR95HF_BAUD_RATE_FAILED_PREVIOUS_VALUE_ON;
				// no need to update MCU_Baud_Rate variable
			}
		}
		else
		{
			// New baud rate is on (after echo command)
			status=CR95HF_SUCCESS_CODE;
			MCU_Baud_Rate=MCU_New_Baud_Rate;
			resp_code_display=CR95HF_SUCCESS_CODE;
		}
	}
	else
	{
		// New baud rate is on (response code ok)
		// IsReader_ok function write successcode into response
		status=CR95HF_SUCCESS_CODE;
		MCU_Baud_Rate=MCU_New_Baud_Rate;
		resp_code_display=CR95HF_SUCCESS_CODE;
	}
	
	return status;
}

// End of procedures

 // @addtogroup CR95HF_Support_Functions
	
	// * @defgroup CR95HF_Transmission_Reception_Functions
 
/**  
* @brief  	This function sends a command to the cr95hf
* @param  	Cmd		:	the command structure to send
* @retval		void	: no return value
*/	
void CR95HF_SendCommand(CR95HF_CommandSending* Cmd)
{
	u8 cptr_datalength=INIT_VALUE;
	u8 i=INIT_VALUE;// counter
	cptr_datalength= Cmd->data_length;// the counter worths the length of data to transmit
	
	// Enable communication (for SPI if selected)
	CR95HF_SPI_Chip_Enable_Send_Byte_Mode();	
	
	// Start sending command 
	CR95HF_Send_Byte(Cmd->cmd_code);// Send the command code for the CR95HF
	if(Cmd->cmd_code!= CR95HF_CMD_ECHO)// For an echo command there is no need to tranmist datalength
		CR95HF_Send_Byte(Cmd->data_length);// Send the length of the data to be transmitted 
	
	
	// Send the data included in the command byte per byte
	while(cptr_datalength!=0)
	{
		CR95HF_Send_Byte(Cmd->data[i]);// Send byte
		i++;// increment byte counter
		cptr_datalength--; // decrement number of bytes to be transmitted 
	}
	
	// Communication ending
	CR95HF_SPI_Chip_Disable();
}

/**  
* @brief  	this function receives and returns the response to a command previously sent to the cr95hf
* @retval		CR95HF_ResponseReceiving : the response into a CR95HF_ResponseReceiving structure format 	
*/	
CR95HF_ResponseReceiving CR95HF_ReceiveAnswer(void)
{
	u8 cptrdataLength=INIT_VALUE;// length of the data to read
	u8 i=INIT_VALUE;// counter
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);// init response
	
	// Enable communication
	CR95HF_SPI_Chip_Enable_Read_Byte_Mode();
	
	// Reading incoming data
	CR95HF_Response.resp_code = CR95HF_Read_Byte();// reading the Resp code	
	if(CR95HF_Response.resp_code!= CR95HF_ECHO_SUCCESS_CODE)// For an echo command there is no need to tranmist datalength
		CR95HF_Response.data_length = CR95HF_Read_Byte();// reading the length of data		
	
	cptrdataLength=CR95HF_Response.data_length;// If command was echo response.data_length remains at its initial value i.e: DATALENGTHNULL
	
	if(cptrdataLength <= MAX_DATARECEIVED_LENGTH)
	{
		while (cptrdataLength!=0)
		{
			// Read the data byte per byte, increment the counter, decrement number of byte to be received
			CR95HF_Response.data[i++] = CR95HF_Read_Byte();
			cptrdataLength--;
		}
	}
	// Communication ending
	CR95HF_SPI_Chip_Disable();

	// return the response into a CR95HF_ResponseReceiving structure format
	return CR95HF_Response;
}


/**  
* @brief  	this function sends a command and returns the response received
* @param  	Cmd :	the command structure to send 
* @retval		CR95HF_ResponseReceiving : the response into a CR95HF_ResponseReceiving structure format 	
*/	
static CR95HF_ResponseReceiving CR95HF_SendReceive_Cmd(CR95HF_CommandSending* Cmd)
{
	CR95HF_Init_ResponseReceiving(&CR95HF_Response);
	
	CR95HF_SendCommand(Cmd); // Send the command
	
	// Wait for anwser and receive it
	CR95HF_Wait_Receive_Answer(&CR95HF_Response);

	return CR95HF_Response;// return the response
}


/**  
* @brief  	This function waits and receive the answer frrom the CR95HF
* @param  	Response : structure to store the answer received into a CR95HF_ResponseReceiving structure format 	
*/
void CR95HF_Wait_Receive_Answer(CR95HF_ResponseReceiving* Response)
{
	// Wait for anwser
	if(CR95HF_Wait_Answer(DEFAULT_DELAY_BEFORE_TIMEOUT)==EXIT_COMMUNICATION_OK)
		*Response=CR95HF_ReceiveAnswer(); // Receive the answer
	else
	{
		Response->resp_code=CR95HF_COMMUNICATION_WITH_MCU_FAILED;
		Response->data_length=DATALENGTH_NULL;
	}
}

// End of CR95HF_Transmission_Reception_Functions 


 // @defgroup CR95HF_Initialisation_Function

/**  
* @brief  	this function initialize the response structure to erase previous data from any other command this allows to be sure that resp_code is not issued by random data
* @param  	Response :	the structure to initialyze
* @retval		void : no return value
*/
void CR95HF_Init_ResponseReceiving(CR95HF_ResponseReceiving* Response)
{
	Response->resp_code=CR95HF_INIT_ResponseReceiving;
	Response->data_length= DATALENGTH_NULL;
	Response->data[INIT_VALUE]=CR95HF_INIT_ResponseReceiving;// first data is written in order corrupt  previous data for CRC check
}

// End of CR95HF_Initialisation_Function


 // @defgroup CR95HF_Add_data_to_Command_Function
/**  
* @brief  	This function add an array of data into the command's data and increments the counter of data
* @param  	Cmd	:	pointer on the commmand to fill in 
* @param  	Cptr_ToIncrement : pointer on the counter
* @param		DataLength		: length of data to include
* @param		Data		: pointer on the data to include
* @retval		void : no return value
*/
void CR95HF_Add_Data_ToCommand(CR95HF_CommandSending* Cmd, u8* Cptr_ToIncrement, u8  DataLength, u8* Data)
{
	///*
	memcpy(Cmd->data+(*Cptr_ToIncrement),Data,DataLength);
	(*Cptr_ToIncrement)+=DataLength;
}
// End of CR95HF_CR95HF_Add_data_to_Command_Function

 // @defgroup CR95HF_Is_Functions
	

/**  
* @brief  	This function checks if the CR95HF has answered a success code.
* @param  	Response :	the structure to initialyze
* @param  	Code_OK :	reference value compared to the CR95HF reply
* @retval		 TRUE_CODE : CR95HF returned a succesful code
* @retval		 FALSE_CODE: CR95HF didn't return a succesful code
*/
u8 CR95HF_IsReaderResultOK(CR95HF_ResponseReceiving* Response,const u8 Code_OK)
{
	if( Response->resp_code==Code_OK)
	{
		Response->resp_code=CR95HF_SUCCESS_CODE;// Generic success code
		return TRUE_CODE;
	}
	// else error code, to indicate error type
	return FALSE_CODE;
}

// End of CR95HF_Is functions


	

/**  
* @brief  	This function computes the parameter byte for iso15693 protocol Select command, bit0 is forced to 1 to append CRC.
* @param  	Bit7_6:	RFU
* @param  	Bit5_4:	DataRate
* @param  	Bit3:	Respect 312s or wait for SOF
* @param  	Bit2:	Modulation
* @param  	Bit1:	Single or dual carrier
* @retval		parameterByte : the parameter byte to transmit within the command 
*/
u8 CR95HF_ISO15693_ComputeParameterByte_ProtocolSelect
( u8 Bit7_6,u8 Bit5_4,u8 Bit3,u8 Bit2,u8 Bit1)
{
	// Computing parameters at once
	return ComputeByte(Bit7_6,NEUTRAL_ELEMENT_OR,Bit5_4,NEUTRAL_ELEMENT_OR,
	Bit3,Bit2,Bit1,CR95HF_BIT0_APPEND_CRC);
}


/**  
* @brief  	the purpose of this function is to update the protocol Config structure containing the last protocol successfully selected. This function is called in the protocol select command. If the command fails, protocol is set to the CR95HF_ERROR_INVALID_PROTOCOL Value and others fiels are forced to 0
* @param Protocol_Settings : pointer on the structure to update
* @param status : status 	of the protocol select command
* @param Protocol : the Protocol now in use
* @param parameters_Length : number of byte for the parameters bytes
* @param Parameters : pointer on the parameters now used for the communication
* @retval		void : no return value
*/
static void CR95HF_Update_Protocol_Selected( CR95HF_Protocol_Config* Protocol_Settings, u8 Status,u8 Protocol, u8 parameters_Length, u8* Parameters)
{
	if(Status==CR95HF_PROTOCOL_SUCCESS_CODE)
	{
		Protocol_Settings->protocol_in_use=Protocol;
		Protocol_Settings->parameters_length=parameters_Length;
		memcpy(Protocol_Settings->parameters,Parameters,parameters_Length);
	}	
	else
	{
		Protocol_Settings->protocol_in_use=CR95HF_ERROR_INVALID_PROTOCOL;
		Protocol_Settings->parameters_length=INIT_VALUE;
		memset(Protocol_Settings->parameters,INIT_VALUE,MAX_PARAMETERS_LENGTH);
	}
}



 // @addtogroup CR95HF_Is_Functions
	

/**  
* @brief  This function is related to the idle mode. Its first goal is to determine whether  an interrupt on IRQ_in  is a wakeup condition (For instance if the CR95HF is in idle Mode, this function could also be called to determine which way to use to wake up the CR95HF).It can also be called once the CR95HF has woken up to determine if the CR95HF woke up because of timeout.Note that for the first case the WuFlags issued for the Idle command sould be given as input parameter (available in the CR95HF_Idle_Config structure) and for the second the case afther having issued CR95HF_Rd_Wake_up_Reg the  last_reason_of_wakeup field of the  CR95HF_Idle_Config structure may be used as input parameter  
* @param WUFlags : the byte containing the expected data : either the last_reason_of_wakeup field of the CR95HF_Idle_Config structure or the WuFlags used for the Idle command (also available in the CR95HF_Idle_Config structure
* @retval		FALSE_CODE :  an interrupt on IRQ_in is not a wakeup condition/ was not the reason of awakening.
* @retval		TRUE_CODE	: an interrupt on IRQ_in is a wakeup condition / was the reason of awakening.
*/
u8 CR95HF_Is_IRQin_Interrupt_Wake_up_Condition(const u8 WUFlags)
{
	if(((WUFlags & CR95HF_IDLE_WUFLAGS_BIT3_EXTERNAL_INTERRUPT_IRQ_IN)>> SHIFT_BIT3) == BIT_SET)
		return TRUE_CODE;
	else
		return FALSE_CODE;
}


/**  
* @brief  This function is related to the idle mode. Its first goal is to determine whether  an interrupt on NSS  is a wakeup condition (For instance if the CR95HF is in idle Mode, this function could also be called to determine which way to use to wake up the CR95HF).It can also be called once the CR95HF has woken up to determine if the CR95HF woke up because of timeout.Note that for the first case the WuFlags issued for the Idle command sould be given as input parameter (available in the CR95HF_Idle_Config structure) and for the second the case afther having issued CR95HF_Rd_Wake_up_Reg the  last_reason_of_wakeup field of the  CR95HF_Idle_Config structure may be used as input parameter  
* @param WUFlags : the byte containing the expected data : either the last_reason_of_wakeup field of the CR95HF_Idle_Config structure or the WuFlags used for the Idle command (also available in the CR95HF_Idle_Config structure
* @retval		FALSE_CODE :  an interrupt on NSS is not a wakeup condition/ was not the reason of awakening.
* @retval		TRUE_CODE	: an interrupt on NSS is a wakeup condition / was the reason of awakening.
*/
u8 CR95HF_Is_NSS_Interrupt_Wake_up_Condition(const u8 WUFlags)
{
	if(((WUFlags & CR95HF_IDLE_WUFLAGS_BIT4_EXTERNAL_INTERRUPT_NSS) >> SHIFT_BIT4) == BIT_SET)
		return TRUE_CODE;
	else
		return FALSE_CODE;
}


/**  
* @brief  This function is related to the idle mode. Its first goal is to determine whether  a tag detection  is a wakeup condition (For instance if the CR95HF is in idle Mode, this function could also be called to determine which way to use to wake up the CR95HF).It can also be called once the CR95HF has woken up to determine if the CR95HF woke up because of timeout.Note that for the first case the WuFlags issued for the Idle command sould be given as input parameter (available in the CR95HF_Idle_Config structure) and for the second the case afther having issued CR95HF_Rd_Wake_up_Reg the  last_reason_of_wakeup field of the  CR95HF_Idle_Config structure may be used as input parameter  
* @param WUFlags : the byte containing the expected data : either the last_reason_of_wakeup field of the CR95HF_Idle_Config structure or the WuFlags used for the Idle command (also available in the CR95HF_Idle_Config structure
* @retval		FALSE_CODE :  a tag detection is not a wakeup condition/ was not the reason of awakening.
* @retval		TRUE_CODE	: a tag detection is a wakeup condition / was the reason of awakening.
*/
u8 CR95HF_Is_Tag_Detected_Wake_up_Condition(const u8 WUFlags)
{

	if(((WUFlags & CR95HF_IDLE_WUFLAGS_BIT1_TAG_DETECTED) >> SHIFT_BIT1) == BIT_SET)
		return TRUE_CODE;
	else
		return FALSE_CODE;
}


/**  
* @brief  This function is related to the idle mode. Its goal is to determine whether the internal timer is a wakeup condition (For instance if the CR95HF is in idle Mode, this fucntion could be called to determine which way to use to wake up the CR95HF).It can also be called once the CR95HF has woken up to determine if the CR95HF woke up because of timeout.Note that for the first case the WuFlags issued for the Idle command sould be given as input parameter (available in the CR95HF_Idle_Config structure) and for the second the case afther having issued CR95HF_Rd_Wake_up_Reg the  last_reason_of_wakeup field of the  CR95HF_Idle_Config structure may be used as input parameter  
 @param WUFlags : the byte containing the expected data : either the last_reason_of_wakeup field of the CR95HF_Idle_Config structure or the WuFlags used for the Idle command (also available in the CR95HF_Idle_Config structure
* @retval		FALSE_CODE : a timeout  is not a wakeup condition/ was not the reason of awakening.
* @retval		TRUE_CODE	: a timeout is a wakeup condition / was the reason of awakening.
*/
u8 CR95HF_Is_Timeout_Wake_up_Condition(const u8 WUFlags)
{
	if( (WUFlags & CR95HF_IDLE_WUFLAGS_BIT0_TIMER) == BIT_SET)
		return TRUE_CODE;
	else
		return FALSE_CODE;
}

// End of CR95HF_Is_Functions 
	
	// End of CR95HF_Supports_Functions

 // @addtogroup CR95HF_Procedures
	

/**  
* @brief  The purpose of this function is to perform the detector calibration according to the Application Note :  
* @param Protocol_Settings : pointer on the structure to update the protocol select. (Turn RF field off)
* @param Idle_Config : pointer on the structure to write the High compare value and low compare value for tag detection and the last wake up reason
* @retval	 CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed
*/
u8 CR95HF_Idle_Detector_Calibration(CR95HF_Protocol_Config* Protocol_Settings,CR95HF_Idle_Config* Idle_Config)
{
	u8 status=CR95HF_ERROR_CODE; //To ensure procedure success
	u8 loop_cnt=CR95HF_DETECTOR_CALIBRATION_LOOP_CNT_INIT;// counter for the research loop
	u8 divider=0x02;// divider for calculation
	u8 reference=CR95HF_DETECTOR_CALIBRATION_REF_0;//To store the value computed of the reference field (dataHigh)
	u8 wakeup_flags_n_1;// To store the n-1 wake up flag
	u8 field_reference=INIT_VALUE;
	// Initialization
	status=CR95HF_Echo();// Send Echo command before any other command
	if(status==CR95HF_SUCCESS_CODE)// turn RF Field off
		status= CR95HF_FieldOff(Protocol_Settings);
		
	if(status==CR95HF_SUCCESS_CODE)
	{
		//Perform the procedure
		status=CR95HF_ERROR_CODE; // Re initialize command status
		// Send first Idle Command Data_L=0 Data_H=0 ->tag detection exit
		if(CR95HF_Tagcal( CR95HF_DETECTOR_CALIBRATION_DACDATA_L,CR95HF_DETECTOR_CALIBRATION_DACDATA_H_MIN)==CR95HF_SUCCESS_CODE)
		{
			// Wait for CR95HF awakening and retrieve the cause
			// if a lower frequency is used for the oscillator the number of 250ms waiting period may be increased
			status=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_Response,Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
			if( status==CR95HF_SUCCESS_CODE && CR95HF_Is_Tag_Detected_Wake_up_Condition(Idle_Config->last_reason_of_wakeup)==TRUE_CODE)
			{
				// First idle succeeded
				// Now try with  Data_L=0 Data_H=3F ->timeout exit
				if(CR95HF_Tagcal( CR95HF_DETECTOR_CALIBRATION_DACDATA_L,CR95HF_DETECTOR_CALIBRATION_DACDATA_H_MAX)==CR95HF_SUCCESS_CODE)
				{
					// Wait for CR95HF awakening and retrieve the cause
					status=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_Response,Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
					// put the reason into wake up flag ( wup in the application note)
					wakeup_flags_n_1=(Idle_Config->last_reason_of_wakeup&CR95HF_IDLE_MASK_WAKE_UP_FLAGS);
					// check that timeout was the reason of waking up
					if( status==CR95HF_SUCCESS_CODE && CR95HF_Is_Timeout_Wake_up_Condition(Idle_Config->last_reason_of_wakeup)==TRUE_CODE)
					{
						// Research loop : dichotomist algortihm
						// error are stored in status and checked in the loop test
						// Each function has a timeout to avoid getting stuck
						while(loop_cnt<=CR95HF_DETECTOR_CALIBRATION_LOOP_CNT_END && status==CR95HF_SUCCESS_CODE)
						{
							/* Given the result of previous waking up reason
							Adjust the value of the field if tag detection exits idle mode then it means 
							that reference is too low. THe algorithm uses a dichotmist reasoning 
							starting at the maximum value and lowing down the value until a tag detection 
							exists idle mode and then  
							*/
							if(wakeup_flags_n_1==CR95HF_WAKE_UP_BECAUSE_TIMEOUT)
								reference=reference-(64/divider);
								
							else
								reference=reference+(64/divider);
								
							//Update divider by multiplying itself per 2
							divider=divider<<SHIFT_BIT1;
							// Send an idle command with the new reference
							status=CR95HF_Tagcal(CR95HF_DETECTOR_CALIBRATION_DACDATA_L, reference);
							// Wait for CR95HF awakening and retrieve the cause
							status=CR95HF_Wait_Wake_up_from_Idle(&CR95HF_Response,Idle_Config,CR95HF_NUMBER_OF_WAITING_PERIOD);
							// update wake up flag for next reference calculation value
							wakeup_flags_n_1=(Idle_Config->last_reason_of_wakeup&CR95HF_IDLE_MASK_WAKE_UP_FLAGS);
							loop_cnt++;// Increment counter for looping
						}
						if(status==CR95HF_SUCCESS_CODE)
						{
							field_reference=reference + wakeup_flags_n_1 - 2;// Compute field reference using reference
					//update values used and computed during the calibration
							Idle_Config->wuFlags = CR95HF_IDLE_WUFLAGS_BIT0_TIMER | CR95HF_IDLE_WUFLAGS_BIT1_TAG_DETECTED;// used during calibration
							// Store dacdata low and dacdata high : note that data are stored on the sixth MSB bits
							if(field_reference>2)
								Idle_Config->dacDataL= (field_reference-2)<<SHIFT_BIT2; //  set  upper margin
							else
								Idle_Config->dacDataL=INIT_VALUE;
							Idle_Config->dacDataH= (field_reference+2)<<SHIFT_BIT2; // set lower margin
							// End of calibration	
						}
					}
					else // Procedure failed 
						status=CR95HF_ERROR_CODE;
				}
			}
			else
				status=CR95HF_ERROR_CODE;
		}
	}
	return status;
}



/**  
* @brief  The purpose of this function is to perform the detector calibration according to the Application Note : AN3433 Tag Detection Calibration Process
* @param void : no paramater expected
* @retval	 CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed
*/
static u8 CR95HF_Tagcal(const u8 DacDataL,const  u8 DacDataH)
{
	u8 WuFlags= (CR95HF_IDLE_WUFLAGS_BIT0_TIMER | CR95HF_IDLE_WUFLAGS_BIT1_TAG_DETECTED);
	CR95HF_Idle_Config dummy_Idle_Config;// Dummy Idle Config for idle Command
	// Shift the data to the 6MSB bits because DacData High and Dac Data low are stored into the sixth MSB bits.
	u8 dacDataL=(DacDataL<<SHIFT_BIT2);
	u8 dacDataH=((DacDataH<<SHIFT_BIT2));
	// Send an idle command with parameters defined for calibration and two variables  to test
	// Note that only DacDataH is used for calibration, DacDataL remains at 0
	return CR95HF_Idle (WuFlags, CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L, CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H,
	CR95HF_DETECTOR_CALIBRATION_WUCtrl_L, CR95HF_DETECTOR_CALIBRATION_WUCtrl_H,
	CR95HF_DETECTOR_CALIBRATION_LeaveCtrl_L, CR95HF_DETECTOR_CALIBRATION_LeaveCtrl_H ,
	CR95HF_DETECTOR_CALIBRATION_WUPeriod, CR95HF_IDLE_OSC_START, CR95HF_IDLE_DAC_START,
  dacDataL,  dacDataH, 
	CR95HF_IDLE_SWING_CNT,CR95HF_DETECTOR_CALIBRATION_Max_Sleep, &dummy_Idle_Config );
	// Idle config is updated at the end of the calibration process
}

// End of CR95HF_Procedures
	
 // @addtogroup CR95HF_Advanced_Commands
	

	
/**  
* @brief  The purpose of this function is to perform the detector calibration according to the Application Note :  AN3433 Tag Detection Calibration Process
* @param DacDataL : Lower comparing value for tag detection
* @param DacDataH : Higher comparing value for tag detection
* @retval	 CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed
*/
 u8 CR95HF_Tagdet(const u8 DacDataL,const  u8 DacDataH)
{
	u8 WuFlags= CR95HF_IDLE_WUFLAGS_BIT0_TIMER | CR95HF_IDLE_WUFLAGS_BIT1_TAG_DETECTED;
	CR95HF_Idle_Config dummy_Idle_Config;// Dummy Idle Config for idle Command
	u8 status=CR95HF_ERROR_CODE;
	// Send an idle command with parameters defined for calibration and two variables  to test
	// Note that only DacDataH is used for calibration, DacDataL remains at 0
	status= CR95HF_Idle (WuFlags, CR95HF_DETECTOR_CALIBRATION_EnterCtrl_L, CR95HF_DETECTOR_CALIBRATION_EnterCtrl_H,
	CR95HF_DETECTOR_CALIBRATION_WUCtrl_L, CR95HF_DETECTOR_CALIBRATION_WUCtrl_H,
	CR95HF_DETECTOR_CALIBRATION_LeaveCtrl_L, CR95HF_DETECTOR_CALIBRATION_LeaveCtrl_H ,
	CR95HF_DETECTOR_CALIBRATION_WUPeriod, CR95HF_IDLE_OSC_START, CR95HF_IDLE_DAC_START,
  DacDataL,  DacDataH, 
	CR95HF_IDLE_SWING_CNT, INIT_VALUE, &dummy_Idle_Config );
	// Idle config is updated at the end of the calibartion process
	return status;
}


/**
  * @brief The purpose of this function is to wait for CR95HF exiting the  idle mode and retrieve what caused the wake-up. The method proposed is to wait for the CR95HF answer and then to send several echo command in order to resume communication and clear the fifo stack. Note that this function waits for CR95HF wakes up on its own (internal timer, tag detection), after this function  the CR95HF_Wake_up_CR95HF fucntion may be called  to wake up the CR95HF with NNS or IRQ_in interrupt.
  * @param CR95HF_Response_Idle :  pointer to store the response 
  * @param Idle_Config :  pointer on thr structure to update with the new last reason of wake up
  * @param Number_of_DELAY_BEFORE_TIMEOUT_250MS :  number of period of 250ms duration to wait before aborting the procedure.
  * @retval EXIT_COMMUNICATION_OK (CR95HF_SUCCESS_CODE): communication is possible 
  * @retval EXIT_COMMUNICATION_TIMEOUT : communication is not possible 
  */
u8 CR95HF_Wait_Wake_up_from_Idle(CR95HF_ResponseReceiving* CR95HF_Response_Idle,CR95HF_Idle_Config* Idle_Config, u8 Number_of_DELAY_BEFORE_TIMEOUT_250MS)
{
	u8 status;
	u8 cr95hf_polling=EXIT_COMMUNICATION_TIMEOUT;
	/* repeat the polling period during Number_of_DELAY_BEFORE_TIMEOUT_250MSx250 ms
		if another oscillator frequency  than 32khz is used (lower), the delay of processs increases (for example for the tag calibration) then it may be necessary to wait more time to let the CR95HF.
		It is also useful if the CR95HF may stay asleep for a long period.
		If the MCU has other task to process or is turned into low power consumption mode too, an interruption on the IRQ_in pin can be used (see user application )
	*/
	while (Number_of_DELAY_BEFORE_TIMEOUT_250MS!=INIT_VALUE && cr95hf_polling!=EXIT_COMMUNICATION_OK)
	{
		cr95hf_polling=CR95HF_Wait_Answer(DELAY_BEFORE_TIMEOUT_250MS);
		// Check if the CR95HF woke up and is ready to transmit its answer 
		if(cr95hf_polling==EXIT_COMMUNICATION_OK)
		{
			// Receive the answer
			*CR95HF_Response_Idle= CR95HF_ReceiveAnswer();
			// Check the response code of the CR95HF 
			status=CR95HF_IsReaderResultOK(CR95HF_Response_Idle,CR95HF_CMD_IDLE_SUCCESS_CODE);
			// Update the Idle Config structure
			if (CR95HF_Response_Idle->data_length>DATALENGTH_NULL)
			{
				// if there is one data then the version is 1.1
				Idle_Config->last_reason_of_wakeup=CR95HF_Response_Idle->data[0];
				// Note that the frequency of the oscillator used during the Idle mode is present on the two MSB bits. 
			}
			else // Datalength = 0 // else the version is 1.0 there is no data
			{
				// Read reason of wake up
				status= CR95HF_Rd_Wake_up_Reg(CR95HF_Response_Idle, Idle_Config);
			}
		}
			
			Number_of_DELAY_BEFORE_TIMEOUT_250MS--;// decrement 
	}
	if(status!=EXIT_COMMUNICATION_OK)
		status=EXIT_COMMUNICATION_TIMEOUT;
	return status;		
}


/**
  * @brief The purpose of this function is to avoid getting stuck when the CR95HF exists idle mode. The method proposed is to send several echo command in order to resume communication and clear the fifo stack.
  * @param Idle_Config :  contains the wake up condition
  * @retval CR95HF_SUCCESS_CODE: the CR95HF woke up, communication is possible 
  * @retval CR95HF_ERROR_CODE : fail to wake up the CR95HF, either the CR95HF waits for another event (tag detection, timer) or communication cannot be resumed.
  */
u8 CR95HF_Wake_up_CR95HF(CR95HF_Idle_Config* Idle_Config)
{
	// Check each wake-up condition that the STM8L can achieve (Interrupt)
	if(CR95HF_Is_IRQin_Interrupt_Wake_up_Condition(Idle_Config->wuFlags)==TRUE_CODE)
	{
		CR95HF_IRQ_in_Interrupt();
		if(CR95HF_Echo()==CR95HF_SUCCESS_CODE)
			return CR95HF_SUCCESS_CODE;
	}
	
	if(CR95HF_Is_NSS_Interrupt_Wake_up_Condition(Idle_Config->wuFlags)==TRUE_CODE)
	{
		CR95HF_NSS_Interrupt();
		if(CR95HF_Echo()==CR95HF_SUCCESS_CODE)
			return CR95HF_SUCCESS_CODE;
	}
	return CR95HF_ERROR_CODE;		
}


	// End of CR95HF_Advanced_Functions



	
//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
//----------------------- Custom Commands ST-Reserved ------------------------------
//----------------------------------------------------------------------------------
//--------------- Note that no display is done for custom commands ------------------
//----------------------------------------------------------------------------------

#ifdef ST_RESERVED_FUNCTIONS_AVAILABLE

/**  
* @brief  	This function returns the value of the field detection flag. 
* @param void : no requested parameter
* @retval		POLL_FIELD_CMD_FLAG_SET : field detected
* @retval		POLL_FIELD_CMD_FLAG_RESET : no field detected
* @retval		CR95HF_ERROR : the response did not match what was expected
*/
CR95HF_ResponseReceiving CR95HF_Poll_Field(void)
{
	u8 status=CR95HF_ERROR;
	CR95HF_CommandSending cmd;
	CR95HF_ResponseReceiving response;
	
	// Init response
	CR95HF_Init_ResponseReceiving(&response);
	
	// Fill the command parameter
	cmd.cmd_code=CR95HF_CMD_POLL_FIELD;
	cmd.data_length=DATALENGTH_NULL;
	
	response=CR95HF_SendReceive_Cmd(cmd);// Send command and receive response
	// Verify response code and data length, if they do not match 
	if(response.resp_code==CR95HF_POLL_FIELD_CMD_SUCCESS_CODE && response.data_length==CR95HF_POLL_FIELD_CMD_DATALENGTH)
	{	
		status=response.data[0]; // There i only one data on LSB
	}
	return response;

}


/**  
* @brief  This function put thje CR95HF in listen mode, application must select a protocol (different from iso15693) before using this command. 
* @param  Response : pointer on the response 
* @retval	 CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed, further information like error code into the response structure provided.
*/
u8 CR95HF_Listen(CR95HF_ResponseReceiving* Response)
{
	u8 status=CR95HF_ERROR;
	CR95HF_CommandSending cmd;
	
	// Init response
	CR95HF_Init_ResponseReceiving(Response);
	
	// Fill the command parameter
	cmd.cmd_code=CR95HF_CMD_LISTEN;
	cmd.data_length=DATALENGTH_NULL;
	
	*Response=CR95HF_SendReceive_Cmd(cmd);// Send command and receive response
	
	// the command has two success code: one for turning into listening mode
	// and one for data
	// testing success of command (listening mode )
	status=CR95HF_IsReaderResultOK(Response,CR95HF_LISTEN_CMD_SUCCESS_CODE);
	// the error is write in the structure at this step success for data communication is interpreted as an error.
	if(status!= CR95HF_SUCCESS_CODE) // either the command has failed to turn into listening mode or data were received  
		status=CR95HF_IsReaderResultOK(Response,CR95HF_LISTEN_CMD_COMMUNICATION_SUCCESS_CODE);
	//CR95HF_IsReaderResultOK function does not affect the error retrieve it  changes the value only if success code is detected
	return status;
}

/**  
* @brief  This function sends a command to the tag without expecting any answer. 
* @param  Command : the command structure to send
* @retval	 CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_INVALID_PROTOCOL : command failed	invalid protocol		
* @retval	 CR95HF_ERROR_INVALID_CMD_LENGTH : command failed, invalid length
*/
u8 CR95HF_Send( CR95HF_CommandSending* Command)
{
	u8 status =CR95HF_ERROR_CODE;
	CR95HF_ResponseReceiving response;
	CR95HF_Init_ResponseReceiving(&response); // Initialize the structure
	// Add the command code, send the command and receive the response
	Command->cmd_code=CR95HF_CMD_SEND;
		
	response=CR95HF_SendReceive_Cmd(*Command);
	// Check if the command is ok
	status=response.resp_code;
	
	return status;
}





/**  
* @brief  	This function reads the internal registers of the CR95HF. The size of data read should not exceed MAX_DATARECEIVED_LENGTH bytes.(see CR95HF_structure.h for further information. 
* @param Response : pointer on the structure to fill in with the data retrieved
* @param Address : Address of the first register to read
* @param RegCount : number of register to read
* @param Flags : Whether to increment address after register read: 00  do not increment address 01  increment address 
* @retval		CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_INVALID_CMD_LENGTH : command failed, invalid length
*/
u8 CR95HF_Rd_Reg( CR95HF_ResponseReceiving* Response, u8 Address, u8 RegCount, u8  Flags)
{
	u8 status=CR95HF_ERROR_CODE;
	u8 data_counter= INIT_VALUE;
	CR95HF_CommandSending cmd;
	
	// init the response
	CR95HF_Init_ResponseReceiving(Response);
	
	// Fill in the command code and the length
	cmd.cmd_code = CR95HF_CMD_RDREG;
	cmd.data_length = CR95HF_CMD_RD_REG_DATALENGTH;
	cmd.data[data_counter++]=Address;
	cmd.data[data_counter++]=RegCount;
	cmd.data[data_counter]=Flags;
	
	// send the command and put the response into the response structure provided 
	*Response = CR95HF_SendReceive_Cmd(cmd);
	
	// Check if the command is ok
	status=Response->resp_code;
	
	return status;
}

/**  
* @brief  	This function writes data into the internal registers of the CR95HF.  
* @param Reg_Data : pointer on the data to write 
* @param Address : Address of the first register to write data
* @param RegCount : number of register to be written
* @param Flags : Whether to increment address after register read: 00  do not increment address 01  increment address 
* @retval		CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_INVALID_CMD_LENGTH : command failed, invalid length
*/
u8 CR95HF_Wr_Reg( u8* Reg_Data, u8 Address, u8 RegCount, u8  Flags)
{
	u8 status=CR95HF_ERROR_CODE;
	u8 data_counter= INIT_VALUE; // counter for data into the command and command datalength
	CR95HF_CommandSending cmd; // to send command
	CR95HF_ResponseReceiving response; // to receive the CR95HF response
	
	// Init the response
	CR95HF_Init_ResponseReceiving(&response);
	
	// Fill in the command code and parameters
	cmd.cmd_code = CR95HF_CMD_WRT_REG;
	cmd.data[data_counter++]=Address;
	cmd.data[data_counter++]=Flags;
	// Add data
	CR95HF_Add_Data_ToCommand(&cmd, &data_counter, RegCount, Reg_Data);
	
	// Add datalength to command
	cmd.data_length = data_counter;
	// Send the command and put the response into the response structure provided 
	response = CR95HF_SendReceive_Cmd(cmd);
	
	// testing success of command
	status=response.resp_code;
	
	return status;
}

/**  
* @brief  	This function aks to the CR95HF the last the frequency divider of Subcarrier frequency obtained during Felica communication. 
* @param Sub_freq_divider : pointer on the data to write the value returned by the CR95HF
* @retval		CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed, invalid length
*/
u8 CR95HF_SubFreqRes(u8* Sub_freq_divider)
{
	/*Note that the subcarrier frequency fs is given the following formula
	fs= fc/ (2(N+1)) where N is the sub_freq_divider and fc the carrier*/
	
	u8 status=CR95HF_ERROR_CODE;
	CR95HF_CommandSending cmd; // to send command
	CR95HF_ResponseReceiving response; // to receive the CR95HF response
	// Init the response
	CR95HF_Init_ResponseReceiving(&response);
	// Fill in the command code and parameters
	cmd.cmd_code = CR95HF_CMD_SUB_FREQ_RES;
	cmd.data_length = DATALENGTH_NULL;
	
	// Send the command and put the response into the response structure provided 
	response = CR95HF_SendReceive_Cmd(cmd);
	
	// testing success of command
	status=CR95HF_IsReaderResultOK(&response,CR95HF_SUB_FREQ_RES_CMD_SUCCESS_CODE);
	 
	if( status=CR95HF_SUCCESS_CODE && response.data_length==CR95HF_SUB_FREQ_RES_CMD_RESPONSE_DATALENGTH)
	{
		*Sub_freq_divider=response.data[0];
	}
	
	return status;
}


/**  
* @brief  	This function activates/deactivates anti-collifion filter. Note that length can be
 7  for 1-cascade level filter
11  for 2-cascade levels filter
15  for 3-cascade levels filter
All other values will cause Invalid command length error.

* @param Length : length of the command ( parameters )
* @param ATQA : on 2 bytes Unused and proprietary bits of SAK (protocol bits will be handled by FW)
* @param SAK : Unused and proprietary bits of SAK (protocol bits will be handled by FW)
* @param UID part 1 : UID for cascade level 1 (Mandatory)
* @param UID part 2 : UID for cascade level 1 (Optionnal)
* @param UID part 3 : UID for cascade level 1 (Optionnal)
* @retval		CR95HF_SUCCESS_CODE : command succeed
* @retval	 CR95HF_ERROR_CODE : command failed, invalid length
*/
u8 CR95HF_AcFilter(u8 Length, u8* ATQA, u8 SAK, u8* UID_part_1, u8* UID_part_2, u8* UID_part_3 )
{
	u8 status=CR95HF_ERROR_CODE;
	u8 data_counter= INIT_VALUE; // counter for data into the command and command datalength
	CR95HF_CommandSending cmd; // to send command
	CR95HF_ResponseReceiving response; // to receive the CR95HF response
	
	// Init the response
	CR95HF_Init_ResponseReceiving(&response);
	
	// Fill in the command code and parameters
	cmd.cmd_code = CR95HF_CMD_AC_FILTER;
	cmd.data_length = Length;
	// check if there are data to add
	if(Length!=DATALENGTH_NULL)
	{
		// Add ATQA to command
		CR95HF_Add_Data_ToCommand(&cmd, &data_counter, CR95HF_ACFILTER_ATQA_LENGTH, ATQA);
		// Add SAK
		cmd.data[data_counter++]=SAK;
		// Add UID part 1 (mandatory)
		CR95HF_Add_Data_ToCommand(&cmd, &data_counter, CR95HF_ACFILTER_UID_PART_LENGTH, UID_part_1);
	
		// Add UID part_2 if necessary Length= 11
		if(Length>CR95HF_ACFILTER_LENGTH_7)
			CR95HF_Add_Data_ToCommand(&cmd, &data_counter, CR95HF_ACFILTER_UID_PART_LENGTH, UID_part_2);
	
		// Add UID part_2 if necessary Length= 11
		if(Length>CR95HF_ACFILTER_LENGTH_11)
			CR95HF_Add_Data_ToCommand(&cmd, &data_counter, CR95HF_ACFILTER_UID_PART_LENGTH, UID_part_3);	
	}
	// Send the command and put the response into the response structure provided 
	response = CR95HF_SendReceive_Cmd(cmd);
	
	// testing success of command
	status=CR95HF_IsReaderResultOK(&response,CR95HF_ACFILTER_CMD_SUCCESS_CODE);
	
	return status;
}
#endif

	// End of CR95HF
	
