/**
  **********************************************************
  * @file lrixk_command.c
  * @brief This file contains all the functions in order to send commands to a contactless tag of the LRIXK family performing iso15693 standards. Note that command dealing with multiple blocks (write read get status) utilize internal eeprom of the MCU whereas other commands use flash program memory.
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Includes ----------------------------------------------
#include "lrixk_command.h"


// this structure is retrieved so that the functions could access the protocol settings used by the CR95HF (used to test configuration :request flags and respect312s delay or wait SOF parameter,  for Lock fucntions for instance and to check data rate for fast commands.
extern CR95HF_Protocol_Config Protocol_Settings;/*!< Protocol and parameters related used*/

// Those two structure are retrieved from the iso15693_command.c file to share the structures in order to save memory space for code
extern CR95HF_CommandSending Cmd;/*!< Common Stucture used  to transmit data within commands*/

extern CR95HF_ResponseReceiving Response;/*!< Common Stucture used  to receive data within commands*/

// Main Architecture of the doxyfile for this file 
 // @addtogroup  Contactless_Tags
	
	// * @defgroup LRIXK_Tags
	
			// * @defgroup  LRIXK_Commands


	
			// * @defgroup  LRIXK_Support_Functions

 // @addtogroup LRIXK_Tags
 // @addtogroup LRIXK_Commands
/**  
* @brief  	This function sends, through the CR95HF, to a contactless tag posssibly of the LRIXK family, an inventory command and fills the Tag structure if some contactless tag answers.
* @param  	MyTag		:	pointer on the structure
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.Here inventory flag should be set, if AFI is present, AFI flag should be set.
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless  tag or the tag did not manage to perform the request.
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : The inventory flag is not set, the mask size overweight the limited size or there is an other eoor into the request flags parameters
* @retval  LRIXK_SUCCESS_CODE : A contactless tag answered the command.
*/
u8 LRIXK_Inventory(ISO15693_Tag* MyTag,const u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flagto 0
	request_flags &= ~LRIXK_MASK_REQUEST_FLAGS_NO_OPTION;
	// Force the inventory flag to 1
	request_flags |= ISO15693_BIT3_INVENTORY_FLAG_SET;
	// Call the ISO15693 command
	return ISO15693_Inventory(MyTag, request_flags,  Tag_AFI, 
 MaskLength, Mask, Inventory_16_slots_Nb_UID_Retrieved);

}


/**  
* @brief  	This function sends, through the CR95HF, a stay quiet command to a unique contactless tag possibly belonging to the LRIXK family designated by its UID.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to put in quiet mode.
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless tag or the tag did not manage to perform the request.
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : The inventory flag is set or request is not well addressesed
* @retval  LRIXK_SUCCESS_CODE : contactless tag is supposed to be in quiet mode.
*/
u8 LRIXK_Stay_Quiet(const u8 Request_flags,const  u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option, the inventory, the select and the protocol extension flags to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OPTION_SELECT;
	// Force the adressed flag
	request_flags |= ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED;

	// Call the ISO15693 command
	return  ISO15693_Stay_Quiet(request_flags, Tag_UID);
	
}


/**  
* @brief  	This function sends, through the CR95HF, a read single block command to the contactless tag possibly of the LRIXK family in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer of the structure to write data read
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Block_number	:	address of the block to read.
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered the command.
*/
u8 LRIXK_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	
	// Call the ISO15693 command
	return  ISO15693_Read_Single_Block(MyTag, request_flags, Tag_UID, Block_number);

}


/**  
* @brief  	This function sends, through the CR95HF, a write single block command to the contactless tag opssibly from the LRIXK family in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Block_number	:	address of the block to write.
* @param  	Data	:	data to write.
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct)
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Write_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number, const u8* Data)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	// Call the ISO15693 command
	return ISO15693_Write_Single_Block(MyTag, request_flags, Tag_UID, Block_number, LRIXK_DEFAULT_BLOCK_SIZE, Data);
}

/**  
* @brief  	This function sends, through the CR95HF, a lock block command to the contactless tag in the field or to an unique contactless tag designated by its UID. 
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag where the data should be written (optionnal).
* @param  	Block_number	:	address of the block where the data are written.
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/

u8 LRIXK_Lock_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID,const  u8 Block_number)
{
	// Note that Option Flag is set, the LRIXK may lock the block and returns that the block has already been locked. 
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag, the inventory and the protocol extension flags to 0
	request_flags&=~LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OR_OPTION;
	// Call the ISO15693 command
	return ISO15693_Lock_Block( MyTag,request_flags,  Tag_UID, Block_number);
}


/**  
* @brief  	This function sends, through the CR95HF, a read multiple block command to the contactless tag possibly belonging to the LRIXK family  in the field or to an unique contactless tag designated by its UID. Note that data received are stored into the internal EEPROM of the MCU.
* @param  	MyTag		:	pointer of the structure to write the command status and eventual error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters   
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered the command.
*/

u8 LRIXK_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag, the inventory and the protocol extension flags to 0
	request_flags&=~LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OR_OPTION;
	// Call the ISO15693 command
	return ISO15693_Read_Multiple_Block(MyTag, request_flags,  Tag_UID, First_Block_number,  Number_of_Blocks);
}


/**  
* @brief  	This function sends, through the CR95HF, a select command to the unique contactless tag designated by its UID possibly belonging to the LRIXK family.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		LRIXK_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Select(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option, the inventory, the selectand the protocol extension flags to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OPTION_SELECT;
	// Force the adressed flag
	request_flags |= ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED;
	
	// Call the ISO15693 command
		return ISO15693_Select( MyTag,request_flags,Tag_UID);
}
	
	
/**  
* @brief  	This function sends, through the CR95HF, a reset to ready state command to a contactless tag possibly belong to the LRIXK family. This command could be used in two different case: 
- thecontactless tag is in quiet mode, it returns into ready mode (request could be addressed or not),
- the contactless tag is in select mode it returns into ready mode (select flag shall be set). Note that for this case another select command with another UID turn automatically the tag into ready mode.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		LRIXK_ERROR_CODE		:the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Reset_to_Ready(ISO15693_Tag* MyTag,const u8 Request_flags, const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option, the inventory flag and the protocol extension flagto 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OR_OPTION;
	
	// Call the ISO15693 command
	return ISO15693_Reset_to_Ready(MyTag, request_flags, Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, a write AFI command to the contacless tag in the field possibly belonging to the  LRIXK family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Tag_AFI	:	AFI of the contacless tag, refer to datasheet for further information.
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  ILRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/

u8 LRIXK_Write_AFI(ISO15693_Tag* MyTag, const u8 Request_flags,const  u8* Tag_UID,const  u8 Tag_AFI)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	// Call the ISO15693 command
	return ISO15693_Write_AFI( MyTag, request_flags,  Tag_UID,  Tag_AFI);
}


/**  
* @brief  	This function sends, through the CR95HF, a Lock AFI command to the contacless tag in the field possibly from the LRIXK family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Lock_AFI(ISO15693_Tag* MyTag, const u8 Request_flags,const  u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	// Call the ISO15693 command
	return ISO15693_Lock_AFI( MyTag,request_flags,  Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, a write DSFID command to the contacless tag in the field possibly belonging to the  LRIXK family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Tag_DSFID	:	DSFID of the contacless tag, refer to datasheet for further information.
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Write_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags,const  u8* Tag_UID,const  u8 Tag_DSFID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	// Call the ISO15693 command
	return  ISO15693_Write_DSFID(MyTag, request_flags,  Tag_UID,  Tag_DSFID);
}


/**  
* @brief  	This function sends, through the CR95HF, a Lock DSFID command to the contacless tag in the field possibly from the LRIXK family or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/

u8 LRIXK_Lock_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	// Call the ISO15693 command
	return ISO15693_Lock_DSFID(MyTag, request_flags, Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, an get System Information command to the contacless tag ,possibly from the LRIXK family, in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Get_System_Info(ISO15693_Tag* MyTag, const u8 Request_flags,const u8* Tag_UID)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option, inventory and protocol extension flags to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OR_OPTION;
	
	// Call the ISO15693 command
	return ISO15693_Get_System_Info( MyTag, request_flags, Tag_UID);
}


/**  
* @brief  	This function sends, through the CR95HF, a get multiple blocks security status command to the contactless tag in the field,possibly from the LRIXK family, or to an unique contactless tag designated by its UID. note that data are stored into the eeprom.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	adress of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Get_Multiple_Blocks_Security_Status(ISO15693_Tag* MyTag,const  u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks)
{
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option, inventory and protocol extension flags to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY_OR_OPTION;
	// Call the ISO15693 command
	return ISO15693_Get_Multiple_Blocks_Security_Status(MyTag, request_flags,  Tag_UID,  First_Block_number, Number_of_Blocks);

}


//--------------------------------------------------------------------------//
// -----------------     Custom LRIXK commands -----------------------------//

/* Note that for Fast command a previous Fast initiate command is necessary so that the tag could answer in the 56kbits/s data rate.
	Note that a protocol select command is necessary so that the CR95HF could detect the answer received with the 56kbits/s data rate.
*/

/**  
* @brief  	This function sends, through the CR95HF, a kill command to the unique contactless tag designated by its UID and belonging to the LRIXK family. On receiving the kill command the contactless tag compares the kill code added to command and if it matches with the kill code previously written into the code. The contactless answers to the command before being deactivated
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to kill.
* @param  	Kill_Code	:	kill code to copare with the one insode the contactless tag in order to access and perform  the kill command
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.d.
*/
u8 LRIXK_Kill(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8* Kill_Code)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	// Force the adressed flag
	request_flags |= ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED;
	// Check the request_flags	
	if(LRIXK_Is_Request_Flags_ok(Request_flags, LRIXK_CMD_KILL)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,LRIXK_CMD_KILL);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		// Add the kill access 
		Cmd.data[cptrdatalength++]=LRIXK_KILL_ACCESS;
		// Add the kill code provided
		CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,LRIXK_DEFAULT_BLOCK_SIZE, Kill_Code);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a kill command to a LRIXK contactless tag in the field or designated by its UID . The kill code provided is transmitted to the contactless tag and will be required to deactivate the LRIXK product. Note that a Lock kill command is necessary after this command to protect the kill code.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to write the kill code if  adressed flag is set within the request flags.
* @param  	Kill_Code	:	kill code to copare with the one insode the contactless tag in order to access and perform  the kill command
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Write_Kill(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8* Kill_Code)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	
	// Force the inventory flag and the protocol extension flagto 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;
	
	// Check the request_flags	
	if(LRIXK_Is_Request_Flags_ok(Request_flags, LRIXK_CMD_WRITE_KILL)==FALSE_CODE )
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,LRIXK_CMD_WRITE_KILL);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		// Add the kill access
		Cmd.data[cptrdatalength++]=LRIXK_KILL_ACCESS;
		// Add the kill code provided
		CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,LRIXK_DEFAULT_BLOCK_SIZE, Kill_Code);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a lock kill command to a LRIXK contactless tag in the field or designated by its UID . The kill code written previously is locked by the  LRIXK contactless tag and once done the kill code cannot be changed.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to lock the kill code if  adressed flag is set within the request flags.
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Lock_Kill(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;	
	// Force the RFU flag 
	request_flags|= LRIXK_BIT8_LOCK_KILL_RFU_FLAG;
	if(LRIXK_Is_Request_Flags_ok(Request_flags, LRIXK_CMD_LOCK_KILL)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,LRIXK_CMD_LOCK_KILL);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, request_flags);
		// Add the kill access
		Cmd.data[cptrdatalength++]=LRIXK_KILL_ACCESS;
		// Add the protect status
		Cmd.data[cptrdatalength++]=LRIXK_KILL_PROTECT_STATUS;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a read single block command to the LIRXK contactless tag in the field or to an unique contactless tag designated by its UID. the particularity of this command is the datatrate used by the contactless tag to answer which is 53kbits/s. 
* @param  	MyTag		:	pointer of the structure to write data read
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Block_number	:	adress of the block to read.
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Fast_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;	
	//Check the request flags
	// the command has to be issued with 52kbits/s parameters for the CR95HF protocol select parameter and high datarate for the request flags
	if(LRIXK_Is_Request_Flags_ok(request_flags,LRIXK_CMD_FAST_READ_SINGLE_BLOCK)==FALSE_CODE||LRIXK_Is_Fast_Datarate_ok(request_flags,&Protocol_Settings)!= TRUE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,LRIXK_CMD_FAST_READ_SINGLE_BLOCK);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE;
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number
		Cmd.data[cptrdatalength++]=Block_number;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Single_Block_cmd( MyTag,&Response, status_cr95hf, Request_flags );
	
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a fast inventory command to the LRIXK contactless tags in the field. The datarate of the response is multiplied by 2 (here 52kbits/s). Note that a fast initiate command should be issued before calling this function.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.Here inventory flag should be set, if AFI is present, AFI flag should be set.
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Fast_Inventory_Initiated(ISO15693_Tag* MyTag,const  u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// Force the option flag and the protocol extension flagto 0
	request_flags &= ~LRIXK_MASK_REQUEST_FLAGS_NO_OPTION;
	// Force the inventory flag to 1
	request_flags |= ISO15693_BIT3_INVENTORY_FLAG_SET;
	
	//Init Inventory_16_slots_Nb_UID_Retrieved
	*Inventory_16_slots_Nb_UID_Retrieved==INIT_VALUE;
	
	// Check the request_flags	
	if(LRIXK_Is_Request_Flags_ok(Request_flags, LRIXK_CMD_FAST_INVENTORY_INITIATED)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,LRIXK_CMD_FAST_INVENTORY_INITIATED);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		if(ISO15693_Get_AFI_Flag(Request_flags)==ISO15693_AFI_PRESENT)
			Cmd.data[cptrdatalength++]=Tag_AFI;
		
		// Add mask 
		ISO15693_Add_Mask_to_command(&Cmd, &cptrdatalength,MaskLength, Mask);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		status=ISO15693_Send_Receive_Inventory_cmd(MyTag, &Response,&Cmd, Request_flags,&status_cr95hf,   Inventory_16_slots_Nb_UID_Retrieved);
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a fast initiate command to a LRIXK contactless tag in the field or designated by its UID . On receiving the fast initiate command the LRIXK contactless tag set the internal initiate flag ( to send a inventory initited command) and returns its UID and its DSFID, with a datarate multiplied by two (here 52kbits/s). Note that the command should not be in adressed or selected mode, the flag is reset afther a power off.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here neither the addressed or select flag should be set. 
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Fast_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	// Check the request_flags	
	// This command has to be issued in neither adressed or select mode
	// force the option, inventory protocol address and select flas to 0 
	request_flags&=~LRIXK_MASK_REQUEST_FLAGS_INITIATE;
	// the datarate of the response is mulitplied by 2, check data rate 
	if((LRIXK_Is_Request_Flags_ok(request_flags, LRIXK_CMD_FAST_INITIATE)==FALSE_CODE)
	||(LRIXK_Is_Fast_Datarate_ok(request_flags,&Protocol_Settings)!= TRUE_CODE))
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,LRIXK_CMD_FAST_INITIATE);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
	status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);// Send command and receive response
		// Fill the contactless tag's structure, the response has the same format as the inventory command
			status = ISO156693_Fill_in_TagStructure_Inventory_cmd(MyTag, &Response, status_cr95hf);
			
	}
	return status;
}

/**  
* @brief  	This function sends, through the CR95HF, a read multiple block command a fast read multiple blocks command to a LRIXK contactless tag in the field or designated by its UID . Note that data received are stored into the internal EEPROM of the MCU.
* @param  	MyTag		:	pointer of the structure to write the command status and eventual error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Fast_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;

	//Check the request flags
	// Force the inventory flag and the protocol extension flag to 0
	request_flags &=~ LRIXK_MASK_REQUEST_FLAGS_NO_INVENTORY;	
	
	// high datarate must be selected
	if((LRIXK_Is_Request_Flags_ok(request_flags,LRIXK_CMD_FAST_READ_MULTIPLE_BLOCKS))==FALSE_CODE || (Number_of_Blocks>ISO15693_MAX_SIZE_BLOCK_READ_CMD)
	||(LRIXK_Is_Fast_Datarate_ok(Request_flags,&Protocol_Settings)!= TRUE_CODE))
	{
		// if the inventory flag is  set the command is not sent. This flag involving different meaning for other bits, it could not be forced to 1 by the function.
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters and add the send receive code
		Cmd.cmd_code=CR95HF_CMD_SEND_REC;
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,LRIXK_CMD_FAST_READ_MULTIPLE_BLOCKS);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE;
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in adressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add first block number
		Cmd.data[cptrdatalength++]=First_Block_number;
		
		// Add number of blocks to read after the first one
		Cmd.data[cptrdatalength++]=Number_of_Blocks;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command 
		CR95HF_SendCommand(&Cmd);
		// Receive the answer
		status_cr95hf=ISO15693_Receive_Multiple_Blocks_cmd(&Response);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd( MyTag,&Response, status_cr95hf,Number_of_Blocks );
			
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, an inventory initiated command to the LRIXK contactless tags in the field. On receiving the initiate command the LRIXK contactless tag set the internal initiate flag ( to send a inventory initited command) and returns its UID and its DSFID. Note that a initiate command should be issued before calling this function.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.Here inventory flag should be set, if AFI is present, AFI flag should be set.
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Inventory_Initiated(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI, const u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 request_flags=Request_flags;// create a buffer in order to force some flags
	
	// Force the option flag and the protocol extension flag to 0
	request_flags &= ~LRIXK_MASK_REQUEST_FLAGS_NO_OPTION;
	// Force the inventory flag to 1
	request_flags |= ISO15693_BIT3_INVENTORY_FLAG_SET;
	
	//Init Inventory_16_slots_Nb_UID_Retrieved
	*Inventory_16_slots_Nb_UID_Retrieved==INIT_VALUE;
	
	// Check the request_flags	
	if(LRIXK_Is_Request_Flags_ok(Request_flags, LRIXK_CMD_INVENTORY_INITIATED)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,LRIXK_CMD_INVENTORY_INITIATED);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		if(ISO15693_Get_AFI_Flag(Request_flags)==ISO15693_AFI_PRESENT)
			Cmd.data[cptrdatalength++]=Tag_AFI;
		
		// Add mask 
		ISO15693_Add_Mask_to_command(&Cmd, &cptrdatalength,MaskLength, Mask);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		status=ISO15693_Send_Receive_Inventory_cmd(MyTag, &Response,&Cmd, Request_flags,&status_cr95hf,   Inventory_16_slots_Nb_UID_Retrieved);
		
	}
	return status;
}




/**  
* @brief  	This function sends, through the CR95HF, an initiate command to a LRIXK contactless tag in the field or designated by its UID . On receiving the initiate command the LRIXK contactless tag set the internal initiate flag ( to send a inventory initited command) and returns its UID and its DSFID. Note that the command should not be in addressed or selected mode, the flag is reset afther a power off.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here neither the addressed or select flag should be set. 
* @retval		LRIXK_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  LRIXK_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 LRIXK_Initiate(ISO15693_Tag* MyTag, const u8 Request_flags)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 request_flags=Request_flags;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	// Check the request_flags	
	// This command has to be issued in neither adressed or select mode
	// force the inventory, address select and protocol flags to 0
	request_flags&=~LRIXK_MASK_REQUEST_FLAGS_INITIATE;
	if(LRIXK_Is_Request_Flags_ok(request_flags, LRIXK_CMD_INITIATE)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=LRIXK_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, request_flags ,LRIXK_CMD_INITIATE);
		// add the IC manufacturer
		Cmd.data[cptrdatalength++]=ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE	;
		// Add UID of the contactless tag :  request is in adressed mode
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
	status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);// Send command and receive response
		// Fill the contactless tag's structure the response has the same format as the inventory command
		status = ISO156693_Fill_in_TagStructure_Inventory_cmd(MyTag, &Response, status_cr95hf);
			
	}
	return status;
}


// End of commands group

//------------------------------------------------------------------------------------
// Support functions -----------------------------------------------------------------

 // @addtogroup LRIXK_Support_Functions

 // @defgroup LRIXK_Is_Functions


/**  
* @brief  	This function performs several checks to verify that the request flag is not wrong for the LRIXK family contactless tags.  
* @param  	Request_flags	:	First byte of RF command sent to contacless tag
* @param  	LRIXK_cmd_code	:	the code of the command to send to the contactless tag
* @retval		FALSE_CODE		: An error has been found out within the request flags
* @retval		TRUE_CODE		: There is no error seen within the request flags
*/
u8 LRIXK_Is_Request_Flags_ok(const u8 Request_flags, const u8 LRIXK_cmd_code)
{
	u8 status= FALSE_CODE;
	u8 test;// to store some values used for comparing value
	
	test=ISO15693_Get_DataRate_parameter_CR95HF(&Protocol_Settings);
	// Check datarate and sub carrier if the command is a fast command datarate and requestflags matching is test into each fast command
	if((test==CR95HF_DATARATE_26KBITS && ISO15693_Get_DataRate_Flag(Request_flags)==ISO15693_HIGH_DATARATE)||
		(test==CR95HF_DATARATE_6KBITS && ISO15693_Get_DataRate_Flag(Request_flags)==ISO15693_LOW_DATARATE)||
		LRIXK_Is_Fast_Command(LRIXK_cmd_code)==TRUE_CODE)
	{
		// datarates are matching, test subcarrier
		// note that the constants have the same value
		if(ISO15693_Get_SubCarrier_parameter_CR95HF(&Protocol_Settings)==	
			  ISO15693_Get_Subcarrier_Flag(Request_flags))
		{
			// to test address and select flags
				test=ISO15693_Get_Select_Flag(Request_flags)&ISO15693_Get_Address_Flag(Request_flags);
				if(test!=BIT_SET)
				{
					if(LRIXK_Is_Memory_programming_function(LRIXK_cmd_code)==TRUE_CODE)
					{
						// carry out further tests to forbidd one configuration
						/*If CR95HF protocol select is set with wait312s, the CR95HF sends its 
						command and if no contactless tag has answered within the 312s, the CR95HF 
						returns no Tag or timeout error, if the option flag is not set that means 
						that the contactless is supposed to answer within those delay. 
						But for memory programmation command, operations could last several ms to 
						be processed so that only error (contactless tag answers immediately 
						if there is some error) can be retrieved.
						If option flag is set (write and lock command) 
						the contactless tag waits for an EOF to provide its response,
						consequently the response could be retrieved.*/
						test=ISO15693_Get_Delay_parameter_CR95HF(&Protocol_Settings);
						if( test!=CR95HF_RESPECT312_DELAY ||  
						ISO15693_Get_Option_Flag(Request_flags)!=ISO15693_OPTION_FLAG_RESET)
						{
							status=TRUE_CODE;
						}
						// else status remains at its initial value: FALSE_CODE
					}	
					else // for others command the contactless is supposed to answer as soon as it performed the command
					{
						// others commands, no further test
						status=TRUE_CODE;
					}
				} // else addressed and select flag are both set
		} 
		// Number of sub carriers is different between the response of the contactless and the number expected by the CR95HF
	}// else data rate is different between the response of the contactless and the datarate expected by the CR95HF
	return status;
}

/**  
* @brief  	This function returns if the LRIXK contactless tag performs a memory programming by processing the command. ( write, lock). Note that this functions includes the LRIXK custom commands
* @param  	LRIXK_cmd_code	:	command code 
* @retval		TRUE_CODE : The command performs a memory programming
* @retval		FALSE_CODE :  The command does not perform a memory programming
*/
u8 LRIXK_Is_Memory_programming_function(const u8 LRIXK_cmd_code)
{
	if(ISO15693_Is_Memory_programming_function( LRIXK_cmd_code)==TRUE_CODE)
	{
			return TRUE_CODE;
	}
	else
	{
		if((LRIXK_cmd_code==LRIXK_CMD_KILL  )||
		(LRIXK_cmd_code== LRIXK_CMD_WRITE_KILL )||
		(LRIXK_cmd_code== LRIXK_CMD_LOCK_KILL))
		{
			return TRUE_CODE;
		}
		else
			return FALSE_CODE;
	}
}


/**  
* @brief  	This function returns for fast commands if the datarate selected with the protocol select function is really 52kbits/s and if the requestflags is really in high datarate ( fast command multiplies the data rate per 2  (High datarate : 26kbits/s) 
* @param  	Request_flags	:	the request flags to transmit to the contactless tag for checking the datarate
* @param  	Protocol_Settings	:	Stucture containing parameters of the protocol set.
* @retval		TRUE_CODE : Parameters matches
* @retval		FALSE_CODE : Parameters do not macth data rates are different
*/
u8 LRIXK_Is_Fast_Datarate_ok(const u8 Request_flags,const CR95HF_Protocol_Config* Protocol_Settings)
{
	if((ISO15693_Get_DataRate_Flag(Request_flags)!=ISO15693_HIGH_DATARATE)||
	(ISO15693_Get_DataRate_parameter_CR95HF(Protocol_Settings) !=CR95HF_DATARATE_52KBITS))
		return FALSE_CODE;
	else
		return TRUE_CODE;
}


/**  
* @brief  	This function returns if the command is a fast command.
* @param  	LRIXK_cmd_code	:	the command code to check
* @retval		TRUE_CODE : Parameters matches
* @retval		FALSE_CODE : Parameters do not macth data rates are different
*/
u8 LRIXK_Is_Fast_Command(const u8 LRIXK_cmd_code)
{
	if((LRIXK_cmd_code==LRIXK_CMD_FAST_READ_SINGLE_BLOCK)||
	(LRIXK_cmd_code==LRIXK_CMD_FAST_INVENTORY_INITIATED)||
	(LRIXK_cmd_code==LRIXK_CMD_FAST_INITIATE)||
	(LRIXK_cmd_code==LRIXK_CMD_FAST_READ_MULTIPLE_BLOCKS))
	
		return TRUE_CODE ;
	else
		return FALSE_CODE;
}
											

// End of Is_Functions

// End of support_Functions

// End of LRIXK_Tags



