/**
  ******************************************************************************
  * @file iso15693_command.c
  * @brief This file contains all the functions in order to send commands to a contactless tag respecting/performing iso15693 standards. Note that command dealing with multiple blocks (write read get status) utilize internal eeprom of the MCU whereas other  commands use flash program memory.
  * @author STMicroelectronics - MMY Application Team 
  * @version V1.0
  * @date 
  ******************************************************************************
  *
  */
// Includes ------------------------------------------------------------------
#include "iso15693_command.h"



// this structure is retrieved so that the functions could access the protocol settings used by the CR95HF (used to test configuration :request flags and respect312s delay or wait SOF parameter,  for Lock fucntions for instance.
extern CR95HF_Protocol_Config Protocol_Settings;/*!< Protocol and parameters related used*/


// this variable in order to send command with the right communication interface (only used for command dealing with multiple blocks
extern u8 SPI_or_UART_Communication;/*!< Communication Interface used*/

// Those two structures are shared by all the iso15693 commands in order to save memory space for code
CR95HF_CommandSending Cmd;/*!< Common Stucture used  to transmit data within commands*/

CR95HF_ResponseReceiving Response;/*!< Common Stucture used  to receive data within commands*/

// Main Architecture of the doxyfile for this file 
 // @defgroup  Contactless_Tags
	
	// * @defgroup  ISO15693_Tags
	
			// * @defgroup  ISO15693_Commands


	
			// * @defgroup  ISO15693_Support_Functions
	

 // @addtogroup ISO15693_Commands

/**  
* @brief  	This function sends, through the CR95HF, to a contactless tag, an inventory command and fills the Tag structure if some contactless tag answers. Note that if there are 16 slots it's recommend to set the CR95HF into WAIT302s parameter with the protocol select command. Indeed, if the CR95HF is in Wait SOF mode it will waits up to 20ms before returning that there is no tag in the field whereas by waiting ionly 302 s it allows to the application to know more rapidly that there is no tag and to send an EOF to switch to the next slot. 
* @param  	MyTag		:	pointer on the structure
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here inventory flag should be set, if AFI is present, AFI flag should be set. 
* @param  	Tag_AFI		:	the optionnal AFI to select a contactless tag family ( Flag indicates if the field is present or not
* @param  	MaskLength		:	Length of the mask  for transmitting, for example to transmit a part of the UID of one contactless tag. MaskLength represents the number of significant bits . The user should ensure that others bits are reset for padding in order to transmit only whole bytes.
* @param  	Mask	:	The mask to transmit. (Mask[0] = LSB)
* @param  	Inventory_16_slots_Nb_UID_Retrieved	:	pointer to retrieve the number of tag seen during the inventory 16 slots.
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless  tag or the tag did not manage to perform the request.
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : The inventory flag is not set, the mask size overweight the limited size or there is another error into the request flags parameters
* @retval  ISO15693_SUCCESS_CODE : A contactless tag answered the command.
*/
u8 ISO15693_Inventory(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Tag_AFI,const  u8 MaskLength, const u8* Mask, u8* Inventory_16_slots_Nb_UID_Retrieved) 
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//Init Inventory_16_slots_Nb_UID_Retrieved
	*Inventory_16_slots_Nb_UID_Retrieved==INIT_VALUE;
	
	//Check if the inventory flag is really set
	if(ISO15693_Is_Request_Flags_ok(Request_flags,ISO15693_CMD_INVENTORY)==FALSE_CODE || MaskLength >ISO15693_UID_SIZE_BITS)
	{
		/* if the inventory flag is not set the command is not sent. This flag involving different meaning for other bits, it could not be forced to 1 by the function.
		 Also if the mask exceeds 64 bits, an error shall occur
		see the function's descritpion for further description about the Request_flags unallowed*/
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_INVENTORY);
		// Add AFI if present
		if(ISO15693_Get_AFI_Flag(Request_flags)==ISO15693_AFI_PRESENT)
			Cmd.data[cptrdatalength++]=Tag_AFI;
		
		// Add mask 
		ISO15693_Add_Mask_to_command(&Cmd,&cptrdatalength, MaskLength, Mask);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		status=ISO15693_Send_Receive_Inventory_cmd(MyTag, &Response,&Cmd, Request_flags,&status_cr95hf,   Inventory_16_slots_Nb_UID_Retrieved);
		
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a stay quiet command to a unique contactless tag designated by its UID.
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag to put in quiet mode.
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the tag did not manage to perform the request.
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : The inventory flag is set or request is not well addressed
* @retval  ISO15693_SUCCESS_CODE : contactless tag is supposed to be in quiet mode.
*/
u8 ISO15693_Stay_Quiet(const u8 Request_flags, const u8* Tag_UID)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE; // Counter for filling in the CR95HF command structure
	//variable to receive response
	
	// Fill the command parameters CR95HF layer will add the send receive code
	ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_STAY_QUIET);
	// Check the request flags
	//+ the command has to be addressed
	if((ISO15693_Is_Request_Flags_ok(Request_flags,ISO15693_CMD_STAY_QUIET)!= TRUE_CODE) ||
	(ISO15693_Get_Address_Flag(Request_flags)!=ISO15693_ADDRESS_FLAG_SET))
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Add UID of the contactless tag to put in quiet mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		
		status_cr95hf=CR95HF_SendRecv(&Response,&Cmd);// Send command, there is no answer even if an error occurs, therefore CR95HF should return an error (87): wait frame times out or no tag 
		// As this response is not significant,  it is not analyzed apart from a communication lost
		
		if(Response.resp_code!=CR95HF_COMMUNICATION_WITH_MCU_FAILED)
			status=ISO15693_SUCCESS_CODE;// Update status assuming 
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a read single block command to the contactless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer of the structure to write data read
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	Block_number	:	address of the block to read.
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered the command.
*/
u8 ISO15693_Read_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//Check if the inventory flag is really set
	if(ISO15693_Is_Request_Flags_ok(Request_flags,ISO15693_CMD_READ_SINGLE_BLOCK)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_READ_SINGLE_BLOCK);
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number
		Cmd.data[cptrdatalength++]=Block_number;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Single_Block_cmd( MyTag,&Response, status_cr95hf, Request_flags );
	
	}
	return status;
}
	

/**  
* @brief  	This function sends, through the CR95HF, a write single block command to the contactless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Block_number	:	address of the block to write.
* @param  	Block_Length	:	number of byte in a block.
* @param  	Data	:	data to write.
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct)
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Write_Single_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number,const u8 Block_Length, const u8* Data)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	
	//Check if the inventory flag is set and if the pamareters are different from : WAIT312s for CR95HF iso15693 procotol  and request_flags option_flag reset.
	
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_WRITE_SINGLE_BLOCK)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_WRITE_SINGLE_BLOCK);
	
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number
		Cmd.data[cptrdatalength++]=Block_number;
		// Add block data
		CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,Block_Length, Data);
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a lock block command to the contactless tag in the field or to an unique contactless tag designated by its UID. 
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag where the data should be written (optionnal).
* @param  	Block_number	:	address of the block where the data are written.
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Lock_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Block_number)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	

	//Check if the inventory flag is set and if the pamareters are different from : WAIT312s for CR95HF iso15693 procotol  and request_flags option_flag reset.
	
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_LOCK_BLOCK)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_LOCK_BLOCK);
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number
		Cmd.data[cptrdatalength++]=Block_number;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a read multiple block command to the contactless tag in the field or to an unique contactless tag designated by its UID. Note that data received are stored into the internal EEPROM of the MCU.
* @param  	MyTag		:	pointer of the structure to write the command status and eventual error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters   
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered the command.
*/
u8 ISO15693_Read_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 Block_Length=INIT_VALUE; // block size for display
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;

	
	//Check if the inventory flag is really set
	if(ISO15693_Is_Request_Flags_ok(Request_flags,ISO15693_CMD_READ_MULTIPLE_BLOCKS)==FALSE_CODE || Number_of_Blocks>ISO15693_MAX_SIZE_BLOCK_READ_CMD)
	{
		// if the inventory flag is  set the command is not sent. This flag involving different meaning for other bits, it could not be forced to 1 by the function.
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters and add the send receive code
		Cmd.cmd_code=CR95HF_CMD_SEND_REC;
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_READ_MULTIPLE_BLOCKS);
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add first block number
		Cmd.data[cptrdatalength++]=First_Block_number;
		
		// Add number of blocks to read after the first one
		Cmd.data[cptrdatalength++]=Number_of_Blocks;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command 
		CR95HF_SendCommand(&Cmd);
		// Receive the answer
		status_cr95hf=ISO15693_Receive_Multiple_Blocks_cmd(&Response);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd( MyTag,&Response, status_cr95hf,Number_of_Blocks );
		
		// Compute Block size from the datalength, the number of blocks and the option flag
		if(ISO15693_Get_Option_Flag(Request_flags)==ISO15693_OPTION_FLAG_SET)
		{
			// If block status is present block size is length of frame minus number of bytes which are not data minus number of status byte (which is equal to number of block and all divided by number of blocks
			Block_Length=(Response.data_length -(
			ISO15693_LENGTH_PARAMETERS_READ_RESPONSE+Number_of_Blocks ))/(Number_of_Blocks+1);
		}
		else // there is only blocks transmitted block size is  length of frame minus byte which are not data divided by the number of blocks in the frame
			Block_Length= (Response.data_length -ISO15693_LENGTH_PARAMETERS_READ_RESPONSE )/(Number_of_Blocks+1);
			
	}
	return status;
}


#ifdef USE_FULL_ISO15693_COMMANDS
/**  
* @brief  	This function sends, through the CR95HF, a write multiple blocks command to the contactless tag in the field or to an unique contactless tag designated by its UID. Note that data to send to the contactless tag are retrieved from the internal MCU EEPROM.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	First_Block_number	:	address of the first block to write.
* @param  	Number_of_Blocks : number of block to write after the first one.For instance 0x06 means that  seven blocks will be written, 0x00 : only one. The number of blocks is limited to 32 (128 bytes, 1kb with block of 4 bytes). 
* @param  	Block_Length : number of bytes in a block 
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct)
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  or  number of blocks is greater than max size allowed
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Write_Multiple_Block(ISO15693_Tag* MyTag, const u8 Request_flags,const  u8* Tag_UID, const u8 First_Block_number,const  u8 Number_of_Blocks,const  u8 Block_Length)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	u8 option_flag; // To store the value of the option_flag of the request_flags
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	u8 data_block_size_byte;// the size of all blocks in bytes
	//variable to receive response
	
	// Store the option flag value
	option_flag=ISO15693_Get_Option_Flag(Request_flags);
	//Check if the inventory flag is set and if the pamareters are different from : WAIT312s for CR95HF iso15693 procotol  and request_flags option_flag reset.
	
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_WRITE_MULTIPLE_BLOCKS)==FALSE_CODE || Number_of_Blocks>ISO15693_MAX_SIZE_BLOCK_WRITE_CMD)
	{
		// if the inventory flag is  set the command is not sent. This flag involving different meaning for other bits, it could not be forced to 1 by the function.
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_WRITE_MULTIPLE_BLOCKS);
	
		// Note that if a contactless tag is in selected mode and if select_flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add block number
		Cmd.data[cptrdatalength++]=First_Block_number;
		// Add number of blocks
		Cmd.data[cptrdatalength++]=Number_of_Blocks;
		// Add Length of data to the command. Command parameters are ready to be transmitted to lower level for adding the blocks data
		Cmd.data_length=cptrdatalength;
		
		// Compute the number of bytes 
		data_block_size_byte=(Number_of_Blocks+1)*Block_Length;
		
		// Send command and receive response
		status_cr95hf = ISO15693_Send_Multiple_Blocks_cmd(&Response,&Cmd,Request_flags, data_block_size_byte);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}
#endif 

/**  
* @brief  	This function sends, through the CR95HF, a select command to the unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. Here command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Select(ISO15693_Tag* MyTag,const u8 Request_flags, const u8* Tag_UID)
{
u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;

	//Check the request_flags 
	// The command has to be issued in addressed mode
	if((ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_SELECT)==FALSE_CODE)||
	(ISO15693_Get_Address_Flag(Request_flags)!=ISO15693_ADDRESS_FLAG_SET))
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_SELECT);
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request 
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a reset to ready state command to a contactless tag . This command could be used in two different case: 
- thecontactless tag is in quiet mode, it returns into ready mode (request could be addressed or not),
- the contactless tag is in select mode it returns into ready mode (select flag shall be set). Note that for this case another select command with another UID turn automatically the tag into ready mode.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		ISO15693_ERROR_CODE		:the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Reset_to_Ready(ISO15693_Tag* MyTag,const u8 Request_flags, const u8* Tag_UID)
{
u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//Check the request flags
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_RESET_TO_READY)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_RESET_TO_READY);
		// Note that if a tag is in selected mode and if select_ flag is set this contactless tag will answer without uid needs to be add
		// Add UID of the contactless tag if request 
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a write AFI command to the contacless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Tag_AFI	:	AFI of the contacless tag, refer to datasheet for further information.
* @retval		ISO15693_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Write_AFI(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Tag_AFI)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Check the request_flags
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_WRITE_AFI)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_WRITE_AFI);
	
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add AFI
		Cmd.data[cptrdatalength++]=Tag_AFI	;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a Lock AFI command to the contacless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		ISO15693_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Lock_AFI(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Check the request_flags
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_LOCK_AFI)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_LOCK_AFI);
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a write DSFID command to the contacless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @param  	Tag_DSFID	:	DSFID of the contacless tag, refer to datasheet for further information.
* @retval		ISO15693_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Write_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 Tag_DSFID)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Check the request_flags	
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_WRITE_DSFID)==FALSE_CODE )
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_WRITE_DSFID);
	
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add AFI
		Cmd.data[cptrdatalength++]=Tag_DSFID	;
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}



/**  
* @brief  	This function sends, through the CR95HF, a Lock DSFID command to the contacless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		ISO15693_ERROR_CODE	: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Lock_DSFID(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Check the request_flags
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_LOCK_DSFID)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_LOCK_DSFID);
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf =ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(&Response,&Cmd,Request_flags);
		
		// Read the answer provided, check if the CR95HF did not return an error and read the answer of the contactless tag. 
		status=ISO15693_Read_Answer_Format_Response_Flags(MyTag,&Response,status_cr95hf);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, an get System Information commandto the contacless tag in the field or to an unique contactless tag designated by its UID.
* @param  	MyTag		:	pointer on the structure
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block writing (optionnal).
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct).
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters  
* @retval  ISO15693_SUCCESS_CODE :  contactless Tag answered  the command.
*/
u8 ISO15693_Get_System_Info(ISO15693_Tag* MyTag, const u8 Request_flags,const u8* Tag_UID)
{ 
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	CR95HF_CommandSending cmd;
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	CR95HF_ResponseReceiving response;

	// Check request_flags
	if(ISO15693_Is_Request_Flags_ok(Request_flags, ISO15693_CMD_GET_SYSTEM_INFO)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_GET_SYSTEM_INFO);// Add command_code
		//Add optionnal UID if necessary
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
	
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command and receive response
		status_cr95hf = CR95HF_SendRecv(&Response, &Cmd);
		
		// Fill the contactless tag's structure
		status = ISO156693_Fill_in_TagStructure_GetSysInfo_cmd(MyTag, &Response, status_cr95hf);

	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a get multiple blocks security status command to the contactless tag in the field or to an unique contactless tag designated by its UID. note that data are stored into the eeprom.
* @param  	MyTag		:	pointer on the structure to write command status
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct )
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered the command.
*/
u8 ISO15693_Get_Multiple_Blocks_Security_Status(ISO15693_Tag* MyTag, const u8 Request_flags, const u8* Tag_UID, const u8 First_Block_number, const u8 Number_of_Blocks)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	// variables to send command
	u8 cptrdatalength=INIT_VALUE;

	
	
	//Check if the inventory flag is really set
	if(ISO15693_Is_Request_Flags_ok(Request_flags,ISO15693_CMD_GET_MULTIPLE_BLOCKS_STATUS)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
		// Fill the command parameters and add the send receive code
		Cmd.cmd_code=CR95HF_CMD_SEND_REC;
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,ISO15693_CMD_GET_MULTIPLE_BLOCKS_STATUS);
		
		// Note that if a contactless tag is in selected mode and if select_ flag is set this tag will answer without uid needs to be add
		// Add UID of the contactless tag if request is in addressed mode
		ISO15693_Add_UID_to_command(&Cmd, &cptrdatalength, Tag_UID, Request_flags);
		
		// Add first block number
		Cmd.data[cptrdatalength++]=First_Block_number;
		
		// Add number of blocks to read after the first one
		Cmd.data[cptrdatalength++]=Number_of_Blocks;
		
		// Add Length of data to the command. Command is ready to be transmitted
		Cmd.data_length=cptrdatalength;
		// Send command 
		CR95HF_SendCommand(&Cmd);
		// Receive the answer
		status_cr95hf=ISO15693_Receive_Multiple_Blocks_cmd(&Response);
		
		// Fill the contactless tag's structure
		status =  ISO156693_Fill_in_TagStructure_Get_Multiple_Block_Status_cmd( MyTag,&Response, status_cr95hf, Number_of_Blocks);
		
	}
	return status;
}


/**  
* @brief  	This function sends, through the CR95HF, a custom command to the contactless tag in the field or to an unique contactless tag designated by its UID.. User should add in parameters all the data needed. Command size is limited to MAX_DATASENT_LENGTH.
* @param  	MyTag		:	pointer on the structure to write status of the command especially for error code
* @param  	Request_flags		:	specifies how the contactless shall perform the command and answer.  Also indicates  whether corresponding fields are present or not. If command is addressed UID should be present and address flag set.
* @param  	Custom	:	the custom command code.
* @param  	IC_Mfg_Code	:	the IC manufacturer code designting manufacturer of the contactless tag
* @param  	Parameters_Length	:	Length of parameters .
* @param  	Parameters	:	data to write.
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the answer is not valid (CR95HF returns an error or CRC residue is not correct)
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS :there is an error within the request flags parameters   
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the command.
*/
u8 ISO15693_Custom_Commands(ISO15693_Tag* MyTag, const u8 Request_flags, const u8 Custom, u8 IC_Mfg_Code, const u8 Parameters_Length, const u8* Parameters)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;

	// variables to send command
	u8 cptrdatalength=INIT_VALUE;
	
	//variable to receive response
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	//Check if the inventory flag is set and if the pamareters are different from : WAIT312s for CR95HF iso15693 procotol  and request_flags option_flag reset.
	
	if(ISO15693_Is_Request_Flags_ok(Request_flags, Custom)==FALSE_CODE)
	{
		// see the function's descritpion for further description about the Request_flags unallowed
		status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		MyTag->command_ok=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
	}
	else
	{
			// Fill the command parameters CR95HF layer will add the send receive code
		ISO15693_Add_Request_Flags_Command_code_to_command(&Cmd, 
&cptrdatalength, Request_flags ,Custom);
		Cmd.data[cptrdatalength++]=IC_Mfg_Code;
		
		// Add parameters if data do not overweigth
		if((Parameters_Length+cptrdatalength)<MAX_DATASENT_LENGTH)
		{
			CR95HF_Add_Data_ToCommand(&Cmd, &cptrdatalength,Parameters_Length,Parameters);
			// Add Length of data to the command. Command is ready to be transmitted
			Cmd.data_length=cptrdatalength;
		
			// Send command and receive response
			status_cr95hf =CR95HF_SendRecv(&Response, &Cmd);
		
			// Read the answer provided, check if the CR95HF did not return an error and check if the data are valid (CRC). 
			status=ISO156693_Fill_in_TagStructure_Custom_cmd( MyTag,&Response, status_cr95hf );
		
		}
		else// data overweight the command is not sent and return error
		{
			status=ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		}
	}
	return status;
}

// End of ISO15693_commands group

	// * @defgroup ISO15693_Procedure_External_EEPROM



//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
// Advanced functions
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

#ifdef BOARD_SELECTED_EVALBOARD
#ifdef USE_EXTERN_EEPROM_FUNCTION
// This command is only available on the STM8L EVALBOARD 1528
/**  
* @brief  	This function sends, through the CR95HF, several read mulitple Block command in order to read up to 64kbits of the contactless tag. Then all data are stored into the external EEPROM available on the STM8L EVAL board 1528. Note that this function is only available for the STM8L EVAL board 1528. Moreover as there is I2C communication to program the external EEPROM, this function is slower than Read Multiple Block command.Also security status is not retrieved and option flag is forced to 0.
* @param  	MyTag		:	pointer of the structure to write the command status and eventual error code
* @param  	Request_flags		:	to indicate the parameters that the contactless shall use to perform  the command and answer. Also indicate the communication mode if UID is present : addressed mode, select mode...
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, should be at least 0x01 (two blocks)
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag, the answer is not valid (CR95HF returns an error or CRC residue is not correct) or an errror occured during data transfer
* @retval  ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS : there is an error within the request flags parameters or block
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered  the comman and data are stored into the EEPROM
*/
u8 ISO15693_Read_Blocks_Write_in_Ext_EEPROM(ISO15693_Tag* MyTag,  u8 Request_flags,const u8* Tag_UID,const u16 First_Block_Number,  u16 Number_of_Blocks)
{
	// As the read multiple block function could be called several times an error can occur
	// to avoid procedure failure each function linked to contactless tag could be called twice
	u8 status=ISO15693_ERROR_CODE;
	
	u8 size_of_a_block;
	u16 numDataRead;// DMA reset this variable when transfer is finished
	u32 number_of_bytes_to_receive=INIT_VALUE;// to check overweight of bytes
	u8 number_of_blocks_per_transfer;// number of block to receive to reach around 128 bytes per command
	u8 size_of_transfer_bytes;// real number of byte retrieved by each tranfer (if there are 3 bytes per block for example 128 bytes is not reachable)
	u16 address_ext_EEPROM=EXTERNAL_EEPROM_FIRST_ADDRESS;
	u16 current_block_number=First_Block_Number; // address of the first block to read
	
	//-----------
	//u8 i=0;
	//-------

	// Force Option flag to 0;
	Request_flags&=~ISO15693_BIT3SET_BIT7_OPTIONFLAG_SET;
	// Send a read single block  in order to get the size of a block note that the data are not stored into the EEPROM so this block will need to be read again
	status=ISO15693_Read_Single_Block(MyTag, Request_flags, Tag_UID, First_Block_Number);
	if(status!= ISO15693_SUCCESS_CODE)
	{
		status=ISO15693_Read_Single_Block(MyTag, Request_flags, Tag_UID, First_Block_Number);
		if(status!=ISO15693_SUCCESS_CODE)
		{
			// comamnd failed twice abort the procedure
			return ISO15693_ERROR_CODE;
		}
	}
	if(status== ISO15693_SUCCESS_CODE)
	{
		// compute the size of a block
		size_of_a_block=(MyTag->receivedDataLength - ISO15693_LENGTH_PARAMETERS_READ_RESPONSE);
		// compute size of data to receive in bytes 
		// add 1 to number of block to include the first block
		number_of_bytes_to_receive=size_of_a_block*(Number_of_Blocks+1);
		if(number_of_bytes_to_receive>EXTERNAL_EEPROM_SIZE_BYTES)
		{
			// data oversize the external EEPROM
			return ISO15693_ERROR_PARAMETERS_REQUEST_FLAGS;
		}
		// compute number of blocks per fransfer			number_of_blocks_per_transfer=SIZE_1KBITS_BYTES/size_of_a_block; // size of block is at least one byte
		// Compute size in bytes of a transfer
		size_of_transfer_bytes=number_of_blocks_per_transfer*size_of_a_block;
		number_of_blocks_per_transfer--;// in command paramaters is minus one
		Number_of_Blocks++;// to get real number of block to read
	}
		
	while(Number_of_Blocks>number_of_blocks_per_transfer)
	{
		// Send Read Multiple Block command with ISO15693_MAX_SIZE_BLOCK_READ_CMD(32 blocks)
		status=ISO15693_Read_Multiple_Block(MyTag, Request_flags, Tag_UID, current_block_number,  number_of_blocks_per_transfer);
		if(status!=ISO15693_SUCCESS_CODE)
		{
			status=ISO15693_Read_Multiple_Block(MyTag, Request_flags, Tag_UID, current_block_number,  number_of_blocks_per_transfer);
			if(status!=ISO15693_SUCCESS_CODE)
			{
				// comamnd failed twice abort the procedure
				return ISO15693_ERROR_CODE;
			}
		}
		Number_of_Blocks-= (number_of_blocks_per_transfer+1);// compute remaining blocks to read
		current_block_number+=(number_of_blocks_per_transfer+1);//+1 to get the real number
		// Write data read through I2C into the external EEPROM at current address_ext_EEPROM,write only data: hide response flags +1
		Transfer_data_internal_to_external_EEPROM(address_ext_EEPROM,(EEPROM_DATA_PHYSICAL_ADDRESS+1), size_of_transfer_bytes);
		address_ext_EEPROM+=size_of_transfer_bytes;// increment address into the the external EEPROM for next transfer
	}
		
	// Manage the remaining blocks (less than size_of_transfer_bytes of data <= 1kbits)
	if(Number_of_Blocks!=INIT_VALUE)
	{
		// Send Read Multiple Block command with either the remaining number of block to read or with the number of block given in parameters
		status=ISO15693_Read_Multiple_Block(MyTag, Request_flags, Tag_UID, current_block_number,  Number_of_Blocks);
		if(status!=ISO15693_SUCCESS_CODE)
		{
			status=ISO15693_Read_Multiple_Block(MyTag, Request_flags, Tag_UID, current_block_number,  Number_of_Blocks);
			if(status!=ISO15693_SUCCESS_CODE)
			{
				// comamnd failed twice abort the procedure
				return ISO15693_ERROR_CODE;
			}
		}
		// Update size in byte sof the last transfer
		size_of_transfer_bytes=(( Number_of_Blocks+1)*size_of_a_block);
		// Write data read through I2C into the external EEPROM at current address_ext_EEPROM,write only data: hide response flags (+1), and CRC...
			Transfer_data_internal_to_external_EEPROM(address_ext_EEPROM, (EEPROM_DATA_PHYSICAL_ADDRESS+1), size_of_transfer_bytes );
		
	}
	// To test data written, uncomment procedure
	/*
	//  corrupt data in internal EEPROM by re writing above and then retrieve the data from external EEPROM store them into the internal EEPROM and display them
	while(i< )// number of kbytes read
	{
		ISO15693_Read_Multiple_Block(MyTag, Request_flags, Tag_UID, 0x18, 0x10);
	
		Transfer_data_external_EEPROM_to_internal_EEPROM((EEPROM_DATA_PHYSICAL_ADDRESS+1),  (EXTERNAL_EEPROM_FIRST_ADDRESS+(128*i)), 128);
		ISO15693_Display_Answer_Read_Multiple_Blocks(*MyTag, Request_flags, (First_Block_Number+(32*i)),31, 0x04);
		i++;
	}
	*/

	return status;
}
#endif
#endif
// End of ISO15693_Procedure_External_EEPROM group

//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
// Support functions
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------

 // @defgroup ISO15693_Support_Functions
	
/**  
* @brief  	This function initialize unusued fields and significant fields of the structure in order to erase previous field filled in by others commands, this aims to detect any error easierly.
* @param  	MyTag		:	pointer on the structure
* @param  	Tag_UID	:	pointer on the UID of the contactless tag for block reading (optionnal).
* @param  	First_Block_number	:	address of the first block to read.
* @param  	Number_of_Blocks	:	number of block to read after the first one. For instance 0x06 means that 7 seven blocks will be read, 0x00 : only one
* @retval		void		: the structure is directly modified
*/
static void ISO15693_Init_TagStructure_Generic(ISO15693_Tag* MyTag)
{
	// Significant fields, used for all commands
	MyTag->command_ok =ISO15693_STRUCT_INITVALUE;// To erase previous value
	MyTag->response_flags=ISO15693_STRUCT_INITVALUE;
	// Unused fields in most of the command
	MyTag->AFI_Tag =ISO15693_STRUCT_INITVALUE;// To indicate that the field's data is unreliable
	MyTag->DSFID_Tag =ISO15693_STRUCT_INITVALUE;
	MyTag->ICRef =ISO15693_STRUCT_INITVALUE;
}


/**  
* @brief  	This function initialize unusued fields of the structure in order to erase previous field filled in by others command. This aims to detect any error easierly.
* @param  	MyTag		:	pointer on the structure
* @retval		void		: the structure is directly modified
*/
static void ISO15693_Init_TagStructure_Inventory_cmd(ISO15693_Tag* MyTag)
{
	MyTag->command_ok =ISO15693_STRUCT_INITVALUE;// To erase previous value
	MyTag->response_flags=ISO15693_STRUCT_INITVALUE;
	
	MyTag->AFI_Tag =ISO15693_STRUCT_INITVALUE;// To indicate that the field's data is unreliable
	MyTag->ICRef =ISO15693_STRUCT_INITVALUE;
	
	ISO15693_Init_TagStructure_MemSize(MyTag);// Init memsize array 
}


/**  
* @brief  	This function initialize unusued fields of the structure in order to erase previous field filled in by others command. This aims to detect any error easierly. The fucntion is specific to Read Mutliple Blocks command.
* @param  	MyTag		:	pointer on the structure
* @retval		void		: the structure is directly modified
*/
static void ISO15693_Init_TagStructure_Read_Multiple_Blocks_cmd(ISO15693_Tag* MyTag)
{
	ISO15693_Init_TagStructure_Generic(MyTag);
	MyTag->receivedDataLength=ISO15693_STRUCT_INITVALUE;// in case that data  are stored within eeprom
}


/**  
* @brief  	this function initialize unusued fields of the structure in order to erase previous field filled in by others command this aims to detect any error easierly.The fucntion is specific to Get System Info command.
* @param  	MyTag		:	pointer on the structure
* @retval		void		: the structure is directly modified
*/
static void ISO15693_Init_TagStructure_GetSysInfo_cmd(ISO15693_Tag* MyTag)
{
	MyTag->command_ok =ISO15693_STRUCT_INITVALUE;// To erase previous value
	MyTag->response_flags=ISO15693_STRUCT_INITVALUE;
	// All the field shall be provided by the contactless tag depending on the option flag
}


/**  
* @brief  	This function initialize unusued fields of the structure in order to erase previous field filled in by others command this aims to detect any error easierly.The fucntion is specific to Get Multiple Blcoks security status command.
* @param  	MyTag		:	pointer on the structure
* @retval		void		: the structure is directly modified
*/
static void ISO15693_Init_TagStructure_Get_Multiple_Blocks_Status_cmd(ISO15693_Tag* MyTag)
{
	ISO15693_Init_TagStructure_Generic(MyTag);
	MyTag->receivedDataLength=ISO15693_STRUCT_INITVALUE;// in case that data  are stored within eeprom
}


/**  
* @brief  	This function initialize unusued fields of the memSize array in order to erase previous field filled in by others command this aims to detect any error easierly.The fucntion is called in inventory function because user expects to have  new contactless tag in the field.
* @param  	MyTag		:	pointer on the structure
* @retval		void		: the structure is directly modified
*/
static void ISO15693_Init_TagStructure_MemSize(ISO15693_Tag* MyTag)
{
	MyTag->memSize[0]=INIT_VALUE;
	MyTag->memSize[1]=INIT_VALUE;
	MyTag->memSize[2]=INIT_VALUE;
}



	// * @defgroup ISO15693_Add_Data_to_Commands_Functions

/**  
* @brief  	This function add the UID into the command's data and increments the counter of data if the command is addressed.
* @param  	Cmd	:	pointer on the commmand to fill in 
* @param  	Cptr_ToIncrement : pointer on the counter incrementing number of data within the command
* @param		Command_Code		: the command code for the contactless tag
* @param		Request_flags		: 
* @retval		void : no return value
*/
void ISO15693_Add_Request_Flags_Command_code_to_command(CR95HF_CommandSending* Cmd, 
u8* Cptr_ToIncrement, const u8 Request_flags ,const u8 Command_Code)
{
	Cmd->data[(*Cptr_ToIncrement)++]=Request_flags;
	Cmd->data[(*Cptr_ToIncrement)++]=Command_Code;
}


/**  
* @brief  	This function add the Mask into the command's data and increments the counter of data.
* @param  	Cmd	:	pointer on the commmand to fill in 
* @param  	Cptr_ToIncrement : pointer on the counter
* @param		MaskLength		: Length in bits of the Mask
* @param		Mask		: pointer on the mask to add
* @retval		void : no return value
*/
void ISO15693_Add_Mask_to_command(CR95HF_CommandSending* Cmd, 
u8* Cptr_ToIncrement,const u8 MaskLength, const u8* Mask)
{
	u8 mask_length_in_bytes=INIT_VALUE;
	// compute mask size in bytes
	if(MaskLength%BYTE_SIZE==INIT_VALUE)// mask length is a multiple of 8:  integer number of byte
		mask_length_in_bytes=(MaskLength/BYTE_SIZE); // Masklength /8 == 
	else // There are some padding bits
		// The result is rounded up to the very next lower integer and  1 is added to include the last byte with padding 
		mask_length_in_bytes=((u8)(MaskLength/BYTE_SIZE)+1);
		
	Cmd->data[*Cptr_ToIncrement]=MaskLength;// Add MaskLength to the command
	(*Cptr_ToIncrement)++;
	// Add Mask to the command if significant 
	CR95HF_Add_Data_ToCommand(Cmd, Cptr_ToIncrement, mask_length_in_bytes, Mask);
}


/**  
* @brief  	This function add the UID into the command's data and increments the counter of data if the command is addressed.
* @param  	Cmd	:	pointer on the commmand to fill in 
* @param  	Cptr_ToIncrement : pointer on the counter
* @param		Tag_UID		: pointer on the UID with UID[0]= LSB
* @param		Request_flags		: the request_flags to know if the command is addressed
* @retval		void : no return value
*/
void ISO15693_Add_UID_to_command(CR95HF_CommandSending* Cmd, 
u8* Cptr_ToIncrement, u8* Tag_UID ,u8 Request_flags)
{
	if(ISO15693_Get_Address_Flag(Request_flags)==ISO15693_ADDRESS_FLAG_SET)
	{
		 //If the command is addressed call the add function to add the data
		CR95HF_Add_Data_ToCommand(Cmd, Cptr_ToIncrement,
		ISO15693_UID_SIZE, Tag_UID);
	}
}


// End of ISO15693_Add_Data_to_Commands_Functions 
	
	// * @defgroup ISO15693_Is_Functions

/**  
* @brief  	This function checks if the response flag is set or not. If set the contactless tag encountered an error while performing the command. 
* @param  	ISO15693_Response_flags	:	response flag provided by the contactless tag 
* @retval		TRUE_CODE		: the contactless tag handled the request
* @retval		FALSE_CODE		: the contactless tag answered but the frame is an error message
*/
u8 ISO15693_Is_ResponseFlags_noError_Detected(const u8 ISO15693_Response_flags)
{
	return ISO15693_RESPONSE_FLAGS_ERRORFLAG_MASK & ISO15693_Response_flags;
}


/**  
* @brief  	This function performs several checks to verify that the request flag is not wrong. This is not a garantee that the request flags is right (depends on the contactless tag and on some commands' particularities).  
* @param  	Request_flags	:	First byte of RF command sent to contacless tag
* @param  	ISO15693_cmd_code	:	the code of the command to send to the contactless tag
* @retval		FALSE_CODE		: An error has been found out within the request flags
* @retval		TRUE_CODE		: There is no error seen within the request flags
*/
u8 ISO15693_Is_Request_Flags_ok(const u8 Request_flags, const u8 ISO15693_cmd_code)
{
	u8 status= FALSE_CODE;
	u8 test;// to store some values used for comparing value
	
	test=ISO15693_Get_DataRate_parameter_CR95HF(&Protocol_Settings);
	// Check datarate and sub carrier
	if((test==CR95HF_DATARATE_26KBITS && ISO15693_Get_DataRate_Flag(Request_flags)==ISO15693_HIGH_DATARATE)||
		(test==CR95HF_DATARATE_6KBITS && ISO15693_Get_DataRate_Flag(Request_flags)==ISO15693_LOW_DATARATE))
	{
		// datarates are matching, test subcarrier
		// note that the constants have the same value
		if(ISO15693_Get_SubCarrier_parameter_CR95HF(&Protocol_Settings)==	
			  ISO15693_Get_Subcarrier_Flag(Request_flags))
		{
			if(ISO15693_cmd_code!=ISO15693_CMD_INVENTORY)
			{
				// Check that the inventory flag is not set and that addressed mode and select mode flags are not both set.
				if(ISO15693_Get_Inventory_Flag(Request_flags)!=ISO15693_INVENTORY_FLAG_SET)
				{
					// to test address and select flags
					test=ISO15693_Get_Select_Flag(Request_flags)&ISO15693_Get_Address_Flag(Request_flags);
					if(test!=BIT_SET)
					{
						if(ISO15693_Is_Memory_programming_function(ISO15693_cmd_code)==TRUE_CODE)
						{
							// carry out further tests to forbidd one configuration
							/*If CR95HF protocol select is set with wait312s, the CR95HF sends its command 
							and if no contactless tag has answered within the 312s, the CR95HF returns 	
							no Tag or timeout error, if the option flag is not set that means that the $
							contactless is supposed to answer within those delay. 
							But for memory programmation command, operations could last several ms to 
							be processed so that only error (contactless tag answers immediately 
							if there is some error) can be retrieved.
							If option flag is set (write and lock command) 
							the contactless tag waits for an EOF to provide its response,
							consequently the response could be retrieved.*/
							test=ISO15693_Get_Delay_parameter_CR95HF(&Protocol_Settings);
							if( test!=CR95HF_RESPECT312_DELAY ||  
							ISO15693_Get_Option_Flag(Request_flags)!=ISO15693_OPTION_FLAG_RESET)
							{
								status=TRUE_CODE;
							}
							// else status remains at its initial value: FALSE_CODE
						}	
						else // for others command the contactless is supposed to answer as soon as it performed the command
						{
							// others commands, no further test
								status=TRUE_CODE;
						}
					}
				}
			}
			else // inventory command
			{
				if(ISO15693_Get_Inventory_Flag(Request_flags)==ISO15693_INVENTORY_FLAG_SET)
					status=TRUE_CODE;
			}
		}
	}
	return status;
}


/**  
* @brief  	This function checks if the check collision flag added by the CR95HF is set or not. If it is, that means that the CRC95HF has detected an collision between two contactless tag answers.
* @param  	Response	:	response returned by CR95HF containing the data 
* @retval		TRUE_CODE		: the collision flag is set
* @retval		FALSE_CODE	: the collision flag  is not set
*/ 
u8 ISO15693_Is_Collision_Flag_Set(CR95HF_ResponseReceiving* Response)
{
	u8 testvalue;// To test if the flag is set or not with a mask followed by a shift
	testvalue=Response->data[Response->data_length-1];
	if( (testvalue & ISO15693_COLLISION_DETECTION_MASK )== BIT_SET)
		return  TRUE_CODE;

	else
		return FALSE_CODE;
}


/**  
* @brief  	This function checks if the check CRC flag added by the CR95HF is set or not. If it is, that means that the CRC95HF has detected an error during the reception
* @param  	Response	:	response returned by CR95HF containing the data 
* @retval		TRUE_CODE		: the CRC flag is set
* @retval		FALSE_CODE		: the CRC flag is not set
*/ 
u8 ISO15693_Is_CRC_Flag_Set( CR95HF_ResponseReceiving* Response)
{
	u8 testvalue;// To test if the flag is set or not with a mask followed by a shift
	testvalue=Response->data[Response->data_length-1];
	// in order to get the last byte in the array
	
	if(((testvalue & ISO15693_CRC_MASK )>> SHIFT_BIT1)== BIT_SET)
		return  TRUE_CODE;

	else
		return FALSE_CODE;
}


/**  
* @brief  	This function checks the check CRC flag and the residue of the received command. If there is any problem: mismatch or error, the function returns error value 
* @param  	Response	:	response of the contactless tag returned by CR95HF containing the data 
* @retval		TRUE_CODE		: 	the CRCResidue is correct, response is sound
* @retval		FALSE_CODE		: mismatch or error in CRC residue, response should contains errors of transmission  
*/
u8 ISO15693_Is_CorrectCRC16_Check(CR95HF_ResponseReceiving* Response)
{
	u8 status=FALSE_CODE;
	u8 CRCDataLength=INIT_VALUE;
	u8 CRC_CR95HF_check=ISO15693_ERRORCODE_GENERICCRC16;
	u8 CRC_MCU_check=ISO15693_ERRORCODE_GENERICCRC16;
	/* CR95HF  cumputes the CRC residue and adds a control byte for the CRC, 
	this must not be included in CRC residue computation*/
	//check the CRC control byte, it is the last one in the response
	CRC_CR95HF_check=ISO15693_Is_CRC_Flag_Set(Response);
	if(Response->data_length!=DATALENGTH_NULL)
	{
		// reduce the datalength counter to hide the CRC check
		CRCDataLength= Response->data_length;
		CRCDataLength--;// in order to cut the control CRC byte added by CR95HF
		// check the CRC residue 
		CRC_MCU_check=ISO15693_IsCorrectCRC16Residue(Response->data,CRCDataLength);
		// Compare the results
		if((CRC_CR95HF_check==FALSE_CODE) && (CRC_MCU_check==ISO15693_RESULTOKCRC16))
		{
			status=ISO15693_RESULTOKCRC16;
		}
	}
	return status;
}


/**  
* @brief  	This function returns TRUE_CODE if the contactless tag has been manufactured by STMicroelectronics, the UID of the contactless tag must be known
* @param  	MyTag	:	pointer on the strcture containing the UID
* @retval		TRUE_CODE : STMicroelectronics is the manufacturer
* @retval		FALSE_CODE : STMicroelectronics is not the manufacturer
*/
u8 ISO15693_Is_TagConstructor_STM(const ISO15693_Tag* MyTag)
{
	if(MyTag->UID_Tag[ISO15693_MANUFACTURER_CURSOR]== ISO15693_STMICROELECTRONICS_MANUFACTURER_CODE)
	{
		return TRUE_CODE;
	}
	else
		return FALSE_CODE;
}


/**  
* @brief  	This function returns if the contactless tag performs a memory programming by processing the command. ( write, lock)
* @param  	ISO15693_cmd_code	:	command code 
* @retval		TRUE_CODE : the command is a memory programming command
* @retval		FALSE_CODE : the command do not perform a memory programming
*/
u8 ISO15693_Is_Memory_programming_function(const u8 ISO15693_cmd_code)
{
	if((ISO15693_cmd_code== ISO15693_CMD_WRITE_SINGLE_BLOCK )||
	(ISO15693_cmd_code== ISO15693_CMD_WRITE_MULTIPLE_BLOCKS )||
	(ISO15693_cmd_code== ISO15693_CMD_LOCK_BLOCK )||
	(ISO15693_cmd_code== ISO15693_CMD_WRITE_AFI )||
	(ISO15693_cmd_code== ISO15693_CMD_LOCK_AFI  )||
	(ISO15693_cmd_code== ISO15693_CMD_WRITE_DSFID )||
	(ISO15693_cmd_code== ISO15693_CMD_LOCK_DSFID ))
	{
		return TRUE_CODE;
	}
	else
		return FALSE_CODE;
}


/**  
* @brief  	This function returns if the DSFID is present within the contactless tag answer to a get system info command.
* @param  	Information_flags	:	first byte transmitted by the contactless tag containing the information  
* @retval		TRUE_CODE : DSFID is supported
* @retval		FALSE_CODE : DSFID is not supported
*/
u8 ISO15693_Is_DSFID_Present_Infoflag(const  u8 Information_flags)
{
	//  Test if the flag is set or not with a mask 
	if((Information_flags & ISO15693_INFORMATION_FLAGS_DSFID_MASK)  == BIT_SET)
		return TRUE_CODE;

	else
		return FALSE_CODE;
}


/**  
* @brief  	This function returns if the AFI is present within the contactless tag answer to a get system info command.
* @param  	Information_flags	:	first byte transmitted by the contactless tag containing the information  
* @retval		TRUE_CODE : AFI is supported
* @retval		FALSE_CODE : AFI is not supported
*/
u8 ISO15693_Is_AFI_Present_Infoflag(const  u8 Information_flags)
{
	u8 testvalue;// To test if the flag is set or not with a mask followed by a shift to the LSB
	testvalue = (Information_flags & ISO15693_INFORMATION_FLAGS_AFI_MASK)>> SHIFT_BIT1;
	if(testvalue == BIT_SET)
		return TRUE_CODE;
	
	else
		return FALSE_CODE;
}


/**  
* @brief  	This function returns if the Memsize is present within the contactless tag answer to a get system info command.
* @param  	Information_flags	:	first byte transmitted by the contactless tag containing the information
* @retval		TRUE_CODE : Memsize is supported
* @retval		FALSE_CODE : Memsize is not supported
*/
u8 ISO15693_Is_MemSize_Present_Infoflag(const  u8 Information_flags)
{
	u8 testvalue;// To test if the flag is set or not with a mask followed by a shift to the LSB
	testvalue = (Information_flags & ISO15693_INFORMATION_FLAGS_MEMSIZE_MASK)>> SHIFT_BIT2;
	if(testvalue == BIT_SET)
		return TRUE_CODE;
	
	else
		return FALSE_CODE;
}


/**  
* @brief  	This function returns if the ICREF is present within the contactless tag answer to a get system info command.
* @param  	Information_flags	:	first byte transmitted by the contactless tag containing the information
* @retval		TRUE_CODE : ICREF is supported
* @retval		FALSE_CODE : ICREF is not supported
*/
u8 ISO15693_Is_ICRef_Present_Infoflag(const  u8 Information_flags)
{
	u8 testvalue;// To test if the flag is set or not with a mask followed by a shift to the LSB
	testvalue = (Information_flags & ISO15693_INFORMATION_FLAGS_ICREF_MASK)>> SHIFT_BIT3;
	if(testvalue == BIT_SET)
		return TRUE_CODE;
	
	else
		return FALSE_CODE;
}

// End of ISO15693_Is_Functions 

	// * @defgroup ISO15693_Get_Functions

/**  
* @brief  	This function returns the subcarrier flag determining if the contactless tag's response is made  with one or two subcarriers.
* @param Request_flags : First byte of RF command sent to contacless contactless tag
* @retval		ISO15693_SINGLE_SUBCARRIER : the contactless tag must answer with a single subcarrier 
* @retval		ISO15693_TWO_SUBCARRIERS : the contactless tag must answer with two subcarriers 
*/
u8 ISO15693_Get_Subcarrier_Flag(const u8 Request_flags)
{
	/*Operate a logic AND with the bit placed in the parameter byte to save useful bit
	if 1 : ISO15693_TWO_SUBCARRIERS, 
	if 0 : ISO15693_SINGLE_SUBCARRIER*/
	return Request_flags & ISO15693_BIT1_DUAL_SUBCARRIERS;
}


/**  
* @brief  	This function returns the datarate flag determining  if the contactless tag's response is made  in high or low datarate.
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_LOW_DATARATE : the contactless tag must answer with low datarate 
* @retval		ISO15693_HIGH_DATARATE : the contactless tag must answer with high datarate 
*/
u8 ISO15693_Get_DataRate_Flag(const u8 Request_flags)
{
	/*Operate a logic AND with the bit placed in the parameter byte and  
	shift the bit to the LSB to save useful bit
	if 0 : ISO15693_LOW_DATARATE, 
	if 1 : ISO15693_HIGH_DATARATE*/
	return (Request_flags & ISO15693_BIT2_HIGH_DATARATE);
}


/**  
* @brief  	This function returns the inventory flag to determine if the ccontactless should run the anticollision sequence (inventory command).
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_INVENTORY_FLAG_SET : the inventory flag is set
* @retval		ISO15693_SELECT_FLAG_RESET : the inventory flag is reset 
*/
u8 ISO15693_Get_Inventory_Flag(const u8 Request_flags)
{
	/*Operate a logic AND with the bit placed in the parameter byte and  
	shift the bit to the LSB to saveful bit
	if 1 : ISO15693_INVENTORY_FLAG_SET, 
	if 0 : ISO15693_INVENTORY_FLAG_RESET*/
	return (Request_flags & ISO15693_BIT3_INVENTORY_FLAG_SET);
}


/**  
* @brief  	This function returns the protocol extension flag.
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_NO_PROTOCOL_EXTENSION : there is no protocol extension 
* @retval		ISO15693_PROTOCOL_EXTENSION : the protocol extension flag is set (for example for a Get System Info with a LRi64k
*/
u8 ISO15693_Get_ProtocolExtension_Flag(const u8 Request_flags)
{
	/*Operate a logic AND with the bit placed in the parameter byte and  
	shift the bit to the LSB to saveful bit
	if 1 : ISO15693_PROTOCOL_EXTENSION, 
	if 0 : ISO15693_NO_PROTOCOL_EXTENSION*/
	u8 status=(Request_flags & ISO15693_BIT4_PROTOCOL_EXTENSION);
	if(status!=ISO15693_NO_PROTOCOL_EXTENSION)
	{
		status=ISO15693_PROTOCOL_EXTENSION;
	}
	return status;
}


/**  
* @brief  	This function returns the select flag to determine if the command is destineted to the contactless tag in selected mode. Note that a check is made to ensure that the inventory flag is reset.
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_SELECT_FLAG_SET : the select flag is set
* @retval		ISO15693_SELECT_FLAG_RESET : the select flag is reset
* @retval		ISO15693_ERROR_INVALID_FLAG : bit 3 is set so the parameter does not exist
*/
u8 ISO15693_Get_Select_Flag(const u8 Request_flags)
{
	// check if the inventory flag is really reset
	if(ISO15693_Get_Inventory_Flag(Request_flags)==ISO15693_INVENTORY_FLAG_RESET)
	{
		/*Operate a logic AND with the bit placed in the parameter byte and  
		shift the bit to the LSB to saveful bit
		if 1 : ISO15693_SELECT_FLAG_SET, 
		if 0 : ISO15693_SELECT_FLAG_RESET*/
		return (Request_flags & ISO15693_BIT3RESET_BIT5_REQUEST_TAG_SELECTED);
	}
	else
		return ISO15693_ERROR_INVALID_FLAG;
}


/**  
* @brief  	This function returns the address flag to determine if the command is addressed to the contactless tag designted by its UID. Note that a check is made to ensure that the inventory flag is reset.
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_ADDRESS_FLAG_SET : the address flag is set
* @retval		ISO15693_ADDRESS_FLAG_RESET : the address flag is reset
* @retval		ISO15693_ERROR_INVALID_FLAG : bit 3 is set so the parameter does not exist
*/
u8 ISO15693_Get_Address_Flag(const u8 Request_flags)
{
	// check if the inventory flag is really reset
	if(ISO15693_Get_Inventory_Flag(Request_flags)==ISO15693_INVENTORY_FLAG_RESET)
	{
		/*Operate a logic AND with the bit placed in the parameter byte and  
		shift the bit to the LSB to saveful bit
		if 1 : ISO15693_ADDRESS_FLAG_SET, 
		if 0 : ISO15693_ADDRESS_FLAG_RESET*/
		return  (Request_flags & ISO15693_BIT3RESET_BIT6_REQUEST_ADDRESSED);
	}
	else
		return ISO15693_ERROR_INVALID_FLAG;
}


/**  
* @brief  	This function returns the AFI flag to determine if the AFI is present within the command (inventory). Note that a check is made to ensure that the inventory flag is set.
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_AFI_PRESENT : the AFI flag is set
* @retval		ISO15693_AFI_NOT_PRESENT : the AFI flag is reset
* @retval		ISO15693_ERROR_INVALID_FLAG : bit 3 is reset so the parameter does not exist
*/
u8 ISO15693_Get_AFI_Flag(const u8 Request_flags)
{
	// check if the inventory flag is really reset
	if(ISO15693_Get_Inventory_Flag(Request_flags)==ISO15693_INVENTORY_FLAG_SET)
	{
		/*Operate a logic AND with the bit placed in the parameter byte and  
		shift the bit to the LSB to saveful bit
		if 1 : ISO15693_AFI_PRESENT, 
		if 0 : ISO15693_AFI_NOT_PRESENT*/
		return(Request_flags & ISO15693_BIT3SET_BIT5_AFI_PRESENT);
	}
	else
		return ISO15693_ERROR_INVALID_FLAG;

}


/**  
* @brief  	This function returns the number of slots flag (1 or 16) that the contactless tag has to answer an inventory command.  Note that a check is made to ensure that the inventory flag is set.
* @param RequestFlags : First byte of RF command sent to contacless tag
* @retval		ISO15693_1_SLOT : the contactless tag has one slot to answer
* @retval		ISO15693_16_SLOTS : the contactless tag has 16 slots to answer
* @retval		ISO15693_ERROR_INVALID_FLAG : bit 3 is reset so the parameter does not exist
*/
u8 ISO15693_Get_Slots_Flag(const u8 RequestFlags)
{
	// check if the inventory flag is really reset
	if(ISO15693_Get_Inventory_Flag(RequestFlags)==ISO15693_INVENTORY_FLAG_SET)
	{
		/*Operate a logic AND with the bit placed in the parameter byte and  
		shift the bit to the LSB to saveful bit
		if 1 : ISO15693_1_SLOT, 
		if 0 : ISO15693_16_SLOTS*/
		return (RequestFlags & ISO15693_BIT3SET_BIT6_1_SLOT);
	}
	else
		return ISO15693_ERROR_INVALID_FLAG;
}


/**  
* @brief  	This function returns the option flag. Note: this function assumes that the option flag has the same meaning whatever the inventory flag value. 
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_OPTION_FLAG_RESET : the option flag is reset
* @retval		ISO15693_OPTION_FLAG_SET : the option flag is set
*/
u8 ISO15693_Get_Option_Flag(const u8 Request_flags)
{
	/*Operate a logic AND with the bit placed in the parameter byte and  
		shift the bit to the LSB to saveful bit
		if 1 : ISO15693_OPTION_FLAG_SET, 
		if 0 : ISO15693_OPTION_FLAG_RESET*/
	return (Request_flags & ISO15693_BIT3SET_BIT7_OPTIONFLAG_SET);
}


/**  
* @brief  	This function returns RFU flag. The function  may need to be tailored  when RFU will be defined. Note: this function assumes the RFU flag is the same whatever the inventory flag.
* @param Request_flags : First byte of RF command sent to contacless tag
* @retval		ISO15693_BIT8_RFU_SET : the address flag is set
* @retval		ISO15693_BIT8_RFU_RESET : the address flag is reset
*/
u8 ISO15693_Get_RFU_Flag(const u8 Request_flags)
{
	/*Operate a logic AND with the bit placed in the parameter byte and  
		shift the bit to the LSB to saveful bit
		if 1 : ISO15693_BIT8_RFU_SET, 
		if 0 : ISO15693_BIT8_RFU_RESET*/
	return (Request_flags&ISO15693_BIT8_RFU_SET_MASK);
}





/**  
* @brief  	This function returns if the protocol in use is set with dual Subcarrier or single subcarrier
* @param Protocol_Settings : pointer on the structure to use for verifying
* @retval		CR95HF_DUAL_SUBCARRIERS : dual subcarrier is in use
* @retval		CR95HF_SINGLE_SUBCARRIER : single subcarrier is in use
* @retval		CR95HF_INVALID_PROTOCOL : the protocol in use is not ISO15693
*/
u8 ISO15693_Get_SubCarrier_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings)
{
	u8 status;
	// Check if the used protocol is ISO15396
	if(Protocol_Settings->protocol_in_use==CR95HF_PROT_SEL_ISO15693)
	{
		status= Protocol_Settings->parameters[CR95HF_PARAMETER_BYTE0] & CR95HF_BIT1_DUAL_CARRIERS;// eliminate other bits with mask
		// shift the bit to the LSB if 1= DUAL_SUBCARRIERS, if 0 SINGLE_SUBCARRIER
		status= status >>SHIFT_BIT1;
	}
	else
		status=CR95HF_INVALID_PROTOCOL;
	
	return status;
}

/**  
* @brief  	this function returns if the protocol in use is set with 10 percent modulation or 100 percent modultaion
* @param Protocol_Settings : pointer on the structure to use for verifying
* @retval		CR95HF_MODULATION_10 : 10 percent modulation is in use
* @retval		CR95HF_MODULATION_100 : 100 percent modulation is in use
* @retval		CR95HF_INVALID_PROTOCOL : the protocol in use is not ISO15693
*/
u8 ISO15693_Get_Modulation_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings)
{
	u8 status;
	// Check if the used protocol is ISO15396
	if(Protocol_Settings->protocol_in_use==CR95HF_PROT_SEL_ISO15693)
	{
		status= Protocol_Settings->parameters[CR95HF_PARAMETER_BYTE0] & CR95HF_BIT2_MODULATION_10;// eliminate other bits with mask
		// shift the bit to the LSB if 1 : MODULATION_10, if 0 : MODULATION_100
		status= status >>SHIFT_BIT2;
	}
	else
		status=CR95HF_INVALID_PROTOCOL;
	
	return status;
}


/**  
* @brief  	this function returns if the protocol in use is set with wait Start of frame or respect 312 micro seconds delay for response slots
* @param Protocol_Settings :  structure to use for verifying
* @retval		CR95HF_RESPECT312_DELAY : respect 312 micro seconds is in use
* @retval		CR95HF_WAIT_FOR_SOF : wait for start of frame  is in use
* @retval		CR95HF_INVALID_PROTOCOL : the protocol in use is not ISO15693
*/
u8 ISO15693_Get_Delay_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings)
{
	u8 status;
	// Check if the used protocol is ISO15396
	if(Protocol_Settings->protocol_in_use==CR95HF_PROT_SEL_ISO15693)
	{
		status= Protocol_Settings->parameters[CR95HF_PARAMETER_BYTE0] & CR95HF_BIT3_WAIT_FOR_SOF;// eliminate other bits with mask
		// shift the bit to the LSB if 1 : MODULATION_10, if 0 : MODULATION_100
		status= status >>SHIFT_BIT3;
	}
	else
		status=CR95HF_INVALID_PROTOCOL;
	
	return status;
}


/**  
* @brief  	this function returns if the protocol in use is set with wait Start of frame or respect 312 micro seconds delay for response slots
* @param Protocol_Settings : pointer on the structure to use for verifying
* @retval		CR95HF_DATARATE_RFU : Reserved for Future Use 
* @retval		CR95HF_DATARATE_6KBITS : 6kbits/s datarate  is in use
* @retval		CR95HF_DATARATE_52KBITS :52kbits/s datarate   is in use
* @retval		CR95HF_DATARATE_26KBITS : 26kbits/s datarate  is in use
* @retval		CR95HF_INVALID_PROTOCOL : the protocol in use is not ISO15693
*/
u8 ISO15693_Get_DataRate_parameter_CR95HF(const CR95HF_Protocol_Config* Protocol_Settings)
{
	u8 status;
	// Check if the used protocol is ISO15396
	if(Protocol_Settings->protocol_in_use==CR95HF_PROT_SEL_ISO15693)
	{
		status= Protocol_Settings->parameters[CR95HF_PARAMETER_BYTE0] & CR95HF_BIT4_5_DATARATE_RFU;
		/* eliminate other bits with mask shift the bits to LSB position 
		if 00 : DATARATE_26KBITS, if 01 : DATARATE_52KBITS
		if 10 : DATARATE_6KBITS, if 11 : DATARATE_RFU*/
		status= status >>SHIFT_BIT4;
	}
	else
		status=CR95HF_INVALID_PROTOCOL;
	
	return status;
}



/**  
* @brief This functions returns the number of byte used to describe the memory size of the contactless tag (STMicroelectronics manufactured tag)	
* @param ICRef :  the ICref to compare with ICref referenced in the miscellaneous.h ( those available at the date of creation of this file, as a matter of fact update should be necessary.)
* @retval		number_of_byte: the number of byte composing the word
* @retval	 NO_ICREF_MATCHING : No Icref matches, 0 is returned 
*/
u8 ISO15693_Get_NumberofByte_MemSize_using_ICREF(const u8 ICRef)
{
	// Compare with ICRef in miscellaneous.h
	return Compare_IcRef_Return_Memsize(ICRef);
}


/**  
* @brief This functions returns the size of a block using the UID of the Tag ( STMicroelectronics' manufactured contactless tag).  	
* @param ICRef : the ICRef to  compared with contactless Tag "database" . ICref referenced in the miscellaneous.h are those available at the date of creation of this file. As a matter of fact update should be necessary.
* @retval		number_of_byte: the number of byte composing the word
* @retval	 ISO15693_BLOCK_SIZE_ERROR : No Icref matches, 0 is returned 
*/
u8 ISO15693_Get_Size_of_Block_using_ICRef(const u8 ICRef)
{
	return Compare_ICRef_Return_Size_of_Block(ICRef);
}


/**  
* @brief This functions returns the size of a block using the MemSize field of the tag structure a get system info command should have been performed before calling this function.  	
* @param MyTag : the structure containing the data drecibing the contactless tag
* @retval	size_of_Block: the number of byte composing the block
* @retval	 ISO15693_BLOCK_SIZE_ERROR : memsize contains no valid value, 0 is returned 
*/
u8 ISO15693_Get_Size_of_Block_using_MemSize(const ISO15693_Tag* MyTag)
{
	u8 size_of_block=ISO15693_BLOCK_SIZE_ERROR;// the value to return
	// to determine if memsize is not at INIT_VALUE
	u8 status_valid_data=INIT_VALUE; 
	u8 mem_size_counter=INIT_VALUE;	// counter for memsize
	
	// check if there data in memsize 
	if(MyTag->memSize[mem_size_counter++]==INIT_VALUE)
	{
		if(MyTag->memSize[mem_size_counter++]==INIT_VALUE)
		{
			// two first field are still at their initial value,the memsize has not been filled by a get system info
			size_of_block=ISO15693_BLOCK_SIZE_ERROR;
			status_valid_data=ISO15693_STRUCT_INITVALUE;
		}
	}
	
	if(status_valid_data!=ISO15693_STRUCT_INITVALUE)
	{
		// there is valid data
		//try  to determine the size of MemSize 
		//mem_size is locate now at the end of memsize
		if(MyTag->memSize[mem_size_counter]==INIT_VALUE)
		{
			// Memsize is on two bytes (value is one less than the real one)
			size_of_block=MyTag->memSize[ISO15693_MEMSIZE_2BYTES_BLOCK_SIZE_CURSOR]+1;
		}
		else// memory size is on 3 bytes (value is one less than the real one)
			size_of_block=MyTag->memSize[ISO15693_MEMSIZE_3BYTES_BLOCK_SIZE_CURSOR]+1;
	}
		
	return size_of_block;
}


/**  
* @brief  	This function returns the memory size of the contactless tag in a whole  number of kilo bits .Note that the size max handled will be 256kbits.
* @param MemSize : pointer on the bytes describing the memory size
* @param Number_of_Bytes_MemSize : number of bytes describing the memory
* @retval	size : the size of the memory in kilo bits max 255 kbits	
* @retval	INIT_VALUE : the size is not a whole number of Kbits
*/
u16 ISO15693_Get_MemSize_kbits(const u8* MemSize,const u8 Number_of_Bytes_MemSize)
{
	u16 size;// size of the memory, limited to 255kbits
	u8 block_size; // size of a block in byte
	u8 number_of_block_LSB;// number of blocks for less than 256 blocks
	u16 number_of_block_MSB=INIT_VALUE_u16;// number of blocks in case the number is on two bytes
	u32 number_of_block=INIT_VALUE_u32; // total number of block if two bytes are necessary
	u32 compute_size;// size in bytes
	u8 number_of_byte;// number of bytes describing the memory
	u8 memsize_ctr=INIT_VALUE;// counter for memsize word
	
	// For Memsize :(less than 32kbits memory, with block of 4 bytes)
	// [0]= number of blocks [1]= number of bytes in a block 
	// up to 2Mbits 65536 blocks with 4 bytes
	// [0]= number of blocks LSB [1]= number of blocks MSB [2]= number of bytes in a block 
	
	// Retrieve number of block then bytes per block and multiply them
	// 00= 1 byte per block, 1F= 32 bytes per block +1 allows to retrieve the right number
	if(Number_of_Bytes_MemSize>ISO15693_MEMSIZE_WORD_BYTE_SIZE)
	{
		// There are 3 bytes to describe memory size
		number_of_block_LSB=MemSize[memsize_ctr++];// Retrieve the LSB byte 
		number_of_block_MSB=MemSize[memsize_ctr++];// Retrieve the MSB byte 
		block_size=MemSize[memsize_ctr]+1;// retrieve size of block which is minus one
		// Merge them
		number_of_block_MSB =number_of_block_MSB<<SHIFT_BYTE;// Put byte into bit 8 to 15
		number_of_block= number_of_block_MSB  | number_of_block_LSB;
		number_of_block++;// Add 1 to get the real number
		compute_size=number_of_block*block_size; // Compute the size in bytes
		// check for a whole number of Kbits
		if(compute_size%128==0)// 128 bytes =1024 bits = 1Kbit
		{
			compute_size=compute_size>>SHIFT_BIT_TO_KILO;// Convert into kilo bytes first to avoid overflow
			compute_size=compute_size<<SHIFT_BYTE_TO_BIT;//convert into  kilo bits
		}
		else
			compute_size=INIT_VALUE;
		/* in order to accept the round up supress the if else statement
		and let the two conversion lines*/
	}
	else// There are 2 bytes (up to 8kbits with block of four bytes)
	{
		number_of_block=MemSize[memsize_ctr++]+1;
		// Retrieve the number of byte per block
		block_size=MemSize[memsize_ctr++]+1;// 00= 1 byte per block, 1F= 32 bytes per block +1 allows to retrieve the right number
		compute_size=number_of_block*block_size; // Compute the size in bytes
		if(compute_size%128==0)// check if the number of bytes is a whole Kbits number
		{
			compute_size=compute_size<<SHIFT_BYTE_TO_BIT;//convert into bits first to be sure that the result is over 1024 before the kilo conversion
			compute_size=compute_size>>SHIFT_BIT_TO_KILO;// Convert into kilo bits
		}
		else
			compute_size=INIT_VALUE;
		/* in order to accept the round up supress the if else statement
		and let the  conversion line within the if statement*/
	}
	
	size=(u16)compute_size;
		
	return size;
}


/**  
* @brief  	This function returns the memory size of the contactless tag in bits. This function should be called if the size of the memory is not an interger number of kilo bits
* @param MemSize : pointer on the bytes describing the memory size.
* @param Number_of_Bytes_MemSize : number of bytes describing the memory
* @retval	size : the size of the memory in  bits	
*/
u32 ISO15693_Get_MemSize_bits(u8* MemSize, u8 Number_of_Bytes_MemSize)
{
	u32 size;// size of the memory, limited to 255kbits
	u8 block_size; // size of a block in byte
	u8 number_of_block_LSB;// number of blocks for less than 256 blocks
	u16 number_of_block_MSB=INIT_VALUE_u16;// number of blocks in case the number is on two bytes
	u32 number_of_block=INIT_VALUE_u32; // total number of block if two bytes are necessary
	u8 memsize_ctr=INIT_VALUE;// counter for memsize word

	if(Number_of_Bytes_MemSize>ISO15693_MEMSIZE_WORD_BYTE_SIZE)
	{
		// There are 3 bytes to describe memory size
		number_of_block_LSB=MemSize[memsize_ctr++];// Retrieve the LSB byte 
		number_of_block_MSB=MemSize[memsize_ctr++];// Retrieve the MSB byte 
		block_size=MemSize[memsize_ctr]+1;// retrieve size of block which is minus one
		// Merge them
		number_of_block_MSB =number_of_block_MSB<<SHIFT_BYTE;// Put byte into bit 8 to 15
		number_of_block= number_of_block_MSB  | number_of_block_LSB;
		number_of_block++;// Add 1 to get the real number
	}
	else// There are 2 bytes
	{
		number_of_block=MemSize[memsize_ctr++]+1;
		// Retrieve the number of byte per block
		block_size=MemSize[memsize_ctr++]+1;// 00= 1 byte per block, 1F= 32 bytes per block +1 allows to retrieve the right number
	}
	
	size=number_of_block*block_size; // Compute the size in bytes
	size=size<<SHIFT_BYTE_TO_BIT;//convert into bits
		
	return size;
}


/**  
* @brief  	This function retrieves the UID stored into the EEPROM by the inventory 16 slots commands.
* @param UID_Tag : pointer on the UID to fill in with the data stored in the EEPROM
* @param UID_number : the number of the UID to retrieved . For instance to retrieved the UID number 3 UID_number=3 and UID_number=1 for the first one
* @retval	ISO15693_ERROR_CODE : the MSB of the UID is not E0 data shouls have been modified
* @retval	ISO15693_SUCCESS_CODE : there is no sign of error into the UID retrieved
*/
u8 ISO15693_Get_UID_from_EEPROM(u8* UID_Tag, u8 UID_number)
{
	u8 status= ISO15693_ERROR_CODE;
	u8 counter=INIT_VALUE;
	u32 address;
	// address is computed as this : place cursor at the beginnning of the UIDs address,add UID_number * size of an UID to get the address of the first UID of the desired UID  
	UID_number--;// to start at the beginning for the UID number 1
	address=EEPROM_UID_PHYSICAL_ADDRESS+(ISO15693_UID_SIZE*UID_number);
	while( counter< ISO15693_UID_SIZE)
	{
		// Read the data at the address computed above shifted of the number of byte already read 
		UID_Tag[counter]= FLASH_ReadByte(address+counter);
		counter++;
	}
	counter--;// to test the UID[7] MSB
	if(UID_Tag[counter]==ISO15693_UID_MSB)
		status=ISO15693_SUCCESS_CODE;
		
	return status;
}



// End of ISO15693_Get_Functions 
	
	

	// * @defgroup ISO15693_CRC_Functions
/**  
* @brief  	This function computes the CRC16 as defined by CRC ISO/IEC 13239
* @param  	DataIn		:	input data 
* @param	NbByte)		: 	Number of bytes of DataIn
* @retval	ResCrc		: 	CRC16 computed
*/
int16_t ISO15693_CRC16 (const u8* DataIn,const u8 NbByte) 
{ 
u8 	i,
		j; 
int32_t ResCrc = ISO15693_PRELOADCRC16;
	
	for (i=0;i<NbByte;i++) 
	{ 
		ResCrc=ResCrc ^ DataIn[i];
		for (j=8;j>0;j--) 
		{
			ResCrc = (ResCrc & ISO15693_MASKCRC16) ? (ResCrc>>1) ^ ISO15693_POLYCRC16 : (ResCrc>>1); 
		}
	} 

	return ((~ResCrc) & 0xFFFF);
	
} 


/**  
* @brief  this function computes the CRC16 residue as defined by CRC ISO/IEC 13239
* @param  DataIn		:	input  data 
* @param	Length 		: 	Number of bytes of DataIn
* @retval ISO15693_RESULTOKCRC16  	:   CRC16 residue is correct	
* @retval ISO15693_ERRORCODE_GENERICCRC16	:  CRC16 residue is false
*/
u8 ISO15693_IsCorrectCRC16Residue (const u8 *DataIn,const u8 Length)
{
int16_t ResCRC=0;


	// check the CRC16 Residue 
	if (Length !=0)
		ResCRC=ISO15693_CRC16 (DataIn, Length);
	
	if (((~ResCRC) & 0xFFFF) != ISO15693_RESIDUECRC16)
		return ISO15693_ERRORCODE_GENERICCRC16;
	
	return ISO15693_RESULTOKCRC16;
}

/**  
* @brief  	This function computes the CRC16 as defined by CRC ISO/IEC 13239 using data stored into the internal EEPROM of the MCU.
* @param  	NbByte		:	number of bytes to check 
* @retval	ResCrc		: 	CRC16 computed
*/
int16_t ISO15693_CRC16_EEPROM (const u8 NbByte) 
{ 
u8 	i,
		j,
		dataIn; 
u32 address=EEPROM_DATA_PHYSICAL_ADDRESS;
int32_t ResCrc = ISO15693_PRELOADCRC16;
	
	for (i=0;i<NbByte;i++) 
	{ 
		dataIn=FLASH_ReadByte(address);
		ResCrc=ResCrc ^ dataIn;
		address++;
		for (j=8;j>0;j--) 
		{
			ResCrc = (ResCrc & ISO15693_MASKCRC16) ? (ResCrc>>1) ^ ISO15693_POLYCRC16 : (ResCrc>>1); 
		}
	} 

	return ((~ResCrc) & 0xFFFF);
	
} 


/**  
* @brief  This function computes the CRC16 residue as defined by CRC ISO/IEC 13239 using data stored into the internal EEPROM of the MCU.
* @param	Length 		: 	Number of bytes of DataIn
* @retval ISO15693_RESULTOKCRC16  	:   CRC16 residue is correct	
* @retval ISO15693_ERRORCODE_GENERICCRC16	:  CRC16 residue is false
*/
u8 ISO15693_IsCorrectCRC16Residue_EEPROM (const u8 Length)
{
int16_t ResCRC=0;


	// check the CRC16 Residue 
	if (Length !=0)
		ResCRC=ISO15693_CRC16_EEPROM (Length);
	
	if (((~ResCRC) & 0xFFFF) != ISO15693_RESIDUECRC16)
		return ISO15693_ERRORCODE_GENERICCRC16;
	
	return ISO15693_RESULTOKCRC16;
}


// End of ISO15693_CRC_Functions 

	// * @defgroup ISO15693_Built_in_Commands_Functions
	
		// * @defgroup ISO15693_Inventory_Function

/**  
* @brief  This function sends a inventory command for conctactless tag.The function checks the type of inventory : 1 or 16 slots and  calls the right fucntion to send the command and to manage answers received
* @param  MyTag		:	pointer on the structure to fill with command status and extra information according to the type of inventory
* @param  Response		:	pointer on the structure designated to contain the answer provided by the CR95HF.
* @param	Cmd 		: 	pointer on the command to send
* @param	status_cr95hf 		: 	pointer on the status of the CR95HF command ( the Inventory command is included into the SendRecv command to transmit it to the CR95HF )
* @param  	Request_flags		:	to indicate the parameters used by the contactless tag
* @param	Inventory_16_slots_Nb_UID_Retrieved		:  pointer to store the  number of UIDs received.
* @retval ISO15693_SUCCESS_CODE  	:   a  least one tag answered and its UID is correctly stored	
* @retval ISO15693_ERROR_CODE	:  no tag answered or several in the same slots without any UID stored 
*/
u8 ISO15693_Send_Receive_Inventory_cmd(ISO15693_Tag* MyTag, CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, u8 Request_flags, u8* status_cr95hf, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 status;
	u8 buffer_CR95HF_Display_Enabled; // To store the value display allowed
	
	// Initialization of unfilled fields for both kind of command 1 and 16 slots
	ISO15693_Init_TagStructure_Inventory_cmd(MyTag);
		
		
		if(	ISO15693_Get_Slots_Flag(Request_flags)==ISO15693_1_SLOT)
		{
			*status_cr95hf = CR95HF_SendRecv(Response, Cmd);// Send command and receive response
			// Fill the contactless tag's structure
			status = ISO156693_Fill_in_TagStructure_Inventory_cmd(MyTag, Response, *status_cr95hf);
		}
		
		else
		{
			// send inventory 16 slots commands, in this case only UIDs will be received
			status = ISO15693_Send_Receive_Inventory_16_slots(Response, Cmd,
			 Inventory_16_slots_Nb_UID_Retrieved);
			 *status_cr95hf=status;// in order to make the display in this case if one tag answer those two variable have the same meaning
			 if(status==ISO15693_SUCCESS_CODE)
			{
				// if there is only one tag seen Write its UID into the tag stucture
				if(*Inventory_16_slots_Nb_UID_Retrieved==1)
				{
					status=ISO15693_Get_UID_from_EEPROM(MyTag->UID_Tag,*Inventory_16_slots_Nb_UID_Retrieved );
					// if STmicroelectronics is the manufacturer write the ICRef
					ISO15693_Retrieve_ICRef( MyTag);
				}
				if(status==ISO15693_SUCCESS_CODE)
				{
					// write command status into the tag structure for display 
					MyTag->command_ok=ISO15693_SUCCESS_CODE;
					MyTag->error_code=ISO15693_SUCCESS_CODE;
				}
			}
			else // fill in tag structure with the last error  retrieve likely 88 or 87
			{
				// write error retrieved by the CR95HF into the tag structure 
				MyTag->command_ok=ISO15693_ERROR_CODE;
				MyTag->error_code=Response->resp_code;
			}
		}
			
	return status;
}


/**  
* @brief  This function sends a inventory function with 16 slots for conctactless tag. The function sends EOF pulse marking boudaries of the slots. If a contacless tag answers its UID is stored into the Inventroy_16_slots_UID_Array and number of UID received is incremented. If two or more tags answer, there is no UID stored and the function does not manage the anticollision algorithm in order to get the UIDs. Only UIDs of contacless tag answerering alone in a slot are strored.  
* @param  Response		:	pointer on the structure designated to contain the answer provided by the CR95HF.
* @param	Cmd 		: 	pointer on the command to send
* @param	Inventory_16_slots_Nb_UID_Retrieved		:  pointer to store the  number of UIDs received.
* @retval ISO15693_SUCCESS_CODE  	:   a  least one tag answered and its UID is correctly stored	
* @retval ISO15693_ERROR_CODE	:  no tag answered or several in the same slots without any UID stored 
*/
static u8 ISO15693_Send_Receive_Inventory_16_slots(CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, u8* Inventory_16_slots_Nb_UID_Retrieved)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	u8 slot_counter= ISO15693_INVENTORY_16_SLOTS_COUNTER;// counter to send EOF determining slot boundaries
	*Inventory_16_slots_Nb_UID_Retrieved=INIT_VALUE;// initialyze number of UIDs received
 // Send command, check for an error answer and send an EOF to receive the answer of the next slot
	status_cr95hf =CR95HF_SendRecv(Response, Cmd);
	if(status_cr95hf==CR95HF_SUCCESS_CODE)
	{
		ISO156693_Fill_in_UID_Array_Inventory_16_slots_cmd( Inventory_16_slots_Nb_UID_Retrieved, Response);// a tag answered in slot 0 retrieved its UID
	}
	
	while(slot_counter!=INIT_VALUE)
	{
		delay_10us_base(WAIT_310us);// minimum delay between two EOF without a tag answer is 309s. However, because of others operations (answer reading CR95HF polling) the real time between two EFO  is greater.
		status_cr95hf = ISO15693_SendEOF(Response);
		if(status_cr95hf==CR95HF_SUCCESS_CODE)
		{
			ISO156693_Fill_in_UID_Array_Inventory_16_slots_cmd( Inventory_16_slots_Nb_UID_Retrieved, Response);
		}
			slot_counter--;
	}
	
	if(*Inventory_16_slots_Nb_UID_Retrieved!= INIT_VALUE)
		status=ISO15693_SUCCESS_CODE;// At least one UID is stored

	// else status= ISO15693_ERROR_CODE by default
	return status;
}

// End of ISO15693_Inventory_Function 
	
		// * @defgroup ISO15693_Memory_Programming_Functions

/**  
* @brief  This function sends to the contactless tag through the CR95HF a Single block (read, write, lock) memory programming command and receive the answer. As the contactless tag takes longer to perform that kind of the command, depending on the option flag and on the protocol parameters set the function handles the  command sending and the contactless tag answer reception. 
* @param  Response		:	pointer on the structure designated to contain the answer provided by the CR95HF.
* @param	Cmd 		: 	pointer on the command to send
* @param	Request_flags		:  to determine how the contactless tag will answer (wait an EOF or as soon as possible)
* @retval CR95HF_SUCCESS_CODE  	:  The command was sent and a response was received 	
* @retval CR95HF_ERROR_CODE	: Command failed
*/
 u8 ISO15693_Send_Receive_Single_Block_Memory_Programming_cmd(CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, const u8 Request_flags)
{
	u8 status_cr95hf;
	
	if(ISO15693_Get_Option_Flag(Request_flags)==ISO15693_OPTION_FLAG_SET)
	{
		// Send command, check for an error answer and send an EOF to receive the success answer
		// The answer of the command could be either an error or timeout error because the contacless tag is performing the memory progarammation.
		
		status_cr95hf =CR95HF_SendRecv(Response, Cmd);
	
		//Set a delay in order to let the contactless tag performs the writing
		delay_ms(WAIT_20ms);// the command shall be performed latest after 20ms
		if(Response->resp_code==CR95HF_ERROR_NOTAG_OR_TIMEOUT)// expected error
			status_cr95hf = ISO15693_SendEOF(Response);
	}
	else // Answer is sent right after the writing
		status_cr95hf = CR95HF_SendRecv(Response, Cmd);
			
	return status_cr95hf;
}

#ifdef USE_FULL_ISO15693_COMMANDS
/**  
* @brief  This function sends to the contactless tag through the CR95HF a Multiple block command ( write) and receive the answer. As the contactless tag takes longer to perform that kind of the command, depending on the option flag and on the protocol parameters set the function handles the  command sending and the contactless tag answer reception. Note that data to send to the contactless tag are retrieved  from the internal eeprom of the MCU.
* @param  Response		:	pointer on the structure designated to contain the answer provided by the CR95HF.
* @param	Cmd 		: 	pointer on the command to send
* @param	Request_flags		:  to determine how the contactless tag will answer (wait an EOF or as soon as possible)
* @param	Number_of_Bytes_to_Transmit : number of bytes to write into the contactless tag memory and contained in ot the internal EEPROM.
* @retval CR95HF_SUCCESS_CODE  	:   CRC16 residue is correct	
* @retval CR95HF_ERROR_CODE	:  CRC16 residue is false
*/
static u8 ISO15693_Send_Multiple_Blocks_cmd(CR95HF_ResponseReceiving* Response,CR95HF_CommandSending* Cmd, const u8 Request_flags, const u8 Number_of_Bytes_to_Transmit)
{
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	if(ISO15693_Get_Option_Flag(Request_flags)==ISO15693_OPTION_FLAG_SET)
	{
		// Send command, check for an error answer and send an EOF to receive the success answer
		// The answer of the command could be either an error or timeout error because the contacless tag is performing the memory progarammation.
			
			// send command
			ISO15693_Send_Multiple_Blocks_CR95HF_level(Cmd,Number_of_Bytes_to_Transmit);
			// receive immediate answer (eventual error code)
			CR95HF_Wait_Receive_Answer(Response);
			status_cr95hf=CR95HF_IsReaderResultOK(Response,CR95HF_SENDRECV_SUCCESS_CODE);

		
		//Set a delay in order to let the contactless tag performs the writing
		delay_ms(WAIT_20ms);// the command shall be performed latest after 20ms
		if(Response->resp_code==CR95HF_ERROR_NOTAG_OR_TIMEOUT)// expected error
			status_cr95hf = ISO15693_SendEOF(Response);// receive contactless tag answer
	}
	else // Answer is sent right after the writing
	{
		// send command
		ISO15693_Send_Multiple_Blocks_CR95HF_level(Cmd,Number_of_Bytes_to_Transmit);
		// receive  answer 
		CR95HF_Wait_Receive_Answer(Response);
		// Update status cr95hf
		 status_cr95hf = CR95HF_IsReaderResultOK(Response,CR95HF_SENDRECV_SUCCESS_CODE);
		 
		
	}

	return status_cr95hf;
}



/**  
* @brief  This function sends byte per byte the command to the CR95HF (wich will transmit those to the cntactless tag) including the data contained within the internal EEPROM. Note that this function is largely inspired from the  CR95HF_SendCommand function.
* @param	Cmd 		: 	pointer on the command to send
* @param	Cursor_in_command_data 		: 	cursor into the command data to know where data (block) can be written, for instance if UID is transmitted the cursor is different.
* @param	Number_of_Bytes_to_Transmit : number of bytes to write into the contactless tag memory and contained in ot the internal EEPROM.
* @retval void : no return value
*/
static void ISO15693_Send_Multiple_Blocks_CR95HF_level(CR95HF_CommandSending* Cmd, u8  Number_of_Bytes_to_Transmit)
{
	u8 cptr_datalength=INIT_VALUE;
	u8 cmd_parameters_counter=INIT_VALUE;// counter for command parameters(request_flags UID...)
	u32 address=EEPROM_DATA_PHYSICAL_ADDRESS;// address of the data into the eeprom
	u8 data;// data to send
	u8 total_command_datalength;
	cptr_datalength= Cmd->data_length;// the counter worths the length of parameters length wihtout blocks data
	
	// Compute the whole length of the command parameters + blocks
	total_command_datalength=Cmd->data_length+ Number_of_Bytes_to_Transmit;
	
	// Enable communication (for SPI if selected)
	CR95HF_SPI_Chip_Enable_Send_Byte_Mode();	
	
	// Start sending command 
	CR95HF_Send_Byte(CR95HF_CMD_SEND_REC);// Send the command code Send Recv for the CR95HF
	CR95HF_Send_Byte(total_command_datalength);// Send the length of the whole command
	// Send parameters of the command 
	while(cptr_datalength!=0)
	{
		CR95HF_Send_Byte(Cmd->data[cmd_parameters_counter]);// Send byte
		cmd_parameters_counter++;// increment adddress for the next data
		cptr_datalength--; // decrement number of bytes remaining to transmit
	}
	// Send the data included in the command byte per byte
	while(Number_of_Bytes_to_Transmit!=INIT_VALUE)
	{
		data= FLASH_ReadByte(address);
		CR95HF_Send_Byte(data);// Send byte
		address++;// increment adddress for the next data
		Number_of_Bytes_to_Transmit--; // decrement number of bytes remaining to transmit
	}
	
	// Communication ending (for SPI if selected)
	CR95HF_SPI_Chip_Disable();
}

#endif

// End of ISO15693_MemoryProgramming_Function_Function 
	

/**  
* @brief  This function sends through the CR95HF an End Of Frame, This is used for Write Lock and Inventory 16 slots command to inform the contactless that it can answer now (Write and Lock commands) or that a new slot is beginning (inventory 16 slots)
* @param  Response		:	pointer on the structure designated to contain the answer provided by the CR95HF.
* @retval ISO15693_SUCCESS_CODE  	:   a contactless tag response has been received
* @retval ISO15693_ERROR_CODE	:  the CR95HF returned an error
*/
static u8 ISO15693_SendEOF(CR95HF_ResponseReceiving* Response)
{
	return CR95HF_SendEOF( Response);
}


	// * @defgroup ISO15693_Receive_Multiple_Blocks_Commands_Functions

/**  
* @brief  This function receives, trough the CR95HF, the response of the contactless tag to a  readMultiple block command (read). Note that data received from the contactless tag are stored into the internal eeprom of the MCU. (the whole contactless tag frame.
* @param  Response		:	pointer on the structure designated to contain the answer provided by the CR95HF.
* @retval CR95HF_SUCCESS_CODE  	:   CRC16 residue is correct	
* @retval CR95HF_ERROR_CODE	:  CRC16 residue is false
*/
u8 ISO15693_Receive_Multiple_Blocks_cmd(CR95HF_ResponseReceiving* Response)
{
	u8 status_cr95hf=CR95HF_ERROR_CODE;
	
	// Wait for answer
	if(CR95HF_Wait_Answer(DELAY_BEFORE_TIMEOUT_220MS)==EXIT_COMMUNICATION_OK)
			ISO15693_Receive_Multiple_Blocks_Store_into_EEPROM(Response); // Receive the answer
	else
	{
		Response->resp_code=CR95HF_COMMUNICATION_WITH_MCU_FAILED;
		Response->data_length=DATALENGTH_NULL;
	}

	// start analyze of data
	// check if the CR95HF returned a success code
	status_cr95hf=CR95HF_IsReaderResultOK(Response,CR95HF_SENDRECV_SUCCESS_CODE);
	
	
	return status_cr95hf;
}


/**  
* @brief  This function receives byte per byte, trough the CR95HF, the response provided by the contactless tag the read or get multiple block status commands. Note that block data are stored into the internal MCU EEPROM
* @param  Response : Pointer on the structure to store part of the answer
* @retval CR95HF_ResponseReceiving : resposne receive ( status, and error frame if retrieved)
*/
static 	ISO15693_Receive_Multiple_Blocks_Store_into_EEPROM(CR95HF_ResponseReceiving* Response)
{
	u8 cptrdataLength=INIT_VALUE;// length of the data to read
	u8 response_flags; // to check if the frame has is error flag set
	u32 address=EEPROM_DATA_PHYSICAL_ADDRESS;
	// create array to receive complete answer before storing it into EEPROM
	u8 data [ISO15693_READ_MULITPLE_BLOCK_SIZE_ISO15693_MAX_SIZE_BLOCK_READ_CMD];
	u8 data_counter=INIT_VALUE;
	
	CR95HF_Init_ResponseReceiving(Response);// init response
	
	// Enable communication (for SPI is selected)
	CR95HF_SPI_Chip_Enable_Read_Byte_Mode();
	
	// Reading incoming data
	Response->resp_code = CR95HF_Read_Byte();// reading the Resp code	
	Response->data_length = CR95HF_Read_Byte();// reading the length of data	
	cptrdataLength=Response->data_length;	
	/* At this level there are two cases: 
	- resp_code is a CR95HF error, datalength is  null, the command is handled like others
	- resp_code is ok, there is an answer from the contactless tag  */
	if(Response->data_length!=INIT_VALUE)
	{
		// Read one more byte (response flags)
		response_flags=CR95HF_Read_Byte();
		cptrdataLength--;// decrement number of data to read
		/*
		if datalength is 5 either the contactless tag answered an error,
		so command is written into the tag structure like others before data validation (CRC), 
		or that is the get multiple block status answer for one block. 
		response flag allows to make the difference between those two case*/
	
		// check for error from the contactless tag
		if(Response->data_length ==ISO15693_ERROR_FRAME_SIZE && ISO15693_Is_ResponseFlags_noError_Detected(response_flags)!=TRUE_CODE)
		{
			// the frame received contains the error 
			Response->data[data_counter++]=response_flags;// store response flags
			while (cptrdataLength!=0)
			{
				Response->data[data_counter++] = CR95HF_Read_Byte();
				cptrdataLength--;
			}
		}
		else// data must be written into the eeprom
		{
			// receive data 
			while (cptrdataLength!=0)
			{
				data[data_counter++]=CR95HF_Read_Byte();
				cptrdataLength--;
			}
			// store them into flash
			// write the response flags into the eeprom at the first address for CRC validation
			FLASH_ProgramByte(address, response_flags);
			address++;// increment address
			// re init counter and data length
			data_counter=INIT_VALUE;
			cptrdataLength=((Response->data_length)-1);//-1 because resp flag is already written 
			while (cptrdataLength!=0)
			{
				FLASH_ProgramByte(address, data[data_counter++]);
				cptrdataLength--;// decrement number of data to write into EEPROM
				address++;// increment EEPROM address
			}
		}
	}
	// Communication ending (for SPI if selected)
	CR95HF_SPI_Chip_Disable();
}

// End of ISO15693_Receive_Multiple_Blocks_Commands_Functions 

		// * @defgroup ISO15693_Analysis_and_Fill_in_Tag_Structure_Functions


/**  
* @brief  	The purpose of this function is to split the response to an inventory command into different identifiable fields gathered within the structure 
* @param  	MyTag		:	pointer on the structure
* @param  	Response		:	pointer on the response provided by the CR95HF
* @param  	CR95HF_command_status		: status of the command concerning CR95HF 
* @retval		ISO15693_SUCCESS_CODE	: the structure was successfully filled in
* @retval		ISO15693_ERROR_CODE	: the structure was unsuccessfully filled in
*/
 u8 ISO156693_Fill_in_TagStructure_Inventory_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status )
{
	u8 status= ISO15693_ERROR_CODE;
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	// Check if CR95HF did not encounter an error.
	if(CR95HF_command_status!=CR95HF_SUCCESS_CODE)
	{
		MyTag->command_ok=ISO15693_ERROR_CODE;
		MyTag->error_code=Response->resp_code;
	}
	else
	{
		//Check if the data received are sound 
		status= ISO156693_Check_Data_Received_Start_Process( MyTag, Response, &cptrresponse );
		if(status==ISO15693_SUCCESS_CODE)
		{
			// Fill in specific data retrieved by the command:
			// DSFID :
			MyTag->DSFID_Tag=Response->data[cptrresponse++];
			// UID :
			ISO15693_Retrieve_UID( MyTag,Response, &cptrresponse);
			// ICRef:
			ISO15693_Retrieve_ICRef(MyTag);
			// Complete the status of the function and success of command
			MyTag->command_ok = ISO15693_SUCCESS_CODE;
			MyTag->error_code = ISO15693_SUCCESS_CODE;
		}
	}
	// If CRC is not ok ,status indicates the error, others errors would  already have been  managed by the CR95HF.
	// Note that the contactless tag does not answser if there is any problem.
	
	return status;		
}


/**  
* @brief  	The purpose of this function is first to check if the data received from the tag are valid (only one tag answered and its response is sound (no crc error) and then call the right function to write the UID received into the EEPROM memory. The funtion assumes that CR95HF answer code is CR95HF_SUCCESS_CODE (even if CRC check should detect any error).Note that UIDs are stored into the EEPROM at addreesses given in miscellaneous.h file. 
* @param  	Inventory_16_slots_Nb_UID_Retrieved		:	number of UIDs already received
* @param  	Response		:	pointer on the response provided by the CR95HF
* @retval		void : none return value
*/
static void ISO156693_Fill_in_UID_Array_Inventory_16_slots_cmd( u8* Inventory_16_slots_Nb_UID_Retrieved,CR95HF_ResponseReceiving* Response )
{
	u8 cptrresponse=INIT_VALUE; // counter for the data
	
	//Check if the data received are sound (only one tag answered)
	if(ISO15693_Is_CorrectCRC16_Check(Response)==TRUE_CODE)
	{
		// store the UID
		ISO15693_Retrieve_UID_16_slots(*Inventory_16_slots_Nb_UID_Retrieved ,Response);
		(*Inventory_16_slots_Nb_UID_Retrieved)++;// increment number of UIDs received
	}
	// If CRC is not ok ,the UID is not written
	// Note that the contactless tag does not answser if there is any problem.

}


/**  
* @brief  	the purpose of this function is to split the response to a read single block command into different identifiable fields gathered within the structure 
* @param  	MyTag		:	pointer on the structure
* @param  	Response		:	pointer on the response provided by the CR95HF
* @param  	CR95HF_command_status		: status of the command concerning CR95HF 
* @param  	Request_flags		: to determine if block status is present
* @retval		ISO15693_SUCCESS_CODE	: the structure was successfully filled in
* @retval		ISO15693_ERROR_CODE	: the structure was unsuccessfully filled in
*/
u8 ISO156693_Fill_in_TagStructure_Read_Single_Block_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status, const u8 Request_flags )
{
	u8 status= ISO15693_ERROR_CODE;
	//u8 cptrdata=ISO15693_INVENTORY_UID_CURSOR;// counter of UID length
	u8 cptrresponse=INIT_VALUE; // counter for the data

	// Initialization of unfilled fields
	ISO15693_Init_TagStructure_Generic(MyTag);
	
	// Check if CR95HF did not encounter an error.
	if(CR95HF_command_status!=CR95HF_SUCCESS_CODE)
	{
		MyTag->command_ok=ISO15693_ERROR_CODE;
		MyTag->error_code=Response->resp_code;
	}
	else
	{
		//Check if the data received are sound 
		status= ISO156693_Check_Data_Received_Start_Process( MyTag, Response, &cptrresponse );
		if(status==ISO15693_SUCCESS_CODE)
		{
			// Check if error flag is set
			if( ISO15693_Is_ResponseFlags_noError_Detected(MyTag->response_flags)==TRUE_CODE)
			{
				// move data to the beginning of the datafield in order to easy data reading, block locking status if present is written before block data to be coherent with read multiple block function. To avoid dealing with block size and option flag for block status, the size of the block and eventual status is deducted from the size of data received and the size of others parameters which are not related to the block data.
				memcpy(MyTag->dataField,Response->data+1,(MyTag->receivedDataLength-ISO15693_LENGTH_PARAMETERS_READ_RESPONSE));
				
				// Complete the status of the function and success of command
				MyTag->command_ok = ISO15693_SUCCESS_CODE;
				MyTag->error_code = ISO15693_SUCCESS_CODE;
			}
			else
			{
				// Analyze the error
				ISO15693_Fill_in_Tag_Structure_ErrorFlag_set(MyTag, Response);
				//The command is considered as successful as the tag answered and the answer is valid
			}
		}
	}
	// If CRC is not ok ,status indicates the error, others errors would  already have been  managed by the CR95HF.
	// Note that the contactless tag does not answser if there is any problem.
	
	return status;		
}


/**  
* @brief  	The purpose of this function is to split the response  to a read mutliple blocks into different identifiable fields gathered within the structure 
* @param  	MyTag		:	pointer on the structure
* @param  	Response		:	pointer on the response provided by the CR95HF
* @param  	CR95HF_command_status		: status of the command concerning CR95HF 
* @param  	Number_of_Blocks : number of blocks read after the first one
* @retval		ISO15693_SUCCESS_CODE	: the structure was successfully filled in
* @retval		ISO15693_ERROR_CODE	: the structure was unsuccessfully filled in
*/
u8 ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status, const u8 Number_of_Blocks )
{
	u8 status= ISO15693_ERROR_CODE;
	u8 cptrresponse=INIT_VALUE; // counter for the data
	u8 number_bytes;// To store number of bytes read
	u8 length_of_data;// to store the structure data length
	// Initialization of unfilled fields
	ISO15693_Init_TagStructure_Get_Multiple_Blocks_Status_cmd(MyTag);
	
	// Check if CR95HF did not encounter an error.
	if(CR95HF_command_status!=CR95HF_SUCCESS_CODE)
	{
		MyTag->command_ok=ISO15693_ERROR_CODE;
		MyTag->error_code=Response->resp_code;
	}
	else
	{
		// look for an error frame
		if( (Response->data_length== ISO15693_ERROR_FRAME_SIZE )&&
		(ISO15693_Is_ResponseFlags_noError_Detected(Response->data[INIT_VALUE])!= TRUE_CODE))
		{
			status= ISO15693_Read_Answer_Format_Response_Flags(MyTag,Response,CR95HF_command_status);
		}
		else// block are expected to be into the EEPROM
		{
			length_of_data = Response->data_length; 
			length_of_data--;// to hide the CRC check byte added by the CR95HF
			if(ISO15693_IsCorrectCRC16Residue_EEPROM (length_of_data)==TRUE_CODE)
			{
				MyTag->receivedDataLength=Response->data_length; 
				// Complete the status of the function and success of command
				MyTag->command_ok = ISO15693_SUCCESS_CODE;
				MyTag->error_code = ISO15693_SUCCESS_CODE;
				status=ISO15693_SUCCESS_CODE;
			}
			else
			{
				MyTag->command_ok = ISO15693_ERROR_CODE;
				MyTag->error_code = ISO15693_ERROR_CRC;
			}
		}
	}
	// If CRC is not ok ,status indicates the error, others errors would  already have been  managed by the CR95HF.
	// Note that the contactless tag does not answser if there is any problem.
	
	return status;		
}


/**  
* @brief  	The purpose of this function is to split the response to a get system info command into different identifiable fields gathered within the structure. 
* @param  	MyTag		:	pointer on the structure
* @param  	Response		:	response provided by the CR95HF supposed to contain the contactless tag answer.
* @param  	CR95HF_command_status		:	command status from the CR95HF point of view
* @retval		ISO15693_SUCCESS_CODE	: the structure was successfully filled in
* @retval		ISO15693_ERROR_CODE	: the structure was unsuccessfully filled in
*/
static u8 ISO156693_Fill_in_TagStructure_GetSysInfo_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status )
{
	u8 status= ISO15693_ERROR_CODE;
	u8 cptrresponse=INIT_VALUE; // counter for the data
	u8 information_flags=INIT_VALUE; // To evaluate available data
	
	// Initialization of unfilled fields
	ISO15693_Init_TagStructure_GetSysInfo_cmd(MyTag);
	// Check if CR95HF did not encounter an error.
	if(CR95HF_command_status!=CR95HF_SUCCESS_CODE)
	{
		MyTag->command_ok=ISO15693_ERROR_CODE;
		MyTag->error_code=Response->resp_code;
	}
	else
	{
		//Check if the data received are sound 
		status= ISO156693_Check_Data_Received_Start_Process( MyTag, Response, &cptrresponse );
		if(status==ISO15693_SUCCESS_CODE)
		{
			if( ISO15693_Is_ResponseFlags_noError_Detected(MyTag->response_flags)==TRUE_CODE)
			{
				// Read the information flags
				information_flags=Response->data[cptrresponse++];
				// Retrieve the data and write them in the contactless Tag structure
				ISO15693_Retrieve_UID(MyTag,Response, &cptrresponse);
				// This function is supposed to analyse all the remaining data
				ISO15693_Retrieve_Data_With_Information_Flags(MyTag, Response,  information_flags, &cptrresponse);
				
				// Complete the status of the function and success of command
				MyTag->command_ok = ISO15693_SUCCESS_CODE;
				MyTag->error_code = ISO15693_SUCCESS_CODE;
			}
			else
			{
				// Analyze the error
				ISO15693_Fill_in_Tag_Structure_ErrorFlag_set(MyTag, Response);
				//The command is considered as successful as the tag answered and the answer is valid
			}
		}
	}
	return status;
}


/**  
* @brief  	The purpose of this function is to split the response  to a get multiple block security status command into different identifiable fields gathered within the structure 
* @param  	MyTag		:	pointer on the structure
* @param  	Response		:	pointer on the response provided by the CR95HF
* @param  	CR95HF_command_status		: status of the command concerning CR95HF 
* @param  	Number_of_Blocks : number of blocks read after the first one
* @retval		ISO15693_SUCCESS_CODE	: the structure was successfully filled in
* @retval		ISO15693_ERROR_CODE	: the structure was unsuccessfully filled in
*/
u8 ISO156693_Fill_in_TagStructure_Get_Multiple_Block_Status_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status, const u8 Number_of_Blocks )
{
	// as this fucntion has exactly the same process as ISO156693_Fill_in_TagStructure_Read_Multiple_Block_cmd this function is called.
	
	u8 status= ISO156693_Fill_in_TagStructure_Read_Multiple_Blocks_cmd(MyTag,Response, CR95HF_command_status, Number_of_Blocks );

	return status;		
}


/**  
* @brief  	The purpose of this function is to split the response to a custom command into different identifiable fields gathered within the structure. 
* @param  	MyTag		:	pointer on the structure
* @param  	Response		:	response provided by the CR95HF supposed to contain the contactless tag answer.
* @param  	CR95HF_command_status		:	command status from the CR95HF point of view
* @retval		ISO15693_SUCCESS_CODE	: the structure was successfully filled in
* @retval		ISO15693_ERROR_CODE	: the structure was unsuccessfully filled in
*/
static u8 ISO156693_Fill_in_TagStructure_Custom_cmd(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 CR95HF_command_status )
{
	u8 status= ISO15693_ERROR_CODE;
	u8 cptrresponse=INIT_VALUE; // counter for the data
	u8 information_flags=INIT_VALUE; // To evaluate available data
	
	// Initialization of unfilled fields
	ISO15693_Init_TagStructure_Generic(MyTag);
	// Check if CR95HF did not encounter an error.
	if(CR95HF_command_status!=CR95HF_SUCCESS_CODE)
	{
		MyTag->command_ok=ISO15693_ERROR_CODE;
		MyTag->error_code=Response->resp_code;
	}
	else
	{
		//Check if the data received are sound 
		status= ISO156693_Check_Data_Received_Start_Process( MyTag, Response, &cptrresponse );
		if(status==ISO15693_SUCCESS_CODE)
		{
			if( ISO15693_Is_ResponseFlags_noError_Detected(MyTag->response_flags)==TRUE_CODE)
			{
				// Complete the status of the function and success of command
				MyTag->command_ok = ISO15693_SUCCESS_CODE;
				MyTag->error_code = ISO15693_SUCCESS_CODE;
			}
			else
			{
				// Analyze the error
				ISO15693_Fill_in_Tag_Structure_ErrorFlag_set(MyTag, Response);
				//The command is considered as successful as the tag answered and the answer is valid
			}
		}
	}
	return status;
}


/**  
* @brief  	The purpose of the function is to fill the contactless Tag structure when the errorFlag is set in the contactless tag response.  
* @param  	MyTag	:	pointer on the struture to fill in
* @param  	Response	:	the contactless tag's response receieved
* @retval		void : no return value, error code is written in MyTag
*/
 void ISO15693_Fill_in_Tag_Structure_ErrorFlag_set(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response)
{
	// Indicates that command failed. 
	MyTag->command_ok=ISO15693_ERROR_CODE;
	// Analyze if the response is reliable
	// fill in the error code field
	MyTag->error_code=Response->data[ISO15693_ERROR_CODE_CURSOR];
	
}

/**  
* @brief  	The purpose of the function is to read the response of the contactless tag when the response expected is only response flags, if error flag is set the function calls the ISO15693_Fill_in_Tag_Structure_ErrorFlag_set function.
* @param  	MyTag	:	pointer on the struture to fill in
* @param  	Response	:	the contactless tag's response receieved
* @param  	Status_cr95hf	:	status of the command from the CR95HF point of view
* @retval		ISO15693_ERROR_CODE		: the command failed. Either there is no contactless tag or the contactless tag did not manage to perform the request. 
* @retval  ISO15693_SUCCESS_CODE : contactless tag answered the command.
*/
u8  ISO15693_Read_Answer_Format_Response_Flags(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response,const u8 Status_cr95hf)
{
	u8 status=ISO15693_ERROR_CODE;
	u8 cptrresponse=INIT_VALUE;/* counter to be placed right after the response flags for the 
	ISO156693_Check_Data_Received_Start_Process function, note that as for that type of command there is no data "useful" this counter has no use here.*/
	
	// Initialyze tag's structure
	ISO15693_Init_TagStructure_Generic(MyTag);
	
	// Analyse the response
	// Check if CR95HF did not encounter an error.
	if(Status_cr95hf!=CR95HF_SUCCESS_CODE)
	{
		MyTag->command_ok=ISO15693_ERROR_CODE;
		MyTag->error_code=Response->resp_code;
	}
	else
	{
		status =  ISO156693_Check_Data_Received_Start_Process( MyTag,Response, &cptrresponse);
		if(status==ISO15693_SUCCESS_CODE)
		{
			// Check if error flag is set
			if( ISO15693_Is_ResponseFlags_noError_Detected(MyTag->response_flags)==TRUE_CODE)
			{
				// Complete the status of the function and success of command
				MyTag->command_ok = ISO15693_SUCCESS_CODE;
				MyTag->error_code = ISO15693_SUCCESS_CODE;
			}
			else// error flag is set retrieve the error and write it in the contactless tag structure
			{
				ISO15693_Fill_in_Tag_Structure_ErrorFlag_set(MyTag,Response);
				//The command is considered as successful as the tag answered and the answer is valid
			}
		}
		/* note that a CRC error will be written into the contactless tag structure 
		by the ISO156693_Check_Data_Received_Start_Process function */
	}
	return status;
}


// End of ISO15693_Fill_in_Tag_Structure_Functions 

		// * @defgroup ISO15693_Answer_Analysis_Function

/**  
* @brief  	This function  checks if the data are sound (CRC ok) and filled in basic data common for all response : copy the whole data, write response_flag and data_length into the structure.
* @param  	MyTag		:	pointer on the structure to complete
* @param  	Response :	the response containing the data
* @param  	Cursor_in_response_data :	actual position of the cursor into data and to increment
* @retval		ISO15693_SUCCESS_CODE		: Control byte is ok and data are sound
* @retval		ISO15693_ERROR_CODE : either CRC / collision flag  is set or CRC residue computed by the application does not match
*/
u8 ISO156693_Check_Data_Received_Start_Process(ISO15693_Tag* MyTag, const CR95HF_ResponseReceiving* Response,  u8* Cursor_in_response_data )
{
	u8 status;
	// Control byte : check the control byte (collision and CRC flag) added by the CR95HF and recompute the CRC as additionnal check
	status= ISO15693_Is_CorrectCRC16_Check(Response);
	// if CRC computation is ok, data are sound and could be processed
	// Also check check if data will not overflow the 
	// No specific error is generated but the CRC one
	if((status==ISO15693_SUCCESS_CODE) && (Response->data_length< ISO15693_DATA_RECEIVED_MAX_SIZE))
	{
		// Put the data response into the dataField stucture
		memcpy(MyTag->dataField,Response->data,Response->data_length);
		// Affect the length of data received
		MyTag->receivedDataLength=Response->data_length;
		// Affect the flags into the structure and increment counter for data
		MyTag->response_flags = Response->data[*Cursor_in_response_data];
		(*Cursor_in_response_data)++;
	}
	else
	{
		MyTag->command_ok = ISO15693_ERROR_CODE;
		// Others errors could only be generated by CR95HF and so, are detected 	earlier so only CRC error and collision detect could occur at this point.
		// retrieve the cause of the error : the CRC (flag or computed).
			MyTag->error_code = ISO15693_ERROR_CRC;
			status=ISO15693_ERROR_CODE; // to return only success or error code
	}
	return status;
}


// End of ISO15693_Answer_Analysis_Function 

		// * @defgroup ISO15693_Retrieve_Data_Into_Answer_Functions


/**  
* @brief  	The function fills the contactless Tag structure with the UID (Inventory and Get system info command). 
* @param  	MyTag		:	pointer on the structure to complete
* @param  	Cursor_in_response_data		:	actual position of the cursor into data. It must be the position of the UID into the response format
* @param  	Response :	the response containing the data
* @retval		void	: no return value
*/
 void ISO15693_Retrieve_UID(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, u8* Cursor_in_response_data)
{
	//u8 UID_counter=INIT_VALUE;// cursor on the LSB of the UID
	// loop increments counter to scan response UID from LSB to MSB and write UID in the structure the same way from the LSB UID[0] to MSB UID[7]
	
	memcpy(MyTag->UID_Tag,Response->data+(*Cursor_in_response_data),ISO15693_UID_SIZE);
	(*Cursor_in_response_data)+=ISO15693_UID_SIZE;
}


/**  
* @brief  	The function fills the contactless Tag structure with the UID (Get system info command). Note that UIDs are stored into the EEPROM at fixed addreess given in miscellaneous.h file
* @param  	Inventory_16_slots_Nb_UID_Retrieved		:	number of UIDs already received (cursor for the array)
* @param  	Response :	the response containing the data
* @retval		void	: no return value
*/
static void ISO15693_Retrieve_UID_16_slots(u8 Inventory_16_slots_Nb_UID_Retrieved ,CR95HF_ResponseReceiving* Response)
{
	u8 UID_counter= INIT_VALUE;
	u8 UID_cursor=ISO15693_UID_CURSOR_CR95HF_RESPONSE_DATA;
	u32 address_array_cursor=EEPROM_UID_PHYSICAL_ADDRESS+Inventory_16_slots_Nb_UID_Retrieved*ISO15693_UID_SIZE;// cursor into the array stored within the EEPROM, set the cursor at the beginning of the new UID into the array
	
	// Store the UID into the EEPROM 
	while(UID_counter <=ISO15693_UID_SIZE)
	{
		// Write the UID[UID_cursor] at the address computed above
		FLASH_ProgramByte(address_array_cursor, Response->data[UID_cursor]);
		UID_cursor++;// increment UID cursor 
		address_array_cursor++;// increment address in EEPROM
		UID_counter++;
	}
}

/**  
* @brief  	The function fills the contactless Tag structure with the ICRef using the UID (Get system info command). Only available for contactless tag manufactured by STMicroelectronics
* @param  	MyTag		:	pointer on the structure to complete
* @retval		void	: no return value
*/
void ISO15693_Retrieve_ICRef( ISO15693_Tag* MyTag)
{
	// ICREF : IC ref is shifted of 2 bits 
	// ICREF is filled in only for STMicroelectronics' tags
	if(ISO15693_Is_TagConstructor_STM(MyTag) ==TRUE_CODE)
		MyTag->ICRef= MyTag->UID_Tag[ISO15693_ICREF_CURSOR_IN_UID];
}


/**  
* @brief  	The function is used in a Get System Info command and using the information flag fills in the contactless Tag structure with the options supported.
* @param  	MyTag		:	pointer on the structure to complete
* @param  	Response	:	the response provided by the contactless tag
* @param  	Information_flags	:	to determine if a field if present or not
* @param  	Cursor_in_response_data		:	actual position of the cursor into the frame
* @retval		void	: no return value
*/
static void ISO15693_Retrieve_Data_With_Information_Flags(ISO15693_Tag* MyTag,CR95HF_ResponseReceiving* Response, const u8 Information_flags, u8* Cursor_in_response_data)
{
	u8 memsize_cursor=INIT_VALUE;
	u8 nb_of_byte_memsize=INIT_VALUE;
	// Check if DSFID is supported and write if it is.
	if(ISO15693_Is_DSFID_Present_Infoflag(Information_flags)==TRUE_CODE)
	{
		MyTag->DSFID_Tag = Response->data[*Cursor_in_response_data];// contactless tag field affectation 
		(*Cursor_in_response_data)++;// incrementation
	}
	else // To avoid error 
		MyTag->DSFID_Tag=ISO15693_STRUCT_INITVALUE;
		
	// Check if AFI is supported and write if it is.
	if(ISO15693_Is_AFI_Present_Infoflag(Information_flags)==TRUE_CODE)
	{
		MyTag->AFI_Tag= Response->data[*Cursor_in_response_data];// contactless tag field affectation 
		(*Cursor_in_response_data)++;// incrementation
	}
	else // To avoid error
		MyTag->AFI_Tag=ISO15693_STRUCT_INITVALUE;
		
	// Check if memSize is supported and write if it is.
	if(ISO15693_Is_MemSize_Present_Infoflag(Information_flags)==TRUE_CODE)
	{
		// If the contactless tag respects iso15693 recommendations, this information is on two bytes
		//Compute the MemSize word size using the length of the frame
		nb_of_byte_memsize=ISO15693_Compute_Number_of_bytes_MemSize_using_GetSysInfo_Frame(Information_flags,Response->data_length);
			
		while(memsize_cursor<nb_of_byte_memsize)
		{
			MyTag->memSize[memsize_cursor++]=Response->data[*Cursor_in_response_data];
			(*Cursor_in_response_data)++;// incrementation
		}
		// Re init last byte in case a tag with a 3 bytes memSize was seen before
		if(nb_of_byte_memsize==ISO15693_MEMSIZE_WORD_BYTE_SIZE)
		{
			MyTag->memSize[memsize_cursor]=INIT_VALUE;
		}
	}
	else
	{
		// MemSize is initialized with error check value for each member
		// Set to 0  because 0xEC could be a valid value
		MyTag->memSize[memsize_cursor++]==INIT_VALUE;
		MyTag->memSize[memsize_cursor++]==INIT_VALUE;
		MyTag->memSize[memsize_cursor++]==INIT_VALUE;
	}
	// Check if ICref is supported and write if it is.
	if( ISO15693_Is_ICRef_Present_Infoflag(Information_flags)==TRUE_CODE)
	{
		MyTag->ICRef= Response->data[*Cursor_in_response_data];// contactless tag field affectation 
			//(*Cursor_in_response_data)++;// incrementation for eventual RFU
	}
	else // To avoid error
		MyTag->ICRef=ISO15693_STRUCT_INITVALUE;	
}


// End of ISO15693_Retrieve_Data_Into_Answer_Functions 


/**  
* @brief  	The function is used in a Get System Info command to count the number field present and compute the number of bytes describing the memory of the contactless tag
* @param  	Response	:	the response provided by the contactless tag
* @param  	Information_flags	:	to determine if a field if present or not
* @retval		number_of bytes	:number of bytes describing the memory of the contactless tag
*/
static u8 ISO15693_Compute_Number_of_bytes_MemSize_using_GetSysInfo_Frame(const u8 Information_Flags,const u8 Response_data_length)
{
	u8 number_of_bytes=INIT_VALUE;
	u8 number_of_field=INIT_VALUE;
	u8 mask=0x01;// mask to hide other bits
	u8 counter=INIT_VALUE;
	u8 buffer;
	// compute number of field present
	while(counter!=BYTE_SIZE)
	{
		// hide others bits and shift bit to LSB
		if(((Information_Flags & mask)>>counter)==BIT_SET)
			number_of_field++;
		
		counter++;
		mask=mask<<SHIFT_BIT1;// shift for the next bit
	}
	// if this function is called Memsize is present so it must be decounted
	number_of_field--;
	//count number of field and number of fix field
	buffer=(number_of_field+ISO15693_LENGTH_FIX_PARAMETERS_GET_SYS_INFO_RESPONSE);
	// MemSize size is the difference between buffer and the size of the frame
	number_of_bytes=Response_data_length-buffer;
	
	return number_of_bytes;
}

	
	// End of ISO15693_Build_in_Command_Functions 
	
	// End of ISO15693_Support_Functions 
	
	// End of ISO15693_Tags 

	// End of Contactless_Tags 

